@@ -10,7 +10,6 @@
 #include <libtorrent/operations.hpp>
 #include <memory>
 #include "bytes.hpp"
-#include "gil.hpp"
 
 #include <boost/type_traits/is_polymorphic.hpp>
 
@@ -29,6 +28,7 @@ bytes get_buffer(read_piece_alert const& rpa)
        : bytes();
 }
 
+#if TORRENT_ABI_VERSION <= 2
 list stats_alert_transferred(stats_alert const& alert)
 {
    list result;
@@ -37,6 +37,7 @@ list stats_alert_transferred(stats_alert const& alert)
    }
    return result;
 }
+#endif
 
 list get_status_from_update_alert(state_update_alert const& alert)
 {
@@ -167,8 +168,7 @@ list dht_sample_infohashes_nodes(dht_sample_infohashes_alert const& alert)
 #if TORRENT_ABI_VERSION == 1
 entry const& get_resume_data_entry(save_resume_data_alert const& self)
 {
-    python_deprecated("resume_data is deprecated");
-    return *self.resume_data;
+	return *self.resume_data;
 }
 #endif
 
@@ -246,7 +246,9 @@ namespace boost
 	POLY(torrent_delete_failed_alert)
 	POLY(save_resume_data_failed_alert)
 	POLY(performance_alert)
+#if TORRENT_ABI_VERSION <= 2
 	POLY(stats_alert)
+#endif
 	POLY(cache_flushed_alert)
 	POLY(incoming_connection_alert)
 	POLY(torrent_need_cert_alert)
@@ -267,11 +269,11 @@ namespace boost
 	POLY(alerts_dropped_alert)
 	POLY(session_stats_alert)
 	POLY(socks5_alert)
+	POLY(file_prio_alert)
 
 #if TORRENT_ABI_VERSION == 1
 	POLY(anonymous_mode_alert)
 	POLY(torrent_added_alert)
-	POLY(torrent_update_alert)
 #endif
 
 #ifndef TORRENT_DISABLE_LOGGING
@@ -312,23 +314,9 @@ void bind_alert()
             .def("message", &alert::message)
             .def("what", &alert::what)
             .def("category", &alert::category)
-#if TORRENT_ABI_VERSION == 1
-            .def("severity", depr(&alert::severity))
-#endif
             .def("__str__", &alert::message)
             ;
 
-#if TORRENT_ABI_VERSION == 1
-        enum_<alert::severity_t>("severity_levels")
-            .value("debug", alert::debug)
-            .value("info", alert::info)
-            .value("warning", alert::warning)
-            .value("critical", alert::critical)
-            .value("fatal", alert::fatal)
-            .value("none", alert::none)
-            ;
-#endif
-
         scope s = class_<dummy3>("category_t");
         s.attr("error_notification") = alert::error_notification;
         s.attr("peer_notification") = alert::peer_notification;
@@ -344,7 +332,9 @@ void bind_alert()
         s.attr("ip_block_notification") = alert::ip_block_notification;
         s.attr("performance_warning") = alert::performance_warning;
         s.attr("dht_notification") = alert::dht_notification;
+#if TORRENT_ABI_VERSION <= 2
         s.attr("stats_notification") = alert::stats_notification;
+#endif
         s.attr("session_log_notification") = alert::session_log_notification;
         s.attr("torrent_log_notification") = alert::torrent_log_notification;
         s.attr("peer_log_notification") = alert::peer_log_notification;
@@ -458,7 +448,10 @@ void bind_alert()
 
     class_<torrent_removed_alert, bases<torrent_alert>, noncopyable>(
         "torrent_removed_alert", no_init)
+#if TORRENT_ABI_VERSION < 3
         .def_readonly("info_hash", &torrent_removed_alert::info_hash)
+#endif
+        .def_readonly("info_hashes", &torrent_removed_alert::info_hashes)
         ;
 
     class_<read_piece_alert, bases<torrent_alert>, noncopyable>(
@@ -487,6 +480,7 @@ void bind_alert()
         .def_readonly("status_code", &tracker_error_alert::status_code)
 #endif
         .def("error_message", &tracker_error_alert::error_message)
+        .def("failure_reason", &tracker_error_alert::failure_reason)
         .def_readonly("times_in_row", &tracker_error_alert::times_in_row)
         .def_readonly("error", &tracker_error_alert::error)
         ;
@@ -564,6 +558,7 @@ void bind_alert()
         .def_readonly("path", &storage_moved_alert::path)
 #endif
         .def("storage_path", &storage_moved_alert::storage_path)
+        .def("old_path", &storage_moved_alert::old_path)
         ;
 
     class_<storage_moved_failed_alert, bases<torrent_alert>, noncopyable>(
@@ -578,7 +573,10 @@ void bind_alert()
 
     class_<torrent_deleted_alert, bases<torrent_alert>, noncopyable>(
         "torrent_deleted_alert", no_init)
+#if TORRENT_ABI_VERSION < 3
         .def_readonly("info_hash", &torrent_deleted_alert::info_hash)
+#endif
+        .def_readonly("info_hashes", &torrent_deleted_alert::info_hashes)
     ;
 
     class_<torrent_paused_alert, bases<torrent_alert>, noncopyable>(
@@ -668,10 +666,16 @@ void bind_alert()
 
     enum_<socket_type_t>("socket_type_t")
        .value("tcp", socket_type_t::tcp)
-       .value("tcp_ssl", socket_type_t::tcp_ssl)
-       .value("udp", socket_type_t::udp)
-       .value("i2p", socket_type_t::i2p)
        .value("socks5", socket_type_t::socks5)
+       .value("http", socket_type_t::http)
+       .value("utp", socket_type_t::utp)
+#if TORRENT_ABI_VERSION <= 2
+       .value("udp", socket_type_t::udp)
+#endif
+       .value("i2p", socket_type_t::i2p)
+       .value("tcp_ssl", socket_type_t::tcp_ssl)
+       .value("socks5_ssl", socket_type_t::socks5_ssl)
+       .value("http_ssl", socket_type_t::http_ssl)
        .value("utp_ssl", socket_type_t::utp_ssl)
        ;
 
@@ -786,6 +790,7 @@ void bind_alert()
         .def_readonly("name", &file_renamed_alert::name)
 #endif
         .def("new_name", &file_renamed_alert::new_name)
+        .def("old_name", &file_renamed_alert::old_name)
         ;
 
     class_<file_rename_failed_alert, bases<torrent_alert>, noncopyable>(
@@ -879,7 +884,10 @@ void bind_alert()
         .def_readonly("msg", &torrent_delete_failed_alert::msg)
 #endif
         .def_readonly("error", &torrent_delete_failed_alert::error)
+#if TORRENT_ABI_VERSION < 3
         .def_readonly("info_hash", &torrent_delete_failed_alert::info_hash)
+#endif
+        .def_readonly("info_hashes", &torrent_delete_failed_alert::info_hashes)
         ;
 
     class_<save_resume_data_failed_alert, bases<torrent_alert>, noncopyable>(
@@ -909,6 +917,7 @@ void bind_alert()
         .value("too_few_file_descriptors", performance_alert::too_few_file_descriptors)
     ;
 
+#if TORRENT_ABI_VERSION <= 2
     class_<stats_alert, bases<torrent_alert>, noncopyable>(
         "stats_alert", no_init)
         .add_property("transferred", &stats_alert_transferred)
@@ -931,6 +940,7 @@ void bind_alert()
         .value("download_tracker_protocol", stats_alert::download_tracker_protocol)
 #endif
     ;
+#endif // TORRENT_ABI_VERSION
 
     class_<cache_flushed_alert, bases<torrent_alert>, noncopyable>(
         "cache_flushed_alert", no_init)
@@ -969,14 +979,6 @@ void bind_alert()
        .add_property("params", &add_torrent_alert::params)
        ;
 
-#if TORRENT_ABI_VERSION == 1
-    class_<torrent_update_alert, bases<torrent_alert>, noncopyable>(
-       "torrent_update_alert", no_init)
-        .def_readonly("old_ih", &torrent_update_alert::old_ih)
-        .def_readonly("new_ih", &torrent_update_alert::new_ih)
-        ;
-#endif
-
     class_<dht_outgoing_get_peers_alert, bases<alert>, noncopyable>(
        "dht_outgoing_get_peers_alert", no_init)
         .def_readonly("info_hash", &dht_outgoing_get_peers_alert::info_hash)
@@ -990,7 +992,7 @@ void bind_alert()
     class_<log_alert, bases<alert>, noncopyable>(
        "log_alert", no_init)
 #if TORRENT_ABI_VERSION == 1
-        .def("msg", depr(&log_alert::msg))
+        .def("msg", &log_alert::msg)
 #endif
         .def("log_message", &log_alert::log_message)
         ;
@@ -998,7 +1000,7 @@ void bind_alert()
     class_<torrent_log_alert, bases<torrent_alert>, noncopyable>(
        "torrent_log_alert", no_init)
 #if TORRENT_ABI_VERSION == 1
-        .def("msg", depr(&torrent_log_alert::msg))
+        .def("msg", &torrent_log_alert::msg)
 #endif
         .def("log_message", &torrent_log_alert::log_message)
         ;
@@ -1006,7 +1008,7 @@ void bind_alert()
     class_<peer_log_alert, bases<peer_alert>, noncopyable>(
        "peer_log_alert", no_init)
 #if TORRENT_ABI_VERSION == 1
-        .def("msg", depr(&peer_log_alert::msg))
+        .def("msg", &peer_log_alert::msg)
 #endif
         .def("log_message", &peer_log_alert::log_message)
         ;
@@ -1029,7 +1031,7 @@ void bind_alert()
         ;
 
     class_<dht_log_alert, bases<alert>, noncopyable>("dht_log_alert", no_init)
-        .add_property("module", make_getter(&dht_log_alert::module, by_value()))
+        .def_readonly("module", &dht_log_alert::module)
         .def("log_message", &dht_log_alert::log_message)
     ;
 
@@ -1040,14 +1042,14 @@ void bind_alert()
 
     class_<dht_immutable_item_alert, bases<alert>, noncopyable>(
        "dht_immutable_item_alert", no_init)
-        .add_property("target", make_getter(&dht_immutable_item_alert::target, by_value()))
+        .def_readonly("target", &dht_immutable_item_alert::target)
         .add_property("item", &dht_immutable_item)
         ;
 
     class_<dht_mutable_item_alert, bases<alert>, noncopyable>(
        "dht_mutable_item_alert", no_init)
-        .add_property("key", make_getter(&dht_mutable_item_alert::key, by_value()))
-        .add_property("signature", make_getter(&dht_mutable_item_alert::signature, by_value()))
+        .def_readonly("key", &dht_mutable_item_alert::key)
+        .def_readonly("signature", &dht_mutable_item_alert::signature)
         .def_readonly("seq", &dht_mutable_item_alert::seq)
         .def_readonly("salt", &dht_mutable_item_alert::salt)
         .add_property("item", &dht_mutable_item)
@@ -1056,9 +1058,9 @@ void bind_alert()
 
     class_<dht_put_alert, bases<alert>, noncopyable>(
        "dht_put_alert", no_init)
-        .add_property("target", make_getter(&dht_put_alert::target, by_value()))
-        .add_property("public_key", make_getter(&dht_put_alert::public_key, by_value()))
-        .add_property("signature", make_getter(&dht_put_alert::signature, by_value()))
+        .def_readonly("target", &dht_put_alert::target)
+        .def_readonly("public_key", &dht_put_alert::public_key)
+        .def_readonly("signature", &dht_put_alert::signature)
         .def_readonly("salt", &dht_put_alert::salt)
         .def_readonly("seq", &dht_put_alert::seq)
         .def_readonly("num_success", &dht_put_alert::num_success)
@@ -1085,7 +1087,7 @@ void bind_alert()
     class_<block_uploaded_alert, bases<peer_alert>, noncopyable>(
        "block_uploaded_alert", no_init)
         .add_property("block_index", &block_uploaded_alert::block_index)
-        .add_property("piece_index", make_getter(&block_uploaded_alert::piece_index, by_value()))
+        .add_property("piece_index", make_getter((&block_uploaded_alert::piece_index), by_value()))
         ;
 
     class_<alerts_dropped_alert, bases<alert>, noncopyable>(
@@ -1100,6 +1102,10 @@ void bind_alert()
         .add_property("ip", make_getter(&socks5_alert::ip, by_value()))
         ;
 
+    class_<file_prio_alert, bases<torrent_alert>, noncopyable>(
+       "file_prio_alert", no_init)
+        ;
+
     class_<dht_live_nodes_alert, bases<alert>, noncopyable>(
        "dht_live_nodes_alert", no_init)
         .add_property("node_id", &dht_live_nodes_alert::node_id)
