@@ -25,10 +25,12 @@ namespace
         c.set_hash(p, sha1_hash(b.arr));
     }
 
+#if TORRENT_ABI_VERSION < 3
     void set_file_hash(create_torrent& c, file_index_t f, bytes const& b)
     {
         c.set_file_hash(f, sha1_hash(b.arr));
     }
+#endif
 
 #ifndef BOOST_NO_EXCEPTIONS
     void set_piece_hashes_callback(create_torrent& c, std::string const& p
@@ -100,14 +102,6 @@ namespace
 
     FileIter end_files(file_storage const& self)
     { return FileIter(self, self.end_file()); }
-
-#ifdef TORRENT_WINDOWS
-    void add_file_wstring(file_storage& fs, std::wstring const& file, std::int64_t size
-       , file_flags_t const flags, std::time_t md, std::string link)
-    {
-       fs.add_file(file, size, flags, md, link);
-    }
-#endif
 #endif // TORRENT_ABI_VERSION
 
     void add_files_callback(file_storage& fs, std::string const& file
@@ -135,19 +129,13 @@ void bind_create_torrent()
 {
     void (file_storage::*set_name0)(std::string const&) = &file_storage::set_name;
     void (file_storage::*rename_file0)(file_index_t, std::string const&) = &file_storage::rename_file;
-#if TORRENT_ABI_VERSION == 1
-#ifdef TORRENT_WINDOWS
-    void (file_storage::*set_name1)(std::wstring const&) = &file_storage::set_name;
-    void (file_storage::*rename_file1)(file_index_t, std::wstring const&) = &file_storage::rename_file;
-#endif
-#endif
 
 #ifndef BOOST_NO_EXCEPTIONS
     void (*set_piece_hashes0)(create_torrent&, std::string const&) = &set_piece_hashes;
 #endif
     void (*add_files0)(file_storage&, std::string const&, create_flags_t) = add_files;
 
-    std::string const& (file_storage::*file_storage_symlink)(file_index_t) const = &file_storage::symlink;
+    std::string (file_storage::*file_storage_symlink)(file_index_t) const = &file_storage::symlink;
     sha1_hash (file_storage::*file_storage_hash)(file_index_t) const = &file_storage::hash;
     std::string (file_storage::*file_storage_file_path)(file_index_t, std::string const&) const = &file_storage::file_path;
     string_view (file_storage::*file_storage_file_name)(file_index_t) const = &file_storage::file_name;
@@ -170,12 +158,9 @@ void bind_create_torrent()
         .def("add_file", add_file_deprecated, arg("entry"))
         .def("__iter__", boost::python::range(&begin_files, &end_files))
         .def("__len__", &file_storage::num_files)
-#ifdef TORRENT_WINDOWS
-        .def("add_file", add_file_wstring, (arg("path"), arg("size"), arg("flags") = 0, arg("mtime") = 0, arg("linkpath") = ""))
-#endif
 #endif // TORRENT_ABI_VERSION
         .def("hash", file_storage_hash)
-        .def("symlink", file_storage_symlink, return_value_policy<copy_const_reference>())
+        .def("symlink", file_storage_symlink)
         .def("file_path", file_storage_file_path, (arg("idx"), arg("save_path") = ""))
         .def("file_name", file_storage_file_name)
         .def("file_size", file_storage_file_size)
@@ -190,12 +175,6 @@ void bind_create_torrent()
         .def("piece_size", &file_storage::piece_size)
         .def("set_name", set_name0)
         .def("rename_file", rename_file0)
-#if TORRENT_ABI_VERSION == 1
-#ifdef TORRENT_WINDOWS
-        .def("set_name", set_name1)
-        .def("rename_file", rename_file1)
-#endif
-#endif
         .def("name", &file_storage::name, return_value_policy<copy_const_reference>())
         ;
 
@@ -217,8 +196,8 @@ void bind_create_torrent()
     scope s = class_<create_torrent>("create_torrent", no_init)
         .def(init<file_storage&>())
         .def(init<torrent_info const&>(arg("ti")))
-        .def(init<file_storage&, int, int, create_flags_t>((arg("storage"), arg("piece_size") = 0
-            , arg("pad_file_limit") = -1, arg("flags") = lt::create_torrent::optimize_alignment)))
+        .def(init<file_storage&, int, create_flags_t>((arg("storage"), arg("piece_size") = 0
+            , arg("flags") = create_flags_t{})))
 
         .def("generate", &create_torrent::generate)
 
@@ -226,7 +205,9 @@ void bind_create_torrent()
         .def("set_comment", &create_torrent::set_comment)
         .def("set_creator", &create_torrent::set_creator)
         .def("set_hash", &set_hash)
+#if TORRENT_ABI_VERSION < 3
         .def("set_file_hash", &set_file_hash)
+#endif
         .def("add_url_seed", &create_torrent::add_url_seed)
         .def("add_http_seed", &create_torrent::add_http_seed)
         .def("add_node", &add_node)
@@ -242,8 +223,13 @@ void bind_create_torrent()
 
         ;
 
+#if TORRENT_ABI_VERSION <= 2
         s.attr("optimize_alignment") = create_torrent::optimize_alignment;
         s.attr("merkle") = create_torrent::merkle;
+#endif
+        s.attr("v2_only") = create_torrent::v2_only;
+        s.attr("v1_only") = create_torrent::v1_only;
+        s.attr("canonical_files") = create_torrent::canonical_files;
         s.attr("modification_time") = create_torrent::modification_time;
         s.attr("symlinks") = create_torrent::symlinks;
     }
@@ -253,8 +239,11 @@ void bind_create_torrent()
 #if TORRENT_ABI_VERSION == 1
         s.attr("optimize") = create_torrent::optimize;
 #endif
+#if TORRENT_ABI_VERSION <= 2
         s.attr("optimize_alignment") = create_torrent::optimize_alignment;
         s.attr("merkle") = create_torrent::merkle;
+#endif
+        s.attr("v2_only") = create_torrent::v2_only;
         s.attr("modification_time") = create_torrent::modification_time;
         s.attr("symlinks") = create_torrent::symlinks;
     }
