@@ -6,10 +6,9 @@
 #include <list>
 #include <string>
 #include <libtorrent/session.hpp>
-#include <libtorrent/storage.hpp>
+#include <libtorrent/session_params.hpp>
 #include <libtorrent/error_code.hpp>
 #include <libtorrent/ip_filter.hpp>
-#include <libtorrent/disk_io_thread.hpp>
 #include <libtorrent/extensions.hpp>
 #include <libtorrent/bdecode.hpp>
 #include <libtorrent/bencode.hpp>
@@ -51,6 +50,9 @@ namespace boost
 using namespace boost::python;
 using namespace lt;
 
+// defined in torrent_info.cpp
+load_torrent_limits dict_to_limits(dict limits);
+
 namespace
 {
 #if TORRENT_ABI_VERSION == 1
@@ -163,39 +165,44 @@ namespace
 		{
 			// deprecated settings are still here, they just have empty names
 			char const* name = name_for_setting(i);
-			if (name[0] != '\0' && sett.has_val(i)) ret[name] = sett.get_str(i);
+			if (name[0] != '\0') ret[name] = sett.get_str(i);
 		}
 
 		for (int i = settings_pack::int_type_base;
 			i < settings_pack::max_int_setting_internal; ++i)
 		{
 			char const* name = name_for_setting(i);
-			if (name[0] != '\0' && sett.has_val(i)) ret[name] = sett.get_int(i);
+			if (name[0] != '\0') ret[name] = sett.get_int(i);
 		}
 
 		for (int i = settings_pack::bool_type_base;
 			i < settings_pack::max_bool_setting_internal; ++i)
 		{
 			char const* name = name_for_setting(i);
-			if (name[0] != '\0' && sett.has_val(i)) ret[name] = sett.get_bool(i);
+			if (name[0] != '\0') ret[name] = sett.get_bool(i);
 		}
 		return ret;
 	}
 
-	std::shared_ptr<lt::session> make_session(boost::python::dict sett, session_flags_t flags)
+	std::shared_ptr<lt::session> make_session(boost::python::dict sett
+		, session_flags_t const flags)
 	{
 		settings_pack p;
 		make_settings_pack(p, sett);
+#if TORRENT_ABI_VERSION <= 2
 		if (flags & lt::session::add_default_plugins)
 		{
+#endif
 			session_params params(std::move(p));
 			return std::make_shared<lt::session>(std::move(params), flags);
+#if TORRENT_ABI_VERSION <= 2
 		}
 		else
 		{
 			session_params params(std::move(p), {});
 			return std::make_shared<lt::session>(std::move(params), flags);
 		}
+#endif
 	}
 
 	void session_apply_settings(lt::session& ses, dict const& sett_dict)
@@ -268,10 +275,27 @@ namespace
                     extract<torrent_info const&>(value));
                 continue;
             }
-            else if(key == "info_hash")
+#if TORRENT_ABI_VERSION < 3
+            else if (key == "info_hash")
             {
-                p.info_hash = sha1_hash(
-                bytes(extract<bytes>(value)).arr.data());
+                if (boost::python::len(value) == sha1_hash::size())
+                {
+                    p.info_hash = sha1_hash(bytes(extract<bytes>(value)).arr.data());
+                }
+            }
+#endif
+            else if (key == "info_hashes")
+            {
+                if (boost::python::len(value) == sha1_hash::size())
+                {
+                    p.info_hashes = info_hash_t(sha1_hash(
+                            bytes(extract<bytes>(value)).arr.data()));
+                }
+                else if (boost::python::len(value) == sha256_hash::size())
+                {
+                    p.info_hashes = info_hash_t(sha256_hash(
+                            bytes(extract<bytes>(value)).arr.data()));
+                }
                 continue;
             }
             else if(key == "name")
@@ -287,17 +311,10 @@ namespace
 #if TORRENT_ABI_VERSION == 1
             else if(key == "resume_data")
             {
-                python_deprecated("the resume_data member is deprecated");
                 std::string resume = extract<std::string>(value);
                 p.resume_data.assign(resume.begin(), resume.end());
                 continue;
             }
-            else if(key == "uuid")
-            {
-                python_deprecated("the uuid member is deprecated");
-                p.uuid = extract<std::string>(value);
-                continue;
-            }
 #endif
             else if(key == "storage_mode")
             {
@@ -349,7 +366,6 @@ namespace
 #if TORRENT_ABI_VERSION == 1
             else if(key == "url")
             {
-                python_deprecated("the url member is deprecated");
                 p.url = extract<std::string>(value);
                 continue;
             }
@@ -558,46 +574,9 @@ namespace
         return ret;
     }
 
-    cache_status get_cache_info1(lt::session& s, torrent_handle h, int flags)
-    {
-       cache_status ret;
-       s.get_cache_info(&ret, h, flags);
-       return ret;
-    }
-
-    list cached_piece_info_list(std::vector<cached_piece_info> const& v)
-    {
-       list pieces;
-       time_point now = clock_type::now();
-       for (std::vector<cached_piece_info>::const_iterator i = v.begin()
-          , end(v.end()); i != end; ++i)
-       {
-          dict d;
-          d["piece"] = i->piece;
-          d["last_use"] = total_milliseconds(now - i->last_use) / 1000.f;
-          d["next_to_hash"] = i->next_to_hash;
-          d["kind"] = static_cast<int>(i->kind);
-          pieces.append(d);
-       }
-       return pieces;
-    }
-
-    list cache_status_pieces(cache_status const& cs)
-    {
-        return cached_piece_info_list(cs.pieces);
-    }
-
 #if TORRENT_ABI_VERSION == 1
-    cache_status get_cache_status(lt::session& s)
-    {
-       cache_status ret;
-       s.get_cache_info(&ret);
-       return ret;
-    }
-
     dict get_utp_stats(session_status const& st)
     {
-        python_deprecated("session_status is deprecated");
         dict ret;
         ret["num_idle"] = st.utp_stats.num_idle;
         ret["num_syn_sent"] = st.utp_stats.num_syn_sent;
@@ -606,25 +585,15 @@ namespace
         ret["num_close_wait"] = st.utp_stats.num_close_wait;
         return ret;
     }
-
-    list get_cache_info2(lt::session& ses, sha1_hash ih)
-    {
-       std::vector<cached_piece_info> ret;
-
-       {
-          allow_threading_guard guard;
-          ses.get_cache_info(ih, ret);
-       }
-
-       return cached_piece_info_list(ret);
-    }
 #endif
 
     entry save_state(lt::session const& s, std::uint32_t const flags)
     {
-        allow_threading_guard guard;
         entry e;
+#if TORRENT_ABI_VERSION <= 2
+        allow_threading_guard guard;
         s.save_state(e, save_state_flags_t(flags));
+#endif
         return e;
     }
 
@@ -646,6 +615,7 @@ namespace
 
 	void load_state(lt::session& ses, entry const& st, std::uint32_t const flags)
 	{
+#if TORRENT_ABI_VERSION <= 2
 		allow_threading_guard guard;
 
 		std::vector<char> buf;
@@ -655,6 +625,7 @@ namespace
 		bdecode(&buf[0], &buf[0] + buf.size(), e, ec);
 		TORRENT_ASSERT(!ec);
 		ses.load_state(e, save_state_flags_t(flags));
+#endif
 	}
 
 	dict get_peer_class(lt::session& ses, lt::peer_class_t const pc)
@@ -756,23 +727,24 @@ namespace
         TORRENT_ASSERT(public_key.size() == 32);
         std::array<char, 32> key;
         std::copy(public_key.begin(), public_key.end(), key.begin());
-        ses.dht_put_item(key, [=]
+        ses.dht_put_item(key
+            , [pk=std::move(public_key), sk=std::move(private_key), d=std::move(data)]
             (entry& e, std::array<char, 64>& sig, std::int64_t& seq, std::string const& salt)
             {
-                put_string(e, sig, seq, salt, public_key, private_key, data);
+                put_string(e, sig, seq, salt, pk, sk, d);
             }
             , salt);
     }
 #endif
 
-    add_torrent_params read_resume_data_wrapper(bytes const& b)
+    add_torrent_params read_resume_data_wrapper0(bytes const& b)
     {
-        error_code ec;
-        add_torrent_params p = read_resume_data(b.arr, ec);
-#ifndef BOOST_NO_EXCEPTIONS
-        if (ec) throw system_error(ec);
-#endif
-        return p;
+        return read_resume_data(b.arr);
+    }
+
+    add_torrent_params read_resume_data_wrapper1(bytes const& b, dict cfg)
+    {
+        return read_resume_data(b.arr, dict_to_limits(cfg));
     }
 
 	 int find_metric_idx_wrap(char const* name)
@@ -861,7 +833,7 @@ void bind_session()
         .def_readonly("dht_node_cache", &session_status::dht_node_cache)
         .def_readonly("dht_torrents", &session_status::dht_torrents)
         .def_readonly("dht_global_nodes", &session_status::dht_global_nodes)
-        .add_property("active_requests", make_getter(&session_status::active_requests, return_value_policy<return_by_value>()))
+        .def_readonly("active_requests", &session_status::active_requests)
         .def_readonly("dht_total_allocations", &session_status::dht_total_allocations)
 #endif // TORRENT_DISABLE_DHT
         .add_property("utp_stats", &get_utp_stats)
@@ -895,7 +867,6 @@ void bind_session()
         .add_property("file_priorities", PROP(&add_torrent_params::file_priorities))
         .def_readwrite("trackerid", &add_torrent_params::trackerid)
         .add_property("flags", PROP(&add_torrent_params::flags))
-        .def_readwrite("info_hash", &add_torrent_params::info_hash)
         .def_readwrite("max_uploads", &add_torrent_params::max_uploads)
         .def_readwrite("max_connections", &add_torrent_params::max_connections)
         .def_readwrite("upload_limit", &add_torrent_params::upload_limit)
@@ -913,6 +884,10 @@ void bind_session()
         .def_readwrite("num_complete", &add_torrent_params::num_complete)
         .def_readwrite("num_incomplete", &add_torrent_params::num_incomplete)
         .def_readwrite("num_downloaded", &add_torrent_params::num_downloaded)
+#if TORRENT_ABI_VERSION < 3
+        .def_readwrite("info_hash", &add_torrent_params::info_hash)
+#endif
+        .def_readwrite("info_hashes", &add_torrent_params::info_hashes)
         .add_property("http_seeds", PROP(&add_torrent_params::http_seeds))
         .add_property("url_seeds", PROP(&add_torrent_params::url_seeds))
         .add_property("peers", PROP(&add_torrent_params::peers))
@@ -921,12 +896,13 @@ void bind_session()
         .add_property("have_pieces", PROP(&add_torrent_params::have_pieces))
         .add_property("verified_pieces", PROP(&add_torrent_params::verified_pieces))
         .add_property("piece_priorities", PROP(&add_torrent_params::piece_priorities))
+#if TORRENT_ABI_VERSION <= 2
         .add_property("merkle_tree", PROP(&add_torrent_params::merkle_tree))
+#endif
         .add_property("renamed_files", PROP(&add_torrent_params::renamed_files))
 
 #if TORRENT_ABI_VERSION == 1
         .def_readwrite("url", &add_torrent_params::url)
-        .def_readwrite("uuid", &add_torrent_params::uuid)
         .add_property("resume_data", PROP(&add_torrent_params::resume_data))
 #endif
       ;
@@ -943,8 +919,10 @@ void bind_session()
 
     {
         scope s = class_<dummy10>("session_flags_t");
-        s.attr("add_default_plugins") = lt::session::add_default_plugins;
         s.attr("paused") = lt::session::paused;
+#if TORRENT_ABI_VERSION <= 2
+        s.attr("add_default_plugins") = lt::session::add_default_plugins;
+#endif
 #if TORRENT_ABI_VERSION == 1
         s.attr("start_default_features") = lt::session::start_default_features;
 #endif
@@ -996,43 +974,6 @@ void bind_session()
     s.attr("default_flags") = add_torrent_params::flag_default_flags;
     }
 #endif
-
-    class_<cache_status>("cache_status")
-        .add_property("pieces", cache_status_pieces)
-#if TORRENT_ABI_VERSION == 1
-        .def_readonly("blocks_written", &cache_status::blocks_written)
-        .def_readonly("writes", &cache_status::writes)
-        .def_readonly("blocks_read", &cache_status::blocks_read)
-        .def_readonly("blocks_read_hit", &cache_status::blocks_read_hit)
-        .def_readonly("reads", &cache_status::reads)
-        .def_readonly("queued_bytes", &cache_status::queued_bytes)
-        .def_readonly("cache_size", &cache_status::cache_size)
-        .def_readonly("write_cache_size", &cache_status::write_cache_size)
-        .def_readonly("read_cache_size", &cache_status::read_cache_size)
-        .def_readonly("pinned_blocks", &cache_status::pinned_blocks)
-        .def_readonly("total_used_buffers", &cache_status::total_used_buffers)
-        .def_readonly("average_read_time", &cache_status::average_read_time)
-        .def_readonly("average_write_time", &cache_status::average_write_time)
-        .def_readonly("average_hash_time", &cache_status::average_hash_time)
-        .def_readonly("average_job_time", &cache_status::average_job_time)
-        .def_readonly("cumulative_job_time", &cache_status::cumulative_job_time)
-        .def_readonly("cumulative_read_time", &cache_status::cumulative_read_time)
-        .def_readonly("cumulative_write_time", &cache_status::cumulative_write_time)
-        .def_readonly("cumulative_hash_time", &cache_status::cumulative_hash_time)
-        .def_readonly("total_read_back", &cache_status::total_read_back)
-        .def_readonly("read_queue_size", &cache_status::read_queue_size)
-        .def_readonly("blocked_jobs", &cache_status::blocked_jobs)
-        .def_readonly("queued_jobs", &cache_status::queued_jobs)
-        .def_readonly("peak_queued", &cache_status::peak_queued)
-        .def_readonly("pending_jobs", &cache_status::pending_jobs)
-        .def_readonly("num_jobs", &cache_status::num_jobs)
-        .def_readonly("num_read_jobs", &cache_status::num_read_jobs)
-        .def_readonly("num_write_jobs", &cache_status::num_write_jobs)
-        .def_readonly("arc_mru_size", &cache_status::arc_mru_size)
-        .def_readonly("arc_mru_ghost_size", &cache_status::arc_mru_ghost_size)
-        .def_readonly("arc_mfu_size", &cache_status::arc_mfu_size)
-        .def_readonly("arc_mfu_ghost_size", &cache_status::arc_mfu_ghost_size)
-#endif
     ;
 
     enum_<lt::portmap_protocol>("portmap_protocol")
@@ -1072,11 +1013,14 @@ void bind_session()
 
     {
     scope s = class_<lt::session, boost::noncopyable>("session", no_init)
-        .def(init<>())
         .def("__init__", boost::python::make_constructor(&make_session
                 , default_call_policies()
                 , (arg("settings"), arg("flags")=
+#if TORRENT_ABI_VERSION <= 2
                     lt::session::add_default_plugins
+#else
+                    lt::session_flags_t{}
+#endif
                     ))
               )
 #if TORRENT_ABI_VERSION == 1
@@ -1086,7 +1030,7 @@ void bind_session()
                 , arg("flags")=lt::session::start_default_features | lt::session::add_default_plugins
                 , arg("alert_mask")=alert::error_notification))
         )
-        .def("outgoing_ports", depr(&outgoing_ports))
+        .def("outgoing_ports", &outgoing_ports)
 #endif
         .def("post_torrent_updates", allow_threads(&lt::session::post_torrent_updates), arg("flags") = 0xffffffff)
         .def("post_dht_stats", allow_threads(&lt::session::post_dht_stats))
@@ -1097,13 +1041,15 @@ void bind_session()
         .def("add_dht_node", &add_dht_node)
 #if TORRENT_ABI_VERSION == 1
         .def(
-            "add_dht_router", depr(&add_dht_router)
+            "add_dht_router", &add_dht_router
           , (arg("router"), "port")
         )
 #endif // TORRENT_ABI_VERSION
         .def("is_dht_running", allow_threads(&lt::session::is_dht_running))
+#if TORRENT_ABI_VERSION <= 2
         .def("set_dht_settings", allow_threads(&lt::session::set_dht_settings))
         .def("get_dht_settings", allow_threads(&lt::session::get_dht_settings))
+#endif
         .def("dht_get_immutable_item", allow_threads(dht_get_immutable_item))
         .def("dht_get_mutable_item", &dht_get_mutable_item)
         .def("dht_put_immutable_item", allow_threads(dht_put_immutable_item))
@@ -1120,7 +1066,7 @@ void bind_session()
 #ifndef BOOST_NO_EXCEPTIONS
 #if TORRENT_ABI_VERSION == 1
         .def(
-            "add_torrent", depr(&add_torrent_depr)
+            "add_torrent", &add_torrent_depr
           , (
                 arg("resume_data") = entry(),
                 arg("storage_mode") = storage_mode_sparse,
@@ -1131,14 +1077,14 @@ void bind_session()
 #endif // BOOST_NO_EXCEPTIONS
         .def("remove_torrent", allow_threads(&lt::session::remove_torrent), arg("option") = 0)
 #if TORRENT_ABI_VERSION == 1
-        .def("status", depr(&lt::session::status))
+        .def("status", allow_threads(&lt::session::status))
 #endif
         .def("get_settings", &session_get_settings)
         .def("apply_settings", &session_apply_settings)
 #if TORRENT_ABI_VERSION == 1
 #ifndef TORRENT_DISABLE_ENCRYPTION
-        .def("set_pe_settings", depr(&lt::session::set_pe_settings))
-        .def("get_pe_settings", depr(&lt::session::get_pe_settings))
+        .def("set_pe_settings", allow_threads(&lt::session::set_pe_settings))
+        .def("get_pe_settings", allow_threads(&lt::session::get_pe_settings))
 #endif
 #endif
         .def("load_state", &load_state, (arg("entry"), arg("flags") = 0xffffffff))
@@ -1150,8 +1096,8 @@ void bind_session()
         .def("add_extension", &add_extension)
 #if TORRENT_ABI_VERSION == 1
 #if TORRENT_USE_I2P
-        .def("set_i2p_proxy", depr(&lt::session::set_i2p_proxy))
-        .def("i2p_proxy", depr(&lt::session::i2p_proxy))
+        .def("set_i2p_proxy", allow_threads(&lt::session::set_i2p_proxy))
+        .def("i2p_proxy", allow_threads(&lt::session::i2p_proxy))
 #endif
 #endif
         .def("set_ip_filter", allow_threads(&lt::session::set_ip_filter))
@@ -1163,7 +1109,6 @@ void bind_session()
         .def("pause", allow_threads(&lt::session::pause))
         .def("resume", allow_threads(&lt::session::resume))
         .def("is_paused", allow_threads(&lt::session::is_paused))
-        .def("get_cache_info", &get_cache_info1, (arg("handle") = torrent_handle(), arg("flags") = 0))
         .def("add_port_mapping", allow_threads(&lt::session::add_port_mapping))
         .def("delete_port_mapping", allow_threads(&lt::session::delete_port_mapping))
         .def("reopen_network_sockets", allow_threads(&lt::session::reopen_network_sockets))
@@ -1175,52 +1120,49 @@ void bind_session()
         .def("set_peer_class", &set_peer_class)
 
 #if TORRENT_ABI_VERSION == 1
-        .def("id", depr(&lt::session::id))
+        .def("id", allow_threads(&lt::session::id))
         .def(
-            "listen_on", depr(&listen_on)
+            "listen_on", &listen_on
           , (arg("min"), "max", arg("interface") = (char const*)nullptr, arg("flags") = 0)
         )
 #ifndef TORRENT_DISABLE_DHT
-        .def("start_dht", depr(start_dht0))
-        .def("stop_dht", depr(&lt::session::stop_dht))
-        .def("start_dht", depr(start_dht1))
-        .def("dht_state", depr(&lt::session::dht_state))
-        .def("set_dht_proxy", depr(&lt::session::set_dht_proxy))
-        .def("dht_proxy", depr(&lt::session::dht_proxy))
+        .def("start_dht", allow_threads(start_dht0))
+        .def("stop_dht", allow_threads(&lt::session::stop_dht))
+        .def("start_dht", allow_threads(start_dht1))
+        .def("dht_state", allow_threads(&lt::session::dht_state))
+        .def("set_dht_proxy", allow_threads(&lt::session::set_dht_proxy))
+        .def("dht_proxy", allow_threads(&lt::session::dht_proxy))
 #endif
-        .def("set_local_download_rate_limit", depr(&lt::session::set_local_download_rate_limit))
-        .def("local_download_rate_limit", depr(&lt::session::local_download_rate_limit))
-        .def("set_local_upload_rate_limit", depr(&lt::session::set_local_upload_rate_limit))
-        .def("local_upload_rate_limit", depr(&lt::session::local_upload_rate_limit))
-        .def("set_download_rate_limit", depr(&lt::session::set_download_rate_limit))
-        .def("download_rate_limit", depr(&lt::session::download_rate_limit))
-        .def("set_upload_rate_limit", depr(&lt::session::set_upload_rate_limit))
-        .def("upload_rate_limit", depr(&lt::session::upload_rate_limit))
-        .def("set_max_uploads", depr(&lt::session::set_max_uploads))
-        .def("set_max_connections", depr(&lt::session::set_max_connections))
-        .def("max_connections", depr(&lt::session::max_connections))
-        .def("num_connections", depr(&lt::session::num_connections))
-        .def("set_max_half_open_connections", depr(&lt::session::set_max_half_open_connections))
-        .def("set_severity_level", depr(&lt::session::set_severity_level))
-        .def("set_alert_queue_size_limit", depr(&lt::session::set_alert_queue_size_limit))
-        .def("set_alert_mask", depr(&lt::session::set_alert_mask))
-        .def("set_peer_proxy", depr(&lt::session::set_peer_proxy))
-        .def("set_tracker_proxy", depr(&lt::session::set_tracker_proxy))
-        .def("set_web_seed_proxy", depr(&lt::session::set_web_seed_proxy))
-        .def("peer_proxy", depr(&lt::session::peer_proxy))
-        .def("tracker_proxy", depr(&lt::session::tracker_proxy))
-        .def("web_seed_proxy", depr(&lt::session::web_seed_proxy))
-        .def("set_proxy", depr(&lt::session::set_proxy))
-        .def("proxy", depr(&lt::session::proxy))
-        .def("start_upnp", depr(&start_upnp))
-        .def("stop_upnp", depr(&lt::session::stop_upnp))
-        .def("start_lsd", depr(&lt::session::start_lsd))
-        .def("stop_lsd", depr(&lt::session::stop_lsd))
-        .def("start_natpmp", depr(&start_natpmp))
-        .def("stop_natpmp", depr(&lt::session::stop_natpmp))
-        .def("get_cache_status", depr(&get_cache_status))
-        .def("get_cache_info", depr(&get_cache_info2))
-        .def("set_peer_id", depr(&lt::session::set_peer_id))
+        .def("set_local_download_rate_limit", allow_threads(&lt::session::set_local_download_rate_limit))
+        .def("local_download_rate_limit", allow_threads(&lt::session::local_download_rate_limit))
+        .def("set_local_upload_rate_limit", allow_threads(&lt::session::set_local_upload_rate_limit))
+        .def("local_upload_rate_limit", allow_threads(&lt::session::local_upload_rate_limit))
+        .def("set_download_rate_limit", allow_threads(&lt::session::set_download_rate_limit))
+        .def("download_rate_limit", allow_threads(&lt::session::download_rate_limit))
+        .def("set_upload_rate_limit", allow_threads(&lt::session::set_upload_rate_limit))
+        .def("upload_rate_limit", allow_threads(&lt::session::upload_rate_limit))
+        .def("set_max_uploads", allow_threads(&lt::session::set_max_uploads))
+        .def("set_max_connections", allow_threads(&lt::session::set_max_connections))
+        .def("max_connections", allow_threads(&lt::session::max_connections))
+        .def("num_connections", allow_threads(&lt::session::num_connections))
+        .def("set_max_half_open_connections", allow_threads(&lt::session::set_max_half_open_connections))
+        .def("set_alert_queue_size_limit", allow_threads(&lt::session::set_alert_queue_size_limit))
+        .def("set_alert_mask", allow_threads(&lt::session::set_alert_mask))
+        .def("set_peer_proxy", allow_threads(&lt::session::set_peer_proxy))
+        .def("set_tracker_proxy", allow_threads(&lt::session::set_tracker_proxy))
+        .def("set_web_seed_proxy", allow_threads(&lt::session::set_web_seed_proxy))
+        .def("peer_proxy", allow_threads(&lt::session::peer_proxy))
+        .def("tracker_proxy", allow_threads(&lt::session::tracker_proxy))
+        .def("web_seed_proxy", allow_threads(&lt::session::web_seed_proxy))
+        .def("set_proxy", allow_threads(&lt::session::set_proxy))
+        .def("proxy", allow_threads(&lt::session::proxy))
+        .def("start_upnp", &start_upnp)
+        .def("stop_upnp", allow_threads(&lt::session::stop_upnp))
+        .def("start_lsd", allow_threads(&lt::session::start_lsd))
+        .def("stop_lsd", allow_threads(&lt::session::stop_lsd))
+        .def("start_natpmp", &start_natpmp)
+        .def("stop_natpmp", allow_threads(&lt::session::stop_natpmp))
+        .def("set_peer_id", allow_threads(&lt::session::set_peer_id))
 #endif // TORRENT_ABI_VERSION
         ;
 
@@ -1248,7 +1190,9 @@ void bind_session()
     {
         scope s = class_<dummy9>("save_state_flags_t");
         s.attr("save_settings") = lt::session::save_settings;
+#if TORRENT_ABI_VERSION <= 2
         s.attr("save_dht_settings") = lt::session::save_dht_settings;
+#endif
         s.attr("save_dht_state") = lt::session::save_dht_state;
 #if TORRENT_ABI_VERSION == 1
         s.attr("save_encryption_settings") = lt::session:: save_encryption_settings;
@@ -1272,7 +1216,8 @@ void bind_session()
     def("high_performance_seed", high_performance_seed_wrapper);
     def("min_memory_usage", min_memory_usage_wrapper);
     def("default_settings", default_settings_wrapper);
-    def("read_resume_data", read_resume_data_wrapper);
+    def("read_resume_data", read_resume_data_wrapper0);
+    def("read_resume_data", read_resume_data_wrapper1);
     def("write_resume_data", write_resume_data);
     def("write_resume_data_buf", write_resume_data_buf_);
 
