@@ -12,7 +12,6 @@
 #include <libtorrent/entry.hpp>
 #include <libtorrent/peer_info.hpp>
 #include "libtorrent/announce_entry.hpp"
-#include <libtorrent/storage.hpp>
 #include <libtorrent/disk_interface.hpp>
 #include "gil.hpp"
 
@@ -88,7 +87,7 @@ namespace
 
 } // namespace unnamed
 
-list file_progress(torrent_handle& handle, int flags)
+list file_progress(torrent_handle& handle, file_progress_flags_t const flags)
 {
     std::vector<std::int64_t> p;
 
@@ -237,12 +236,21 @@ void add_tracker(torrent_handle& h, dict d)
 
 namespace
 {
+#if defined BOOST_ASIO_HAS_STD_CHRONO
    using std::chrono::system_clock;
+#else
+   using boost::chrono::system_clock;
+#endif
 
-   time_t to_ptime(time_point tpt)
+   object to_ptime(time_point tpt)
    {
-      return system_clock::to_time_t(system_clock::now()
-         + duration_cast<system_clock::duration>(tpt - clock_type::now()));
+      object ret;
+      if (tpt > min_time())
+      {
+         ret = long_(system_clock::to_time_t(system_clock::now()
+            + duration_cast<system_clock::duration>(tpt - clock_type::now())));
+      }
+      return ret;
    }
 }
 
@@ -264,30 +272,21 @@ list trackers(torrent_handle& h)
         if (!i->endpoints.empty())
         {
             announce_endpoint const& aep = i->endpoints.front();
-            d["message"] = aep.message;
+            announce_infohash const& aih = aep.info_hashes[protocol_version::V1];
+            d["message"] = aih.message;
             dict last_error;
-            last_error["value"] = aep.last_error.value();
-            last_error["category"] = aep.last_error.category().name();
+            last_error["value"] = aih.last_error.value();
+            last_error["category"] = aih.last_error.category().name();
             d["last_error"] = last_error;
-            if (aep.next_announce > min_time()) {
-                d["next_announce"] = to_ptime(aep.next_announce);
-            }
-            else {
-                d["next_announce"] = object();
-            }
-            if (aep.min_announce > min_time()) {
-                d["min_announce"] = to_ptime(aep.min_announce);
-            }
-            else {
-                d["min_announce"] = object();
-            }
-            d["scrape_incomplete"] = aep.scrape_incomplete;
-            d["scrape_complete"] = aep.scrape_complete;
-            d["scrape_downloaded"] = aep.scrape_downloaded;
-            d["fails"] = aep.fails;
-            d["updating"] = aep.updating;
-            d["start_sent"] = aep.start_sent;
-            d["complete_sent"] = aep.complete_sent;
+            d["next_announce"] = to_ptime(aih.next_announce);
+            d["min_announce"] = to_ptime(aih.min_announce);
+            d["scrape_incomplete"] = aih.scrape_incomplete;
+            d["scrape_complete"] = aih.scrape_complete;
+            d["scrape_downloaded"] = aih.scrape_downloaded;
+            d["fails"] = aih.fails;
+            d["updating"] = aih.updating;
+            d["start_sent"] = aih.start_sent;
+            d["complete_sent"] = aih.complete_sent;
         }
         else
         {
@@ -312,39 +311,55 @@ list trackers(torrent_handle& h)
         for (auto const& aep : i->endpoints)
         {
             dict e;
-            e["message"] = aep.message;
             e["local_address"] = boost::python::make_tuple(aep.local_endpoint.address().to_string(), aep.local_endpoint.port());
+
+            list aihs;
+            for (auto const& aih : aep.info_hashes)
+            {
+                dict i;
+                i["message"] = aih.message;
+                dict last_error;
+                last_error["value"] = aih.last_error.value();
+                last_error["category"] = aih.last_error.category().name();
+                i["last_error"] = last_error;
+                i["next_announce"] = to_ptime(aih.next_announce);
+                i["min_announce"] = to_ptime(aih.min_announce);
+                i["scrape_incomplete"] = aih.scrape_incomplete;
+                i["scrape_complete"] = aih.scrape_complete;
+                i["scrape_downloaded"] = aih.scrape_downloaded;
+                i["fails"] = aih.fails;
+                i["updating"] = aih.updating;
+                i["start_sent"] = aih.start_sent;
+                i["complete_sent"] = aih.complete_sent;
+                aihs.append(std::move(i));
+            }
+            e["info_hashes"] = std::move(aihs);
+
+#if TORRENT_ABI_VERSION <= 2
+            announce_infohash const& aih = aep.info_hashes[protocol_version::V1];
+            e["message"] = aih.message;
             dict last_error;
-            last_error["value"] = aep.last_error.value();
-            last_error["category"] = aep.last_error.category().name();
+            last_error["value"] = aih.last_error.value();
+            last_error["category"] = aih.last_error.category().name();
             e["last_error"] = last_error;
-            if (aep.next_announce > min_time()) {
-                e["next_announce"] = to_ptime(aep.next_announce);
-            }
-            else {
-                e["next_announce"] = object();
-            }
-            if (aep.min_announce > min_time()) {
-                e["min_announce"] = to_ptime(aep.min_announce);
-            }
-            else {
-                e["min_announce"] = object();
-            }
-            e["scrape_incomplete"] = aep.scrape_incomplete;
-            e["scrape_complete"] = aep.scrape_complete;
-            e["scrape_downloaded"] = aep.scrape_downloaded;
-            e["fails"] = aep.fails;
-            e["updating"] = aep.updating;
-            e["start_sent"] = aep.start_sent;
-            e["complete_sent"] = aep.complete_sent;
-            aeps.append(e);
+            e["next_announce"] = to_ptime(aih.next_announce);
+            e["min_announce"] = to_ptime(aih.min_announce);
+            e["scrape_incomplete"] = aih.scrape_incomplete;
+            e["scrape_complete"] = aih.scrape_complete;
+            e["scrape_downloaded"] = aih.scrape_downloaded;
+            e["fails"] = aih.fails;
+            e["updating"] = aih.updating;
+            e["start_sent"] = aih.start_sent;
+            e["complete_sent"] = aih.complete_sent;
+#endif
+            aeps.append(std::move(e));
         }
-        d["endpoints"] = aeps;
+        d["endpoints"] = std::move(aeps);
 
 #if TORRENT_ABI_VERSION == 1
         d["send_stats"] = i->send_stats;
 #endif
-        ret.append(d);
+        ret.append(std::move(d));
     }
     return ret;
 }
@@ -357,7 +372,7 @@ list get_download_queue(torrent_handle& handle)
 
     {
         allow_threading_guard guard;
-        handle.get_download_queue(downloading);
+        downloading = handle.get_download_queue();
     }
 
     for (std::vector<partial_piece_info>::iterator i = downloading.begin()
@@ -421,6 +436,7 @@ class dummy6 {};
 class dummy7 {};
 class dummy8 {};
 class dummy15 {};
+class dummy16 {};
 
 using by_value = return_value_policy<return_by_value>;
 void bind_torrent_handle()
@@ -449,8 +465,6 @@ void bind_torrent_handle()
         .value("always_replace_files", move_flags_t::always_replace_files)
         .value("fail_if_exist", move_flags_t::fail_if_exist)
         .value("dont_replace", move_flags_t::dont_replace)
-        .value("reset_save_path", move_flags_t::reset_save_path)
-        .value("reset_save_path_unchecked", move_flags_t::reset_save_path_unchecked)
     ;
 
 #if TORRENT_ABI_VERSION == 1
@@ -470,7 +484,7 @@ void bind_torrent_handle()
         .def("get_peer_info", get_peer_info)
         .def("status", _(&torrent_handle::status), arg("flags") = 0xffffffff)
         .def("get_download_queue", get_download_queue)
-        .def("file_progress", file_progress, arg("flags") = 0)
+        .def("file_progress", file_progress, arg("flags") = file_progress_flags_t{})
         .def("trackers", trackers)
         .def("replace_trackers", replace_trackers)
         .def("add_tracker", add_tracker)
@@ -530,6 +544,7 @@ void bind_torrent_handle()
         .def("max_connections", _(&torrent_handle::max_connections))
         .def("move_storage", _(move_storage0), (arg("path"), arg("flags") = move_flags_t::always_replace_files))
         .def("info_hash", _(&torrent_handle::info_hash))
+        .def("info_hashes", _(&torrent_handle::info_hashes))
         .def("force_recheck", _(&torrent_handle::force_recheck))
         .def("rename_file", _(rename_file0))
         .def("set_ssl_certificate", &torrent_handle::set_ssl_certificate, (arg("cert"), arg("private_key"), arg("dh_params"), arg("passphrase")=""))
@@ -541,35 +556,35 @@ void bind_torrent_handle()
 #if TORRENT_ABI_VERSION == 1
         .def("piece_priorities", &piece_priorities)
         .def("file_priorities", &file_priorities)
-        .def("stop_when_ready", depr(&torrent_handle::stop_when_ready))
-        .def("super_seeding", depr(super_seeding1))
-        .def("auto_managed", depr(&torrent_handle::auto_managed))
-        .def("set_priority", depr(&torrent_handle::set_priority))
-        .def("get_torrent_info", depr(&get_torrent_info))
-        .def("super_seeding", depr(super_seeding0))
-        .def("write_resume_data", depr(&torrent_handle::write_resume_data))
-        .def("is_seed", depr(&torrent_handle::is_seed))
-        .def("is_finished", depr(&torrent_handle::is_finished))
-        .def("has_metadata", depr(&torrent_handle::has_metadata))
-        .def("use_interface", depr(&torrent_handle::use_interface))
-        .def("name", depr(&torrent_handle::name))
-        .def("is_paused", depr(&torrent_handle::is_paused))
-        .def("is_auto_managed", depr(&torrent_handle::is_auto_managed))
-        .def("set_upload_mode", depr(&torrent_handle::set_upload_mode))
-        .def("set_share_mode", depr(&torrent_handle::set_share_mode))
-        .def("apply_ip_filter", depr(&torrent_handle::apply_ip_filter))
-        .def("set_sequential_download", depr(&torrent_handle::set_sequential_download))
-        .def("set_peer_upload_limit", depr(&torrent_handle::set_peer_upload_limit))
-        .def("set_peer_download_limit", depr(&torrent_handle::set_peer_download_limit))
-        .def("set_ratio", depr(&torrent_handle::set_ratio))
-        .def("save_path", depr(&torrent_handle::save_path))
-        .def("set_tracker_login", depr(&torrent_handle::set_tracker_login))
+        .def("stop_when_ready", _(&torrent_handle::stop_when_ready))
+        .def("super_seeding", super_seeding1)
+        .def("auto_managed", _(&torrent_handle::auto_managed))
+        .def("set_priority", _(&torrent_handle::set_priority))
+        .def("get_torrent_info", &get_torrent_info)
+        .def("super_seeding", super_seeding0)
+        .def("write_resume_data", _(&torrent_handle::write_resume_data))
+        .def("is_seed", _(&torrent_handle::is_seed))
+        .def("is_finished", _(&torrent_handle::is_finished))
+        .def("has_metadata", _(&torrent_handle::has_metadata))
+        .def("use_interface", &torrent_handle::use_interface)
+        .def("name", _(&torrent_handle::name))
+        .def("is_paused", _(&torrent_handle::is_paused))
+        .def("is_auto_managed", _(&torrent_handle::is_auto_managed))
+        .def("set_upload_mode", _(&torrent_handle::set_upload_mode))
+        .def("set_share_mode", _(&torrent_handle::set_share_mode))
+        .def("apply_ip_filter", &torrent_handle::apply_ip_filter)
+        .def("set_sequential_download", _(&torrent_handle::set_sequential_download))
+        .def("set_peer_upload_limit", &torrent_handle::set_peer_upload_limit)
+        .def("set_peer_download_limit", &torrent_handle::set_peer_download_limit)
+        .def("set_ratio", _(&torrent_handle::set_ratio))
+        .def("save_path", _(&torrent_handle::save_path))
+        .def("set_tracker_login", &torrent_handle::set_tracker_login)
 #endif
         ;
 
     s.attr("ignore_min_interval") = torrent_handle::ignore_min_interval;
     s.attr("overwrite_existing") = torrent_handle::overwrite_existing;
-    s.attr("piece_granularity") = int(torrent_handle::piece_granularity);
+    s.attr("piece_granularity") = torrent_handle::piece_granularity;
     s.attr("graceful_pause") = torrent_handle::graceful_pause;
     s.attr("flush_disk_cache") = torrent_handle::flush_disk_cache;
     s.attr("save_info_dict") = torrent_handle::save_info_dict;
@@ -602,9 +617,10 @@ void bind_torrent_handle()
 #endif
     }
 
-    enum_<torrent_handle::file_progress_flags_t>("file_progress_flags")
-        .value("piece_granularity", torrent_handle::piece_granularity)
-    ;
+    {
+    scope s = class_<dummy16>("file_progress_flags_t");
+    s.attr("piece_granularity") = torrent_handle::piece_granularity;
+    }
 
     {
     scope s = class_<dummy6>("add_piece_flags_t");
