@@ -1,6 +1,16 @@
 /*
 
-Copyright (c) 2003-2018, Arvid Norberg
+Copyright (c) 2003-2020, Arvid Norberg
+Copyright (c) 2003, Daniel Wallin
+Copyright (c) 2004, Magnus Jonsson
+Copyright (c) 2016-2020, Alden Torres
+Copyright (c) 2017, Falcosc
+Copyright (c) 2017, Pavel Pimenov
+Copyright (c) 2017, AllSeeingEyeTolledEweSew
+Copyright (c) 2018, d-komarov
+Copyright (c) 2018-2019, Steven Siloti
+Copyright (c) 2019, ghbplayer
+Copyright (c) 2020, Paul-Louis Ageneau
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -41,6 +51,10 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <limits> // for numeric_limits
 #include <memory> // for unique_ptr
 
+#include "libtorrent/aux_/disable_warnings_push.hpp"
+#include <boost/logic/tribool.hpp>
+#include "libtorrent/aux_/disable_warnings_pop.hpp"
+
 #include "libtorrent/fwd.hpp"
 #include "libtorrent/optional.hpp"
 #include "libtorrent/torrent_handle.hpp"
@@ -53,9 +67,10 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/stat.hpp"
 #include "libtorrent/alert.hpp"
 #include "libtorrent/piece_picker.hpp"
+#include "libtorrent/hash_picker.hpp"
 #include "libtorrent/config.hpp"
-#include "libtorrent/bandwidth_limit.hpp"
-#include "libtorrent/bandwidth_queue_entry.hpp"
+#include "libtorrent/aux_/bandwidth_limit.hpp"
+#include "libtorrent/aux_/bandwidth_queue_entry.hpp"
 #include "libtorrent/storage_defs.hpp"
 #include "libtorrent/assert.hpp"
 #include "libtorrent/aux_/session_interface.hpp"
@@ -64,7 +79,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/peer_class_set.hpp"
 #include "libtorrent/link.hpp"
 #include "libtorrent/vector_utils.hpp"
-#include "libtorrent/linked_list.hpp"
 #include "libtorrent/debug.hpp"
 #include "libtorrent/piece_block.hpp"
 #include "libtorrent/disk_interface.hpp"
@@ -74,9 +88,11 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/aux_/vector.hpp"
 #include "libtorrent/aux_/deferred_handler.hpp"
 #include "libtorrent/aux_/allocating_handler.hpp"
+#include "libtorrent/aux_/announce_entry.hpp"
 #include "libtorrent/extensions.hpp" // for add_peer_flags_t
+#include "libtorrent/ssl.hpp"
 
-#ifdef TORRENT_USE_OPENSSL
+#ifdef TORRENT_SSL_PEERS
 // there is no forward declaration header for asio
 namespace boost {
 namespace asio {
@@ -101,7 +117,7 @@ namespace libtorrent {
 
 	class http_parser;
 	struct tracker_request;
-	class bt_peer_connection;
+	struct bt_peer_connection;
 
 	using web_seed_flag_t = flags::bitfield_flag<std::uint8_t, struct web_seed_flag_tag>;
 
@@ -186,6 +202,11 @@ namespace libtorrent {
 		// saved in the resume data
 		bool ephemeral = false;
 
+		// this is set to true when this web seed was created from a redirect
+		// from a global IP, and SSRF mitigation is enabled. It prevents this
+		// web seed from resolving to any local network IPs.
+		bool no_local_ips = false;
+
 		// if the web server doesn't support keepalive or a block request was
 		// interrupted, the block received so far is kept here for the next
 		// connection to pick up
@@ -216,6 +237,7 @@ namespace libtorrent {
 			resolving = std::move(rhs.resolving);
 			removed = std::move(rhs.removed);
 			ephemeral = std::move(rhs.ephemeral);
+			no_local_ips = std::move(rhs.no_local_ips);
 			restart_request = std::move(rhs.restart_request);
 			restart_piece = std::move(rhs.restart_piece);
 			redirects = std::move(rhs.redirects);
@@ -243,6 +265,8 @@ namespace libtorrent {
 		// the case there is no piece picker, see m_have_all.
 		std::unique_ptr<piece_picker> m_picker;
 
+		std::unique_ptr<hash_picker> m_hash_picker;
+
 		// TODO: make this a raw pointer. perhaps keep the shared_ptr
 		// around further down the object to maintain an owner
 		std::shared_ptr<torrent_info> m_torrent_file;
@@ -321,22 +345,21 @@ namespace libtorrent {
 	// a torrent is a class that holds information
 	// for a specific download. It updates itself against
 	// the tracker
-	class TORRENT_EXTRA_EXPORT torrent
+	struct TORRENT_EXTRA_EXPORT torrent
 		: private single_threaded
 		, private torrent_hot_members
-		, public request_callback
-		, public peer_class_set
-		, public aux::error_handler_interface
-		, public std::enable_shared_from_this<torrent>
+		, request_callback
+		, peer_class_set
+		, std::enable_shared_from_this<torrent>
 	{
-	public:
-
-		torrent(aux::session_interface& ses
-			, bool session_paused, add_torrent_params const& p);
+		// add_torrent_params may contain large merkle trees that are best
+		// moved. Deleting the const& overload ensures that it's always moved in.
+		torrent(aux::session_interface& ses, bool session_paused, add_torrent_params&& p);
+		torrent(aux::session_interface&, bool, add_torrent_params const& p) = delete;
 		~torrent() override;
 
 		// This may be called from multiple threads
-		sha1_hash const& info_hash() const { return m_info_hash; }
+		info_hash_t const& info_hash() const { return m_info_hash; }
 
 		bool is_deleted() const { return m_deleted; }
 
@@ -359,11 +382,6 @@ namespace libtorrent {
 			update_gauge();
 		}
 
-#if TORRENT_ABI_VERSION == 1
-		// deprecated in 1.2
-		void start_download_url();
-#endif
-
 		// returns which stats gauge this torrent currently
 		// has incremented.
 		int current_stats_state() const;
@@ -371,8 +389,8 @@ namespace libtorrent {
 #ifndef TORRENT_DISABLE_EXTENSIONS
 		void add_extension(std::shared_ptr<torrent_plugin>);
 		void remove_extension(std::shared_ptr<torrent_plugin>);
-		void add_extension_fun(std::function<std::shared_ptr<torrent_plugin>(torrent_handle const&, void*)> const& ext
-			, void* userdata);
+		void add_extension_fun(std::function<std::shared_ptr<torrent_plugin>(torrent_handle const&, client_data_t)> const& ext
+			, client_data_t userdata);
 		void notify_extension_add_peer(tcp::endpoint const& ip
 			, peer_source_flags_t src, add_peer_flags_t flags);
 #endif
@@ -389,6 +407,9 @@ namespace libtorrent {
 		// it will initialize the storage and the piece-picker
 		void init();
 
+		void load_merkle_trees(aux::vector<std::vector<sha256_hash>, file_index_t> t
+			, aux::vector<std::vector<bool>, file_index_t> mask);
+
 		// find the peer that introduced us to the given endpoint. This is
 		// used when trying to holepunch. We need the introducer so that we
 		// can send a rendezvous connect message
@@ -405,7 +426,8 @@ namespace libtorrent {
 
 		void on_resume_data_checked(status_t status, storage_error const& error);
 		void on_force_recheck(status_t status, storage_error const& error);
-		void on_piece_hashed(piece_index_t piece, sha1_hash const& piece_hash
+		void on_piece_hashed(aux::vector<sha256_hash> block_hashes
+			, piece_index_t piece, sha1_hash const& piece_hash
 			, storage_error const& error);
 		void files_checked();
 		void start_checking();
@@ -435,7 +457,6 @@ namespace libtorrent {
 		int seed_rank(aux::session_settings const& s) const;
 
 		void add_piece(piece_index_t piece, char const* data, add_piece_flags_t flags);
-		void add_piece_async(piece_index_t piece, std::vector<char> data, add_piece_flags_t flags);
 		void on_disk_write_complete(storage_error const& error
 			, peer_request const& p);
 
@@ -447,9 +468,9 @@ namespace libtorrent {
 			bool fail;
 			error_code error;
 		};
-		void read_piece(piece_index_t piece);
-		void on_disk_read_complete(disk_buffer_holder block, disk_job_flags_t, storage_error const& se
-			, peer_request const& r, std::shared_ptr<read_piece_struct> rp);
+		void read_piece(piece_index_t);
+		void on_disk_read_complete(disk_buffer_holder, storage_error const&
+			, peer_request const&, std::shared_ptr<read_piece_struct>);
 
 		storage_mode_t storage_mode() const;
 
@@ -527,7 +548,7 @@ namespace libtorrent {
 		void resume();
 
 		void set_session_paused(bool b);
-		void set_paused(bool b, pause_flags_t flags = {});
+		void set_paused(bool b, pause_flags_t flags = torrent_handle::clear_disk_cache);
 		void set_announce_to_dht(bool b) { m_announce_to_dht = b; }
 		void set_announce_to_trackers(bool b) { m_announce_to_trackers = b; }
 		void set_announce_to_lsd(bool b) { m_announce_to_lsd = b; }
@@ -536,7 +557,7 @@ namespace libtorrent {
 
 		time_point32 started() const { return m_started; }
 		void step_session_time(int seconds);
-		void do_pause(bool was_paused = false);
+		void do_pause(pause_flags_t flags = torrent_handle::clear_disk_cache);
 		void do_resume();
 
 		seconds32 finished_time() const;
@@ -553,9 +574,7 @@ namespace libtorrent {
 
 		void set_need_save_resume()
 		{
-			if (m_need_save_resume_data) return;
 			m_need_save_resume_data = true;
-			state_updated();
 		}
 
 		bool is_auto_managed() const { return m_auto_managed; }
@@ -604,14 +623,14 @@ namespace libtorrent {
 		// it, add it to the m_state_updates list in session_impl
 		void state_updated();
 
-		void file_progress(aux::vector<std::int64_t, file_index_t>& fp, int flags = 0);
+		void file_progress(aux::vector<std::int64_t, file_index_t>& fp, file_progress_flags_t flags);
 
 #if TORRENT_ABI_VERSION == 1
 		void use_interface(std::string net_interface);
 #endif
 
-		void connect_to_url_seed(std::list<web_seed_t>::iterator url);
-		bool connect_to_peer(torrent_peer* peerinfo, bool ignore_limit = false);
+		void connect_to_url_seed(std::list<web_seed_t>::iterator);
+		bool connect_to_peer(torrent_peer*, bool ignore_limit = false);
 
 		int priority() const;
 #if TORRENT_ABI_VERSION == 1
@@ -637,6 +656,7 @@ namespace libtorrent {
 		// PEER MANAGEMENT
 
 		static constexpr web_seed_flag_t ephemeral = 0_bit;
+		static constexpr web_seed_flag_t no_local_ips = 1_bit;
 
 		// add_web_seed won't add duplicates. If we have already added an entry
 		// with this URL, we'll get back the existing entry
@@ -735,10 +755,10 @@ namespace libtorrent {
 		void tracker_response(
 			tracker_request const& r
 			, address const& tracker_ip
-			, std::list<address> const& ip_list
+			, std::list<address> const& tracker_ips
 			, struct tracker_response const& resp) override;
 		void tracker_request_error(tracker_request const& r
-			, error_code const& ec, const std::string& msg
+			, error_code const& ec, operation_t op, const std::string& msg
 			, seconds32 retry_interval) override;
 		void tracker_warning(tracker_request const& req
 			, std::string const& msg) override;
@@ -767,8 +787,7 @@ namespace libtorrent {
 		// forcefully sets next_announce to the current time
 		void force_tracker_request(time_point, int tracker_idx, reannounce_flags_t flags);
 		void scrape_tracker(int idx, bool user_triggered);
-		void announce_with_tracker(std::uint8_t e
-			= tracker_request::none);
+		void announce_with_tracker(event_t = event_t::none);
 
 #ifndef TORRENT_DISABLE_DHT
 		void dht_announce();
@@ -780,7 +799,7 @@ namespace libtorrent {
 		void set_tracker_login(std::string const& name, std::string const& pw);
 #endif
 
-		announce_entry* find_tracker(std::string const& url);
+		aux::announce_entry* find_tracker(std::string const& url);
 // --------------------------------------------
 		// PIECE MANAGEMENT
 
@@ -841,6 +860,10 @@ namespace libtorrent {
 		// only once per piece
 		void we_have(piece_index_t index);
 
+		// process the v2 block hashes for a piece
+		boost::tribool on_blocks_hashed(piece_index_t piece
+			, span<sha256_hash const> block_hashes);
+
 	public:
 
 		int num_have() const
@@ -902,16 +925,17 @@ namespace libtorrent {
 
 		// this is the asio callback that is called when a name
 		// lookup for a PEER is completed.
-		void on_peer_name_lookup(error_code const& e
-			, std::vector<address> const& addrs
-			, int port);
+		void on_peer_name_lookup(error_code const&
+			, std::vector<address> const&
+			, int port
+			, protocol_version);
 
 		// this is the asio callback that is called when a name
 		// lookup for a WEB SEED is completed.
-		void on_name_lookup(error_code const& e
-			, std::vector<address> const& addrs
+		void on_name_lookup(error_code const&
+			, std::vector<address> const&
 			, int port
-			, std::list<web_seed_t>::iterator web);
+			, std::list<web_seed_t>::iterator);
 
 		void connect_web_seed(std::list<web_seed_t>::iterator web, tcp::endpoint a);
 
@@ -945,7 +969,8 @@ namespace libtorrent {
 		void resume_download();
 
 		void verify_piece(piece_index_t piece);
-		void on_piece_verified(piece_index_t piece
+		void on_piece_verified(aux::vector<sha256_hash> block_hashes
+			, piece_index_t piece
 			, sha1_hash const& piece_hash, storage_error const& error);
 
 		// this is called whenever a peer in this swarm becomes interesting
@@ -959,11 +984,14 @@ namespace libtorrent {
 		void piece_passed(piece_index_t index);
 
 		// piece_failed is called when a piece fails the hash check
-		void piece_failed(piece_index_t index);
+		// for failures detected with v2 hashes the failing blocks(s)
+		// are specified in blocks
+		// *blocks must be sorted in acending order*
+		void piece_failed(piece_index_t index, std::vector<int> blocks = std::vector<int>());
 
 		// this is the handler for hash failure piece synchronization
 		// i.e. resetting the piece
-		void on_piece_sync(piece_index_t piece);
+		void on_piece_sync(piece_index_t piece, std::vector<int> const& blocks);
 
 		// this is the handler for write failure piece synchronization
 		void on_piece_fail_sync(piece_index_t piece, piece_block b);
@@ -981,7 +1009,7 @@ namespace libtorrent {
 		bool is_inactive() const;
 
 		std::string save_path() const;
-		alert_manager& alerts() const;
+		aux::alert_manager& alerts() const;
 		piece_picker& picker()
 		{
 			TORRENT_ASSERT(m_picker.get());
@@ -998,6 +1026,23 @@ namespace libtorrent {
 			return m_picker.get() != nullptr;
 		}
 
+		hash_picker& get_hash_picker()
+		{
+			TORRENT_ASSERT(m_hash_picker.get());
+			return *m_hash_picker;
+		}
+		hash_picker const& get_hash_picker() const
+		{
+			TORRENT_ASSERT(m_hash_picker.get());
+			return *m_hash_picker;
+		}
+
+		void need_hash_picker(aux::vector<std::vector<bool>, file_index_t> verified = {});
+		bool has_hash_picker() const
+		{
+			return m_hash_picker.get() != nullptr;
+		}
+
 		void update_max_failcount()
 		{
 			if (!m_peer_list) return;
@@ -1011,25 +1056,23 @@ namespace libtorrent {
 
 		bool has_storage() const { return bool(m_storage); }
 		storage_index_t storage() const { return m_storage; }
-		storage_interface* get_storage_impl() const;
 
 		torrent_info const& torrent_file() const
 		{ return *m_torrent_file; }
 
-		std::shared_ptr<const torrent_info> get_torrent_copy();
+		hash_request pick_hashes(peer_connection* peer);
+		std::vector<sha256_hash> get_hashes(hash_request const& req) const;
+		bool add_hashes(hash_request const& req, span<sha256_hash> hashes);
+		void hashes_rejected(hash_request const& req);
+		void verify_block_hashes(piece_index_t index);
 
-#if TORRENT_ABI_VERSION == 1
-		// deprecated in 1.2
-		std::string const& uuid() const { return m_uuid; }
-		void set_uuid(std::string const& s) { m_uuid = s; }
-		std::string const& url() const { return m_url; }
-		void set_url(std::string const& s) { m_url = s; }
-		std::string const& source_feed_url() const { return m_source_feed_url; }
-		void set_source_feed_url(std::string const& s) { m_source_feed_url = s; }
-#endif
+		std::shared_ptr<const torrent_info> get_torrent_file() const;
 
-		std::vector<announce_entry> const& trackers() const
-		{ return m_trackers; }
+		std::shared_ptr<torrent_info> get_torrent_copy_with_hashes() const;
+
+		std::vector<std::vector<sha256_hash>> get_piece_layers() const;
+
+		std::vector<announce_entry> trackers() const;
 
 		// this sets all the "enabled" states on all trackers, giving them
 		// all one more chance of being tried
@@ -1087,6 +1130,8 @@ namespace libtorrent {
 		bool are_files_checked() const
 		{ return m_files_checked; }
 
+		void initialize_merkle_trees();
+
 		// parses the info section from the given
 		// bencoded tree and moves the torrent
 		// to the checker thread for initial checking
@@ -1094,11 +1139,6 @@ namespace libtorrent {
 		// a return value of false indicates an error
 		bool set_metadata(span<char const> metadata);
 
-#if TORRENT_ABI_VERSION == 1
-		void on_torrent_download(error_code const& ec, http_parser const& parser
-			, span<char const> data);
-#endif
-
 		queue_position_t sequence_number() const { return m_sequence_number; }
 
 		bool seed_mode() const { return m_seed_mode; }
@@ -1120,8 +1160,6 @@ namespace libtorrent {
 		{ return m_verified.get_bit(piece); }
 		void verified(piece_index_t piece);
 
-		bool add_merkle_nodes(std::map<int, sha1_hash> const& n, piece_index_t piece);
-
 		// this is called once periodically for torrents
 		// that are not private
 		void lsd_announce();
@@ -1164,7 +1202,7 @@ namespace libtorrent {
 		}
 
 		bool is_ssl_torrent() const { return m_ssl_torrent; }
-#ifdef TORRENT_USE_OPENSSL
+#ifdef TORRENT_SSL_PEERS
 		void set_ssl_cert(std::string const& certificate
 			, std::string const& private_key
 			, std::string const& dh_params
@@ -1172,7 +1210,7 @@ namespace libtorrent {
 		void set_ssl_cert_buffer(std::string const& certificate
 			, std::string const& private_key
 			, std::string const& dh_params);
-		boost::asio::ssl::context* ssl_ctx() const { return m_ssl_ctx.get(); }
+		ssl::context* ssl_ctx() const { return m_ssl_ctx.get(); }
 #endif
 
 		int num_time_critical_pieces() const
@@ -1192,15 +1230,14 @@ namespace libtorrent {
 		}
 		void add_suggest_piece(piece_index_t index);
 
-		void* get_userdata() const { return m_userdata; }
+		client_data_t get_userdata() const { return m_userdata; }
 
 		static constexpr int no_gauge_state = 0xf;
 
 	private:
 
-		void update_tracker_endpoints();
-		void on_exception(std::exception const& e) override;
-		void on_error(error_code const& ec) override;
+		void on_exception(std::exception const& e);
+		void on_error(error_code const& ec);
 
 		// trigger deferred disconnection of peers
 		void on_remove_peers() noexcept;
@@ -1245,8 +1282,8 @@ namespace libtorrent {
 
 #ifndef TORRENT_DISABLE_DHT
 		static void on_dht_announce_response_disp(std::weak_ptr<torrent> t
-			, std::vector<tcp::endpoint> const& peers);
-		void on_dht_announce_response(std::vector<tcp::endpoint> const& peers);
+			, protocol_version v, std::vector<tcp::endpoint> const& peers);
+		void on_dht_announce_response(protocol_version v, std::vector<tcp::endpoint> const& peers);
 		bool should_announce_dht() const;
 #endif
 
@@ -1271,10 +1308,10 @@ namespace libtorrent {
 		// subsystem.
 		storage_holder m_storage;
 
-#ifdef TORRENT_USE_OPENSSL
-		std::unique_ptr<boost::asio::ssl::context> m_ssl_ctx;
+#ifdef TORRENT_SSL_PEERS
+		std::unique_ptr<ssl::context> m_ssl_ctx;
 
-		bool verify_peer_cert(bool preverified, boost::asio::ssl::verify_context& ctx);
+		bool verify_peer_cert(bool const preverified, ssl::verify_context& ctx);
 
 		void init_ssl(string_view cert);
 #endif
@@ -1324,7 +1361,7 @@ namespace libtorrent {
 		// us.
 		aux::suggest_piece m_suggest_pieces;
 
-		aux::vector<announce_entry> m_trackers;
+		aux::vector<aux::announce_entry> m_trackers;
 
 #ifndef TORRENT_DISABLE_STREAMING
 		// this list is sorted by time_critical_piece::deadline
@@ -1340,22 +1377,6 @@ namespace libtorrent {
 
 		std::string m_save_path;
 
-#if TORRENT_ABI_VERSION == 1
-		// deprecated in 1.2
-
-		// if we don't have the metadata, this is a url to
-		// the torrent file
-		std::string m_url;
-
-		// if this was added from an RSS feed, this is the unique
-		// identifier in the feed.
-		std::string m_uuid;
-
-		// if this torrent was added by an RSS feed, this is the
-		// URL to that feed
-		std::string m_source_feed_url;
-#endif
-
 #ifndef TORRENT_DISABLE_PREDICTIVE_PIECES
 		// this is a list of all pieces that we have announced
 		// as having, without actually having yet. If we receive
@@ -1370,6 +1391,9 @@ namespace libtorrent {
 		std::vector<piece_index_t> m_predictive_pieces;
 #endif
 
+		// v2 merkle tree for each file
+		aux::vector<aux::merkle_tree, file_index_t> m_merkle_trees;
+
 		// the performance counters of this session
 		counters& m_stats_counters;
 
@@ -1397,8 +1421,6 @@ namespace libtorrent {
 		// longer be used and will be reset
 		std::unique_ptr<std::string> m_name;
 
-		storage_constructor_type m_storage_constructor;
-
 		// the posix time this torrent was added and when
 		// it was completed. If the torrent isn't yet
 		// completed, m_completed_time is 0
@@ -1414,7 +1436,7 @@ namespace libtorrent {
 
 		// keep a copy if the info-hash here, so it can be accessed from multiple
 		// threads, and be cheap to access from the client
-		sha1_hash m_info_hash;
+		info_hash_t m_info_hash;
 
 	public:
 		// these are the lists this torrent belongs to. For more
@@ -1439,12 +1461,17 @@ namespace libtorrent {
 		// bias towards keeping seeding torrents that
 		// recently was started, to avoid oscillation
 		// this is specified at a second granularity
+		// in session-time. see session_impl for details.
+		// the reference point is stepped forward every 4
+		// hours to keep the timestamps fit in 16 bits
 		time_point32 m_started = aux::time_now32();
 
-		// if we're a seed, this is the timestamp of when we became one
+		// if we're a seed, this is the session time
+		// timestamp of when we became one
 		time_point32 m_became_seed = aux::time_now32();
 
-		// if we're finished, this is the timestamp of when we finished
+		// if we're finished, this is the session time
+		// timestamp of when we finished
 		time_point32 m_became_finished = aux::time_now32();
 
 		// when checking, this is the first piece we have not
@@ -1477,7 +1504,7 @@ namespace libtorrent {
 		// used to post a message to defer disconnecting peers
 		std::vector<std::shared_ptr<peer_connection>> m_peers_to_disconnect;
 		aux::deferred_handler m_deferred_disconnect;
-		aux::handler_storage<104> m_deferred_handler_storage;
+		aux::handler_storage<aux::deferred_handler_max_size, aux::defer_handler> m_deferred_handler_storage;
 
 		// these are the peer IDs we've used for our outgoing peer connections for
 		// this torrent. If we get an incoming peer claiming to have one of these,
@@ -1508,8 +1535,8 @@ namespace libtorrent {
 		// 8 bits after each one
 		// ==============================
 
-		// if we're currently in upload-mode this is the time timestamp of when
-		// we entered it
+		// the session time timestamp of when we entered upload mode
+		// if we're currently in upload-mode
 		time_point32 m_upload_mode_time = aux::time_now32();
 
 		// true when this torrent should announce to
@@ -1637,6 +1664,10 @@ namespace libtorrent {
 		// quarantine
 		bool m_pending_active_change:1;
 
+		// this is set to true if all piece layers were successfully loaded and
+		// validated. Only for v2 torrents
+		bool m_v2_piece_layers_validated:1;
+
 // ----
 
 		// the number of (16kiB) blocks that fall entirely in pad files
@@ -1691,7 +1722,7 @@ namespace libtorrent {
 		time_point32 m_last_upload{seconds32(0)};
 
 		// user data as passed in by add_torrent_params
-		void* m_userdata;
+		client_data_t m_userdata;
 
 // ----
 
