@@ -1,6 +1,7 @@
 /*
 
-Copyright (c) 2003-2018, Arvid Norberg
+Copyright (c) 2004, 2006, 2010, 2012, 2015, 2017-2020, Arvid Norberg
+Copyright (c) 2016, Jan Berkel
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -34,20 +35,32 @@ POSSIBILITY OF SUCH DAMAGE.
 #define TORRENT_VERSION_HPP_INCLUDED
 
 #include "libtorrent/aux_/export.hpp"
+#include <cstdint>
 
-#define LIBTORRENT_VERSION_MAJOR 1
-#define LIBTORRENT_VERSION_MINOR 2
-#define LIBTORRENT_VERSION_TINY 20
+#define LIBTORRENT_VERSION_MAJOR 2
+#define LIBTORRENT_VERSION_MINOR 0
+#define LIBTORRENT_VERSION_TINY 3
 
 // the format of this version is: MMmmtt
 // M = Major version, m = minor version, t = tiny version
 #define LIBTORRENT_VERSION_NUM ((LIBTORRENT_VERSION_MAJOR * 10000) + (LIBTORRENT_VERSION_MINOR * 100) + LIBTORRENT_VERSION_TINY)
 
-#define LIBTORRENT_VERSION "1.2.20.0"
-#define LIBTORRENT_REVISION "077262424"
+#define LIBTORRENT_VERSION "2.0.3.0"
+#define LIBTORRENT_REVISION "de8e69a79"
 
 namespace libtorrent {
 
+	// the major, minor and tiny versions of libtorrent
+	constexpr int version_major = 2;
+	constexpr int version_minor = 0;
+	constexpr int version_tiny = 3;
+
+	// the libtorrent version in string form
+	constexpr char const* version_str = "2.0.3.0";
+
+	// the git commit of this libtorrent version
+	constexpr std::uint64_t version_revision = 0xde8e69a79;
+
 	// returns the libtorrent version as string form in this format:
 	// "<major>.<minor>.<tiny>.<tag>"
 	TORRENT_EXPORT char const* version();
