@@ -1,6 +1,8 @@
 /*
 
-Copyright (c) 2007-2018, Arvid Norberg
+Copyright (c) 2007-2010, 2012-2020, Arvid Norberg
+Copyright (c) 2016-2017, Alden Torres
+Copyright (c) 2018, Steven Siloti
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -46,10 +48,22 @@ namespace libtorrent {
 	{
 		if (!handle.is_valid()) return "";
 
-		std::string ret;
-		sha1_hash const& ih = handle.info_hash();
-		ret += "magnet:?xt=urn:btih:";
-		ret += aux::to_hex(ih);
+		std::string ret = "magnet:?";
+
+		if (handle.info_hashes().has_v1())
+		{
+			sha1_hash const& ih = handle.info_hashes().v1;
+			ret += "xt=urn:btih:";
+			ret += aux::to_hex(ih);
+		}
+
+		if (handle.info_hashes().has_v2())
+		{
+			if (handle.info_hashes().has_v1()) ret += '&';
+			sha256_hash const& ih = handle.info_hashes().v2;
+			ret += "xt=urn:btmh:1220";
+			ret += aux::to_hex(ih);
+		}
 
 		torrent_status st = handle.status(torrent_handle::query_name);
 		if (!st.name.empty())
@@ -75,10 +89,22 @@ namespace libtorrent {
 
 	std::string make_magnet_uri(torrent_info const& info)
 	{
-		std::string ret;
-		sha1_hash const& ih = info.info_hash();
-		ret += "magnet:?xt=urn:btih:";
-		ret += aux::to_hex(ih);
+		std::string ret = "magnet:?";
+
+		if (info.info_hashes().has_v1())
+		{
+			sha1_hash const& ih = info.info_hashes().v1;
+			ret += "xt=urn:btih:";
+			ret += aux::to_hex(ih);
+		}
+
+		if (info.info_hashes().has_v2())
+		{
+			if (info.info_hashes().has_v1()) ret += '&';
+			sha256_hash const& ih = info.info_hashes().v2;
+			ret += "xt=urn:btmh:1220";
+			ret += aux::to_hex(ih);
+		}
 
 		std::string const& name = info.name();
 
@@ -129,14 +155,12 @@ namespace libtorrent {
 		, std::string const& save_path
 		, storage_mode_t storage_mode
 		, bool paused
-		, storage_constructor_type sc
-		, void* userdata)
+		, void*)
 	{
-		add_torrent_params params(std::move(sc));
+		add_torrent_params params;
 		error_code ec;
 		parse_magnet_uri(uri, params, ec);
 		params.storage_mode = storage_mode;
-		params.userdata = userdata;
 		params.save_path = save_path;
 
 		if (paused) params.flags |= add_torrent_params::flag_paused;
@@ -177,7 +201,7 @@ namespace libtorrent {
 		sv = sv.substr(8);
 
 		int tier = 0;
-		bool has_ih = false;
+		bool has_ih[2] = { false, false };
 		while (!sv.empty())
 		{
 			string_view name;
@@ -195,12 +219,12 @@ namespace libtorrent {
 			if (std::all_of(number.begin(), number.end(), [](char const c) { return is_digit(c); } ))
 				name = stripped_name;
 
-			if (name == "dn"_sv) // display name
+			if (string_equal_no_case(name, "dn"_sv)) // display name
 			{
 				error_code e;
 				display_name = unescape_string(value, e);
 			}
-			else if (name == "tr"_sv) // tracker
+			else if (string_equal_no_case(name, "tr"_sv)) // tracker
 			{
 				// since we're about to assign tiers to the trackers, make sure the two
 				// vectors are aligned
@@ -214,13 +238,13 @@ namespace libtorrent {
 					p.tracker_tiers.push_back(tier++);
 				}
 			}
-			else if (name == "ws"_sv) // web seed
+			else if (string_equal_no_case(name, "ws"_sv)) // web seed
 			{
 				error_code e;
 				std::string webseed = unescape_string(value, e);
 				if (!e) p.url_seeds.push_back(std::move(webseed));
 			}
-			else if (name == "xt"_sv)
+			else if (string_equal_no_case(name, "xt"_sv))
 			{
 				std::string unescaped_btih;
 				if (value.find('%') != string_view::npos)
@@ -230,30 +254,53 @@ namespace libtorrent {
 					value = unescaped_btih;
 				}
 
-				if (value.substr(0, 9) != "urn:btih:") continue;
-				value = value.substr(9);
-
-				sha1_hash info_hash;
-				if (value.size() == 40) aux::from_hex({value.data(), 40}, info_hash.data());
-				else if (value.size() == 32)
+				if (value.substr(0, 9) == "urn:btih:")
 				{
-					std::string const ih = base32decode(value);
-					if (ih.size() != 20)
+					value = value.substr(9);
+
+					sha1_hash info_hash;
+					if (value.size() == 40) aux::from_hex(value, info_hash.data());
+					else if (value.size() == 32)
+					{
+						std::string const ih = base32decode(value);
+						if (ih.size() != 20)
+						{
+							ec = errors::invalid_info_hash;
+							return;
+						}
+						info_hash.assign(ih);
+					}
+					else
 					{
 						ec = errors::invalid_info_hash;
 						return;
 					}
-					info_hash.assign(ih);
+					p.info_hashes.v1 = info_hash;
+					has_ih[0] = true;
 				}
-				else
+				else if (value.substr(0, 9) == "urn:btmh:")
 				{
-					ec = errors::invalid_info_hash;
-					return;
+					value = value.substr(9);
+
+					// hash must be sha256
+					if (value.substr(0, 4) != "1220")
+					{
+						ec = errors::invalid_info_hash;
+						return;
+					}
+
+					value = value.substr(4);
+
+					if (value.size() != 64)
+					{
+						ec = errors::invalid_info_hash;
+						return;
+					}
+					aux::from_hex(value, p.info_hashes.v2.data());
+					has_ih[1] = true;
 				}
-				p.info_hash = info_hash;
-				has_ih = true;
 			}
-			else if (name == "so"_sv) // select-only (files)
+			else if (string_equal_no_case(name, "so"_sv)) // select-only (files)
 			{
 				// accept only digits, '-' and ','
 				if (std::any_of(value.begin(), value.end(), [](char c)
@@ -305,14 +352,14 @@ namespace libtorrent {
 
 				} while (!value.empty());
 			}
-			else if (name == "x.pe")
+			else if (string_equal_no_case(name, "x.pe"_sv))
 			{
 				error_code e;
 				tcp::endpoint endp = parse_endpoint(value, e);
 				if (!e) p.peers.push_back(std::move(endp));
 			}
 #ifndef TORRENT_DISABLE_DHT
-			else if (name == "dht"_sv)
+			else if (string_equal_no_case(name, "dht"_sv))
 			{
 				auto const divider = value.find_last_of(':');
 				if (divider != std::string::npos)
@@ -325,12 +372,15 @@ namespace libtorrent {
 #endif
 		}
 
-		if (!has_ih)
+		if (!has_ih[0] && !has_ih[1])
 		{
 			ec = errors::missing_info_hash_in_uri;
 			return;
 		}
 
+#if TORRENT_ABI_VERSION < 3
+		p.info_hash = p.info_hashes.get_best();
+#endif
 		if (!display_name.empty()) p.name = display_name;
 	}
 
