@@ -1,6 +1,11 @@
 /*
 
-Copyright (c) 2003-2018, Arvid Norberg
+Copyright (c) 2003-2020, Arvid Norberg
+Copyright (c) 2004, Magnus Jonsson
+Copyright (c) 2009, Daniel Wallin
+Copyright (c) 2016-2018, Alden Torres
+Copyright (c) 2016, 2018, Steven Siloti
+Copyright (c) 2016, Andrei Kurushin
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -37,18 +42,17 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/peer_list.hpp"
 #include "libtorrent/socket.hpp"
 #include "libtorrent/aux_/socket_type.hpp"
-#include "libtorrent/invariant_check.hpp"
+#include "libtorrent/aux_/invariant_check.hpp"
 #include "libtorrent/time.hpp"
 #include "libtorrent/aux_/session_interface.hpp"
 #include "libtorrent/piece_picker.hpp"
-#include "libtorrent/broadcast_socket.hpp"
 #include "libtorrent/peer_info.hpp"
 #include "libtorrent/random.hpp"
 #include "libtorrent/extensions.hpp"
 #include "libtorrent/ip_filter.hpp"
 #include "libtorrent/torrent_peer_allocator.hpp"
 #include "libtorrent/ip_voter.hpp" // for external_ip
-#include "libtorrent/broadcast_socket.hpp" // for is_v6
+#include "libtorrent/aux_/ip_helpers.hpp" // for is_v6
 
 #if TORRENT_USE_ASSERTS
 #include "libtorrent/socket_io.hpp" // for print_endpoint
@@ -79,65 +83,7 @@ namespace {
 		address const& m_addr;
 		std::uint16_t m_port;
 	};
-
-	// this returns true if lhs is a better erase candidate than rhs
-	bool compare_peer_erase(torrent_peer const& lhs, torrent_peer const& rhs)
-	{
-		TORRENT_ASSERT(lhs.connection == nullptr);
-		TORRENT_ASSERT(rhs.connection == nullptr);
-
-		// primarily, prefer getting rid of peers we've already tried and failed
-		if (lhs.failcount != rhs.failcount)
-			return lhs.failcount > rhs.failcount;
-
-		bool const lhs_resume_data_source = lhs.peer_source() == peer_info::resume_data;
-		bool const rhs_resume_data_source = rhs.peer_source() == peer_info::resume_data;
-
-		// prefer to drop peers whose only source is resume data
-		if (lhs_resume_data_source != rhs_resume_data_source)
-			return int(lhs_resume_data_source) > int(rhs_resume_data_source);
-
-		if (lhs.connectable != rhs.connectable)
-			return int(lhs.connectable) < int(rhs.connectable);
-
-		return lhs.trust_points < rhs.trust_points;
-	}
-
-	// this returns true if lhs is a better connect candidate than rhs
-	bool compare_peer(torrent_peer const* lhs, torrent_peer const* rhs
-		, external_ip const& external, int const external_port, bool const finished)
-	{
-		// prefer peers with lower failcount
-		if (lhs->failcount != rhs->failcount)
-			return lhs->failcount < rhs->failcount;
-
-		// Local peers should always be tried first
-		bool const lhs_local = is_local(lhs->address());
-		bool const rhs_local = is_local(rhs->address());
-		if (lhs_local != rhs_local) return int(lhs_local) > int(rhs_local);
-
-		if (lhs->last_connected != rhs->last_connected)
-			return lhs->last_connected < rhs->last_connected;
-
-		if (finished && lhs->maybe_upload_only != rhs->maybe_upload_only)
-		{
-			// if we're finished, de-prioritze peers we think may be seeds
-			// since being upload-only doesn't necessarily mean it's a good peer
-			// to be connected to as a downloader, we don't prioritize the
-			// inverse when we're not finished
-			return rhs->maybe_upload_only;
-		}
-
-		int const lhs_rank = source_rank(lhs->peer_source());
-		int const rhs_rank = source_rank(rhs->peer_source());
-		if (lhs_rank != rhs_rank) return lhs_rank > rhs_rank;
-
-		std::uint32_t const lhs_peer_rank = lhs->rank(external, external_port);
-		std::uint32_t const rhs_peer_rank = rhs->rank(external, external_port);
-		return lhs_peer_rank > rhs_peer_rank;
-	}
-
-} // anonymous namespace
+}
 
 namespace libtorrent {
 
@@ -157,9 +103,7 @@ namespace libtorrent {
 		for (auto const p : m_peers)
 			m_peer_allocator.free_peer_entry(p);
 		m_peers.clear();
-		m_candidate_cache.clear();
 		m_num_connect_candidates = 0;
-		m_num_seeds = 0;
 	}
 
 	peer_list::~peer_list()
@@ -186,7 +130,7 @@ namespace libtorrent {
 		TORRENT_ASSERT(is_single_thread());
 		INVARIANT_CHECK;
 
-		for (iterator i = m_peers.begin(); i != m_peers.end();)
+		for (auto i = m_peers.begin(); i != m_peers.end();)
 		{
 			if ((filter.access((*i)->address()) & ip_filter::blocked) == 0)
 			{
@@ -248,7 +192,7 @@ namespace libtorrent {
 		TORRENT_ASSERT(is_single_thread());
 		INVARIANT_CHECK;
 
-		for (iterator i = m_peers.begin(); i != m_peers.end();)
+		for (auto i = m_peers.begin(); i != m_peers.end();)
 		{
 			if ((filter.access((*i)->port) & port_filter::blocked) == 0)
 			{
@@ -331,7 +275,7 @@ namespace libtorrent {
 
 		// if this peer is in the connect candidate
 		// cache, erase it from there as well
-		std::vector<torrent_peer*>::iterator ci = std::find(m_candidate_cache.begin(), m_candidate_cache.end(), *i);
+		auto const ci = std::find(m_candidate_cache.begin(), m_candidate_cache.end(), *i);
 		if (ci != m_candidate_cache.end()) m_candidate_cache.erase(ci);
 
 		m_peer_allocator.free_peer_entry(*i);
@@ -463,9 +407,6 @@ namespace libtorrent {
 
 		const bool was_conn_cand = is_connect_candidate(*p);
 		p->connection = c;
-		// now that we're connected, no need to assume ther peer is a seed
-		// anymore. We'll soon know.
-		p->maybe_upload_only = false;
 		if (was_conn_cand) update_connect_candidates(-1);
 	}
 
@@ -582,14 +523,14 @@ namespace libtorrent {
 			// pe, which is the peer m_round_robin points to. If it is, just
 			// keep looking.
 			if (peers.size() == candidate_count
-				&& compare_peer(peers.back(), &pe, external, external_port, m_finished)) continue;
+				&& compare_peer(peers.back(), &pe, external, external_port)) continue;
 
 			if (peers.size() >= candidate_count)
 				peers.resize(candidate_count - 1);
 
 			// insert this candidate sorted into peers
 			auto const i = std::lower_bound(peers.begin(), peers.end()
-				, &pe, std::bind(&compare_peer, _1, _2, std::cref(external), external_port, bool(m_finished)));
+				, &pe, std::bind(&peer_list::compare_peer, this, _1, _2, std::cref(external), external_port));
 
 			peers.insert(i, &pe);
 		}
@@ -608,6 +549,8 @@ namespace libtorrent {
 
 		INVARIANT_CHECK;
 
+		TORRENT_ASSERT(!state->is_paused);
+
 		iterator iter;
 		torrent_peer* i = nullptr;
 
@@ -713,9 +656,10 @@ namespace libtorrent {
 
 					// decide which peer connection to disconnect
 					// if the ports are equal, pick on at random
-					bool const disconnect1 = ((our_port < other_port) && !outgoing1)
+					bool disconnect1 = ((our_port < other_port) && !outgoing1)
 						|| ((our_port > other_port) && outgoing1)
 						|| ((our_port == other_port) && random(1));
+					disconnect1 &= !i->connection->failed();
 
 #ifndef TORRENT_DISABLE_LOGGING
 					if (c.should_log(peer_log_alert::info))
@@ -766,7 +710,7 @@ namespace libtorrent {
 				);
 			}
 
-			bool const is_v6 = lt::is_v6(c.remote());
+			bool const is_v6 = lt::aux::is_v6(c.remote());
 			torrent_peer* p = m_peer_allocator.allocate_peer_entry(
 				is_v6 ? torrent_peer_allocator_interface::ipv6_peer_type
 				: torrent_peer_allocator_interface::ipv4_peer_type);
@@ -889,7 +833,7 @@ namespace libtorrent {
 		TORRENT_ASSERT(is_single_thread());
 		if (p == nullptr) return;
 		TORRENT_ASSERT(p->in_use);
-		if (bool(p->seed) == s) return;
+		if (p->seed == s) return;
 		bool const was_conn_cand = is_connect_candidate(*p);
 		p->seed = s;
 		if (was_conn_cand && !is_connect_candidate(*p))
@@ -952,11 +896,17 @@ namespace libtorrent {
 		if (flags & pex_encryption) p->pe_support = true;
 #endif
 		if (flags & pex_seed)
-			p->maybe_upload_only = true;
+		{
+			p->seed = true;
+			TORRENT_ASSERT(m_num_seeds < int(m_peers.size()));
+			++m_num_seeds;
+		}
 		if (flags & pex_utp)
 			p->supports_utp = true;
 		if (flags & pex_holepunch)
 			p->supports_holepunch = true;
+		if (flags & pex_lt_v2)
+			p->protocol_v2 = true;
 		if (is_connect_candidate(*p))
 			update_connect_candidates(1);
 
@@ -987,11 +937,20 @@ namespace libtorrent {
 		// we already know if it's a seed or not
 		// so we don't have to trust this source
 		if ((flags & pex_seed) && !p->connection)
-			p->maybe_upload_only = true;
+		{
+			if (!p->seed)
+			{
+				TORRENT_ASSERT(m_num_seeds < int(m_peers.size()));
+				++m_num_seeds;
+			}
+			p->seed = true;
+		}
 		if (flags & pex_utp)
 			p->supports_utp = true;
 		if (flags & pex_holepunch)
 			p->supports_holepunch = true;
+		if (flags & pex_lt_v2)
+			p->protocol_v2 = true;
 
 		if (was_conn_cand != is_connect_candidate(*p))
 		{
@@ -1019,7 +978,7 @@ namespace libtorrent {
 		TORRENT_ASSERT(is_single_thread());
 		INVARIANT_CHECK;
 
-		iterator iter = std::lower_bound(m_peers.begin(), m_peers.end()
+		auto iter = std::lower_bound(m_peers.begin(), m_peers.end()
 			, destination, peer_address_compare());
 
 		if (iter != m_peers.end() && (*iter)->dest() == destination)
@@ -1268,7 +1227,7 @@ namespace libtorrent {
 
 		TORRENT_ASSERT(c);
 
-		iterator iter = std::lower_bound(m_peers.begin(), m_peers.end()
+		auto const iter = std::lower_bound(m_peers.begin(), m_peers.end()
 			, c->remote().address(), peer_address_compare());
 
 		if (iter != m_peers.end() && (*iter)->address() == c->remote().address())
@@ -1325,4 +1284,53 @@ namespace libtorrent {
 	}
 #endif
 
+	// this returns true if lhs is a better erase candidate than rhs
+	bool peer_list::compare_peer_erase(torrent_peer const& lhs, torrent_peer const& rhs) const
+	{
+		TORRENT_ASSERT(is_single_thread());
+		TORRENT_ASSERT(lhs.connection == nullptr);
+		TORRENT_ASSERT(rhs.connection == nullptr);
+
+		// primarily, prefer getting rid of peers we've already tried and failed
+		if (lhs.failcount != rhs.failcount)
+			return lhs.failcount > rhs.failcount;
+
+		bool const lhs_resume_data_source = lhs.peer_source() == peer_info::resume_data;
+		bool const rhs_resume_data_source = rhs.peer_source() == peer_info::resume_data;
+
+		// prefer to drop peers whose only source is resume data
+		if (lhs_resume_data_source != rhs_resume_data_source)
+			return int(lhs_resume_data_source) > int(rhs_resume_data_source);
+
+		if (lhs.connectable != rhs.connectable)
+			return int(lhs.connectable) < int(rhs.connectable);
+
+		return lhs.trust_points < rhs.trust_points;
+	}
+
+	// this returns true if lhs is a better connect candidate than rhs
+	bool peer_list::compare_peer(torrent_peer const* lhs, torrent_peer const* rhs
+		, external_ip const& external, int external_port) const
+	{
+		TORRENT_ASSERT(is_single_thread());
+		// prefer peers with lower failcount
+		if (lhs->failcount != rhs->failcount)
+			return lhs->failcount < rhs->failcount;
+
+		// Local peers should always be tried first
+		bool const lhs_local = aux::is_local(lhs->address());
+		bool const rhs_local = aux::is_local(rhs->address());
+		if (lhs_local != rhs_local) return int(lhs_local) > int(rhs_local);
+
+		if (lhs->last_connected != rhs->last_connected)
+			return lhs->last_connected < rhs->last_connected;
+
+		int const lhs_rank = source_rank(lhs->peer_source());
+		int const rhs_rank = source_rank(rhs->peer_source());
+		if (lhs_rank != rhs_rank) return lhs_rank > rhs_rank;
+
+		std::uint32_t const lhs_peer_rank = lhs->rank(external, external_port);
+		std::uint32_t const rhs_peer_rank = rhs->rank(external, external_port);
+		return lhs_peer_rank > rhs_peer_rank;
+	}
 }
