@@ -1,6 +1,7 @@
 /*
 
-Copyright (c) 2003-2018, Arvid Norberg
+Copyright (c) 2014-2020, Arvid Norberg
+Copyright (c) 2016, 2019-2020, Alden Torres
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -36,7 +37,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/aux_/socket_type.hpp"
 #include "libtorrent/peer_info.hpp" // for peer_info flags
 #include "libtorrent/request_blocks.hpp"
-#include "libtorrent/alert_manager.hpp"
+#include "libtorrent/aux_/alert_manager.hpp"
 #include "libtorrent/aux_/has_block.hpp"
 
 #include <vector>
@@ -142,7 +143,7 @@ namespace libtorrent {
 		// than we requested.
 #if TORRENT_USE_ASSERTS
 		error_code ec;
-		TORRENT_ASSERT(c.remote() == c.get_socket()->remote_endpoint(ec) || ec);
+		TORRENT_ASSERT(c.remote() == c.get_socket().remote_endpoint(ec) || ec);
 #endif
 
 		aux::session_interface& ses = t.session();
@@ -251,8 +252,7 @@ namespace libtorrent {
 				|| std::find_if(rq.begin(), rq.end(), aux::has_block(pb)) != rq.end())
 			{
 #if TORRENT_USE_ASSERTS
-				std::vector<pending_block>::const_iterator j
-					= std::find_if(dq.begin(), dq.end(), aux::has_block(pb));
+				auto const j = std::find_if(dq.begin(), dq.end(), aux::has_block(pb));
 				if (j != dq.end()) TORRENT_ASSERT(j->timed_out || j->not_wanted);
 #endif
 #ifndef TORRENT_DISABLE_LOGGING
