@@ -1,6 +1,11 @@
 /*
 
-Copyright (c) 2008, Arvid Norberg
+Copyright (c) 2006-2020, Arvid Norberg
+Copyright (c) 2015-2017, Alden Torres
+Copyright (c) 2016-2018, Steven Siloti
+Copyright (c) 2016, Andrei Kurushin
+Copyright (c) 2017, 2020, AllSeeingEyeTolledEweSew
+Copyright (c) 2018, d-komarov
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -30,7 +35,6 @@ POSSIBILITY OF SUCH DAMAGE.
 
 */
 
-#include <fstream>
 #include <map>
 #include <tuple>
 #include <functional>
@@ -46,10 +50,12 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/session_stats.hpp"
 #include "libtorrent/random.hpp"
 #include "libtorrent/torrent_info.hpp"
-#include "libtorrent/broadcast_socket.hpp" // for supports_ipv6()
 #include "libtorrent/hex.hpp" // to_hex
 #include "libtorrent/aux_/vector.hpp"
 #include "libtorrent/aux_/path.hpp"
+#include "libtorrent/aux_/merkle.hpp"
+#include "libtorrent/disk_interface.hpp" // for default_block_size
+#include "libtorrent/aux_/ip_helpers.hpp"
 
 #include "test.hpp"
 #include "test_utils.hpp"
@@ -80,15 +86,15 @@ std::shared_ptr<torrent_info> generate_torrent(bool const with_files, bool const
 		create_directories("test_resume", ec);
 		std::vector<char> a(128 * 1024 * 8);
 		std::vector<char> b(128 * 1024);
-		std::ofstream("test_resume/tmp1").write(a.data(), std::streamsize(a.size()));
-		std::ofstream("test_resume/tmp2").write(b.data(), std::streamsize(b.size()));
-		std::ofstream("test_resume/tmp3").write(b.data(), std::streamsize(b.size()));
+		ofstream("test_resume/tmp1").write(a.data(), std::streamsize(a.size()));
+		ofstream("test_resume/tmp2").write(b.data(), std::streamsize(b.size()));
+		ofstream("test_resume/tmp3").write(b.data(), std::streamsize(b.size()));
 	}
 	file_storage fs;
 	fs.add_file("test_resume/tmp1", 128 * 1024 * 8);
 	fs.add_file("test_resume/tmp2", 128 * 1024);
 	fs.add_file("test_resume/tmp3", 128 * 1024);
-	lt::create_torrent t(fs, 128 * 1024, 6);
+	lt::create_torrent t(fs, 128 * 1024);
 
 	t.set_comment("test comment");
 	t.set_creator("libtorrent test");
@@ -109,6 +115,14 @@ std::shared_ptr<torrent_info> generate_torrent(bool const with_files, bool const
 			aux::random_bytes(ph);
 			t.set_hash(i, ph);
 		}
+
+		for (piece_index_t i : fs.piece_range())
+		{
+			sha256_hash ph;
+			aux::random_bytes(ph);
+			file_index_t const f(fs.file_index_at_piece(i));
+			t.set_hash2(f, i - fs.piece_index_at_file(f), ph);
+		}
 	}
 
 	std::vector<char> buf;
@@ -138,7 +152,7 @@ address rand_v4()
 	{
 		g_addr += 0x3080ca;
 		ret = address_v4(g_addr);
-	} while (is_any(ret) || is_local(ret) || is_loopback(ret));
+	} while (ret.is_unspecified() || aux::is_local(ret) || ret.is_loopback());
 	return ret;
 }
 
@@ -182,7 +196,7 @@ address rand_v6()
 		add_mp(g_addr6, add);
 		ret = address_v6(g_addr6);
 
-	} while (is_any(ret) || is_local(ret) || is_loopback(ret));
+	} while (ret.is_unspecified() || aux::is_local(ret) || ret.is_loopback());
 	return ret;
 }
 
@@ -201,6 +215,28 @@ udp::endpoint rand_udp_ep(lt::address(&rand_addr)())
 	return udp::endpoint(rand_addr(), g_port + 1024);
 }
 
+bool supports_ipv6()
+{
+#if defined TORRENT_BUILD_SIMULATOR
+	return true;
+#elif defined TORRENT_WINDOWS
+	TORRENT_TRY {
+		error_code ec;
+		make_address("::1", ec);
+		return !ec;
+	} TORRENT_CATCH(std::exception const&) { return false; }
+#else
+	io_context ios;
+	tcp::socket test(ios);
+	error_code ec;
+	test.open(tcp::v6(), ec);
+	if (ec) return false;
+	error_code ignore;
+	test.bind(tcp::endpoint(make_address_v6("::1", ignore), 0), ec);
+	return !bool(ec);
+#endif
+}
+
 std::map<std::string, std::int64_t> get_counters(lt::session& s)
 {
 	using namespace lt;
@@ -361,7 +397,8 @@ bool print_alerts(lt::session& ses, char const* name
 	{
 		if (peer_disconnected_alert const* p = alert_cast<peer_disconnected_alert>(a))
 		{
-			std::printf("%s: %s: [%s] (%s): %s\n", time_now_string().c_str(), name, a->what()
+			std::printf("%s: %s: [%s] (%s): %s\n", time_to_string(a->timestamp()).c_str()
+				, name, a->what()
 				, print_endpoint(p->endpoint).c_str(), p->message().c_str());
 		}
 		else if (a->type() == invalid_request_alert::alert_type)
@@ -515,8 +552,8 @@ pid_type async_run(char const* cmdline)
 	startup.hStdInput = GetStdHandle(STD_INPUT_HANDLE);
 	startup.hStdOutput = GetStdHandle(STD_OUTPUT_HANDLE);
 	startup.hStdError = GetStdHandle(STD_OUTPUT_HANDLE);
-	int const ret = CreateProcessA(NULL, buf, NULL, NULL, TRUE
-		, 0, NULL, NULL, &startup, &pi);
+	int const ret = CreateProcessA(nullptr, buf, nullptr, nullptr, TRUE
+		, 0, nullptr, nullptr, &startup, &pi);
 
 	if (ret == 0)
 	{
@@ -570,7 +607,7 @@ void stop_process(pid_type p)
 {
 #ifdef _WIN32
 	HANDLE proc = OpenProcess(PROCESS_TERMINATE | SYNCHRONIZE, FALSE, p);
-	if (proc == NULL) return;
+	if (proc == nullptr) return;
 	TerminateProcess(proc, 138);
 	WaitForSingleObject(proc, 5000);
 	CloseHandle(proc);
@@ -622,7 +659,7 @@ void wait_for_port(int const port)
 {
 	// wait until the python program is ready to accept connections
 	int i = 0;
-	io_service ios;
+	io_context ios;
 	for (;;)
 	{
 		tcp::socket s(ios);
@@ -634,7 +671,7 @@ void wait_for_port(int const port)
 				, ec.value(), ec.message().c_str());
 			return;
 		}
-		s.connect(tcp::endpoint(address::from_string("127.0.0.1")
+		s.connect(tcp::endpoint(make_address("127.0.0.1")
 			, std::uint16_t(port)), ec);
 		if (ec == boost::system::errc::connection_refused)
 		{
@@ -659,39 +696,25 @@ void wait_for_port(int const port)
 }
 #endif
 
-std::string get_python()
+std::vector<std::string> get_python()
 {
+	std::vector<std::string> ret;
 #ifdef _WIN32
 	char dummy[1];
 	DWORD const req_size = GetEnvironmentVariable("PYTHON_INTERPRETER", dummy, sizeof(dummy));
 	if (req_size > 1 && req_size < 4096)
 	{
 		std::vector<char> buf(req_size);
-		DWORD const sz = GetEnvironmentVariable("PYTHON_INTERPRETER", buf.data(), DWORD(buf.size()));
-		if (size_t(sz) == buf.size() - 1) return buf.data();
+		DWORD const sz = GetEnvironmentVariable("PYTHON_INTERPRETER", buf.data(), buf.size());
+		if (sz == buf.size() - 1) ret.emplace_back(buf.data(), buf.size());
 	}
 #endif
-	return "python3";
+	ret.push_back("python3");
+	ret.push_back("python");
+	return ret;
 }
 
-int find_available_port()
-{
-	int port = 2000 + (::getpid() + unit_test::g_test_idx + std::rand()) % 60000;
-	error_code ec;
-	io_service ios;
-
-	// make sure the port we pick is free
-	do {
-		++port;
-		tcp::socket s(ios);
-		s.open(tcp::v4(), ec);
-		if (ec) break;
-		s.bind(tcp::endpoint(address::from_string("127.0.0.1")
-			, std::uint16_t(port)), ec);
-	} while (ec);
-	return port;
-}
-}
+} // anonymous namespace
 
 // returns a port on success and -1 on failure
 int start_proxy(int proxy_type)
@@ -704,7 +727,20 @@ int start_proxy(int proxy_type)
 		if (i->second.type == proxy_type) { return i->first; }
 	}
 
-	int const port = find_available_port();
+	int port = 10000 + static_cast<int>(std::rand() % 50000);
+	error_code ec;
+	io_context ios;
+
+	// make sure the port we pick is free
+	do {
+		++port;
+		tcp::socket s(ios);
+		s.open(tcp::v4(), ec);
+		if (ec) break;
+		s.bind(tcp::endpoint(make_address("127.0.0.1")
+			, std::uint16_t(port)), ec);
+	} while (ec);
+
 
 	char const* type = "";
 	char const* auth = "";
@@ -736,20 +772,24 @@ int start_proxy(int proxy_type)
 			cmd = ".." SEPARATOR "http_proxy.py";
 			break;
 	}
-	std::string python_exe = get_python();
-	char buf[1024];
-	std::snprintf(buf, sizeof(buf), "%s %s --port %d%s", python_exe.c_str(), cmd, port, auth);
+	std::vector<std::string> python_exes = get_python();
+	for (auto const& python_exe : python_exes)
+	{
+		char buf[1024];
+		std::snprintf(buf, sizeof(buf), "%s %s --port %d%s", python_exe.c_str(), cmd, port, auth);
 
-	std::printf("%s starting proxy on port %d (%s %s)...\n", time_now_string().c_str(), port, type, auth);
-	std::printf("%s\n", buf);
-	pid_type r = async_run(buf);
-	if (r == 0) abort();
-	proxy_t t = { r, proxy_type };
-	running_proxies.insert(std::make_pair(port, t));
-	std::printf("%s launched\n", time_now_string().c_str());
-	std::this_thread::sleep_for(lt::milliseconds(500));
-	wait_for_port(port);
-	return port;
+		std::printf("%s starting proxy on port %d (%s %s)...\n", time_now_string().c_str(), port, type, auth);
+		std::printf("%s\n", buf);
+		pid_type r = async_run(buf);
+		if (r == 0) continue;
+		proxy_t t = { r, proxy_type };
+		running_proxies.insert(std::make_pair(port, t));
+		std::printf("%s launched\n", time_now_string().c_str());
+		std::this_thread::sleep_for(lt::milliseconds(500));
+		wait_for_port(port);
+		return port;
+	}
+	abort();
 }
 
 using namespace lt;
@@ -786,7 +826,7 @@ lt::file_storage make_file_storage(span<const int> const file_sizes
 	}
 
 	fs.set_piece_length(piece_size);
-	fs.set_num_pieces(int((fs.total_size() + piece_size - 1) / piece_size));
+	fs.set_num_pieces(aux::calc_num_pieces(fs));
 
 	return fs;
 }
@@ -796,14 +836,29 @@ std::shared_ptr<lt::torrent_info> make_torrent(span<const int> const file_sizes
 {
 	using namespace lt;
 	file_storage fs = make_file_storage(file_sizes, piece_size);
+	return make_torrent(fs);
+}
 
-	lt::create_torrent ct(fs, piece_size, 0x4000
-		, lt::create_torrent::optimize_alignment);
+std::shared_ptr<lt::torrent_info> make_torrent(lt::file_storage& fs)
+{
+	lt::create_torrent ct(fs, fs.piece_length());
 
 	for (auto const i : fs.piece_range())
 	{
 		std::vector<char> piece = generate_piece(i, fs.piece_size(i));
 		ct.set_hash(i, hasher(piece).final());
+
+		aux::vector<sha256_hash> tree(merkle_num_nodes(fs.piece_length() / default_block_size));
+
+		int const blocks_per_piece = fs.piece_length() / default_block_size;
+		for (int j = 0; j < int(piece.size()); j += default_block_size)
+		{
+			tree[tree.end_index() - blocks_per_piece + j / default_block_size]
+				= hasher256(piece.data() + j, std::min(default_block_size, int(piece.size()) - j)).final();
+		}
+		merkle_fill_tree(tree, fs.piece_length() / default_block_size);
+		file_index_t const f(fs.file_index_at_piece(i));
+		ct.set_hash2(f, i - fs.piece_index_at_file(f), tree[0]);
 	}
 
 	std::vector<char> buf;
@@ -834,17 +889,12 @@ void create_random_files(std::string const& path, span<const int> file_sizes
 
 		int to_write = file_sizes[i];
 		if (fs) fs->add_file(full_path, to_write);
-		file f(full_path, open_mode::write_only, ec);
-		if (ec) std::printf("failed to create file \"%s\": (%d) %s\n"
-			, full_path.c_str(), ec.value(), ec.message().c_str());
+		ofstream f(full_path.c_str());
 		std::int64_t offset = 0;
 		while (to_write > 0)
 		{
 			int const s = std::min(to_write, static_cast<int>(random_data.size()));
-			iovec_t const b = { random_data.data(), s};
-			f.writev(offset, b, ec);
-			if (ec) std::printf("failed to write file \"%s\": (%d) %s\n"
-				, full_path.c_str(), ec.value(), ec.message().c_str());
+			f.write(random_data.data(), s);
 			offset += s;
 			to_write -= s;
 		}
@@ -853,7 +903,8 @@ void create_random_files(std::string const& path, span<const int> file_sizes
 
 std::shared_ptr<torrent_info> create_torrent(std::ostream* file
 	, char const* name, int piece_size
-	, int num_pieces, bool add_tracker, std::string ssl_certificate)
+	, int num_pieces, bool add_tracker, lt::create_flags_t const flags
+	, std::string ssl_certificate)
 {
 	// exercise the path when encountering invalid urls
 	char const* invalid_tracker_url = "http:";
@@ -862,7 +913,7 @@ std::shared_ptr<torrent_info> create_torrent(std::ostream* file
 	file_storage fs;
 	int total_size = piece_size * num_pieces;
 	fs.add_file(name, total_size);
-	lt::create_torrent t(fs, piece_size);
+	lt::create_torrent t(fs, piece_size, flags);
 	if (add_tracker)
 	{
 		t.add_tracker(invalid_tracker_url);
@@ -890,16 +941,34 @@ std::shared_ptr<torrent_info> create_torrent(std::ostream* file
 	for (int i = 0; i < piece.end_index(); ++i)
 		piece[i] = (i % 26) + 'A';
 
-	// calculate the hash for all pieces
-	sha1_hash ph = hasher(piece).final();
-	for (auto const i : fs.piece_range())
-		t.set_hash(i, ph);
+	if (!(flags & create_torrent::v2_only))
+	{
+		// calculate the hash for all pieces
+		sha1_hash ph = hasher(piece).final();
+		for (auto const i : fs.piece_range())
+			t.set_hash(i, ph);
+	}
+
+	if (!(flags & create_torrent::v1_only))
+	{
+		int const blocks_in_piece = piece_size / default_block_size;
+		aux::vector<sha256_hash> v2tree(merkle_num_nodes(merkle_num_leafs(blocks_in_piece)));
+		for (int i = 0; i < blocks_in_piece; ++i)
+		{
+			sha256_hash const block_hash = hasher256(span<char>(piece).subspan(i * default_block_size, default_block_size)).final();
+			v2tree[v2tree.end_index() - merkle_num_leafs(blocks_in_piece) + i] = block_hash;
+		}
+		merkle_fill_tree(v2tree, merkle_num_leafs(blocks_in_piece));
+
+		for (piece_index_t i(0); i < t.files().end_piece(); ++i)
+			t.set_hash2(file_index_t{ 0 }, i - 0_piece, v2tree[0]);
+	}
 
 	if (file)
 	{
 		while (total_size > 0)
 		{
-			file->write(&piece[0], std::min(piece.end_index(), total_size));
+			file->write(piece.data(), std::min(piece.end_index(), total_size));
 			total_size -= piece.end_index();
 		}
 	}
@@ -918,7 +987,7 @@ setup_transfer(lt::session* ses1, lt::session* ses2, lt::session* ses3
 	, std::string suffix, int piece_size
 	, std::shared_ptr<torrent_info>* torrent, bool super_seeding
 	, add_torrent_params const* p, bool stop_lsd, bool use_ssl_ports
-	, std::shared_ptr<torrent_info>* torrent2)
+	, std::shared_ptr<torrent_info>* torrent2, create_flags_t const flags)
 {
 	TORRENT_ASSERT(ses1);
 	TORRENT_ASSERT(ses2);
@@ -935,8 +1004,8 @@ setup_transfer(lt::session* ses1, lt::session* ses2, lt::session* ses3
 	// This has the effect of applying the global
 	// rule to all peers, regardless of if they're local or not
 	ip_filter f;
-	f.add_rule(address_v4::from_string("0.0.0.0")
-		, address_v4::from_string("255.255.255.255")
+	f.add_rule(make_address_v4("0.0.0.0")
+		, make_address_v4("255.255.255.255")
 		, 1 << static_cast<std::uint32_t>(lt::session::global_peer_class_id));
 	ses1->set_peer_class_filter(f);
 	ses2->set_peer_class_filter(f);
@@ -959,15 +1028,15 @@ setup_transfer(lt::session* ses1, lt::session* ses2, lt::session* ses3
 		error_code ec;
 		create_directory("tmp1" + suffix, ec);
 		std::string const file_path = combine_path("tmp1" + suffix, "temporary");
-		std::ofstream file(file_path.c_str());
-		t = ::create_torrent(&file, "temporary", piece_size, 9, false);
+		ofstream file(file_path.c_str());
+		t = ::create_torrent(&file, "temporary", piece_size, 9, false, flags);
 		file.close();
 		if (clear_files)
 		{
 			remove_all(combine_path("tmp2" + suffix, "temporary"), ec);
 			remove_all(combine_path("tmp3" + suffix, "temporary"), ec);
 		}
-		std::printf("generated torrent: %s %s\n", aux::to_hex(t->info_hash()).c_str(), file_path.c_str());
+		std::printf("generated torrent: %s %s\n", aux::to_hex(t->info_hashes().v2).c_str(), file_path.c_str());
 	}
 	else
 	{
@@ -1015,7 +1084,7 @@ setup_transfer(lt::session* ses1, lt::session* ses2, lt::session* ses3
 	if (use_metadata_transfer)
 	{
 		param.ti.reset();
-		param.info_hash = t->info_hash();
+		param.info_hashes = t->info_hashes();
 	}
 	else if (torrent2)
 	{
@@ -1054,7 +1123,7 @@ setup_transfer(lt::session* ses1, lt::session* ses2, lt::session* ses3
 
 		std::printf("%s: ses1: connecting peer port: %d\n"
 			, time_now_string().c_str(), port);
-		tor1.connect_peer(tcp::endpoint(address::from_string("127.0.0.1", ec)
+		tor1.connect_peer(tcp::endpoint(make_address("127.0.0.1", ec)
 			, std::uint16_t(port)));
 
 		if (ses3)
@@ -1077,10 +1146,10 @@ setup_transfer(lt::session* ses1, lt::session* ses2, lt::session* ses3
 
 			std::printf("ses3: connecting peer port: %d\n", port);
 			tor3.connect_peer(tcp::endpoint(
-					address::from_string("127.0.0.1", ec), std::uint16_t(port)));
+					make_address("127.0.0.1", ec), std::uint16_t(port)));
 			std::printf("ses3: connecting peer port: %d\n", port2);
 				tor3.connect_peer(tcp::endpoint(
-					address::from_string("127.0.0.1", ec)
+					make_address("127.0.0.1", ec)
 					, std::uint16_t(port2)));
 		}
 	}
@@ -1094,24 +1163,40 @@ pid_type web_server_pid = 0;
 
 int start_web_server(bool ssl, bool chunked_encoding, bool keepalive, int min_interval)
 {
-	int const port = find_available_port();
+	int port = 2000 + static_cast<int>(std::rand() % 6000);
+	error_code ec;
+	io_context ios;
 
-	std::string python_exe = get_python();
+	// make sure the port we pick is free
+	do {
+		++port;
+		tcp::socket s(ios);
+		s.open(tcp::v4(), ec);
+		if (ec) break;
+		s.bind(tcp::endpoint(make_address("127.0.0.1")
+			, std::uint16_t(port)), ec);
+	} while (ec);
 
-	char buf[200];
-	std::snprintf(buf, sizeof(buf), "%s .." SEPARATOR "web_server.py %d %d %d %d %d"
-		, python_exe.c_str(), port, chunked_encoding, ssl, keepalive, min_interval);
+	std::vector<std::string> python_exes = get_python();
 
-	std::printf("%s starting web_server on port %d...\n", time_now_string().c_str(), port);
+	for (auto const& python_exe : python_exes)
+	{
+		char buf[200];
+		std::snprintf(buf, sizeof(buf), "%s .." SEPARATOR "web_server.py %d %d %d %d %d"
+			, python_exe.c_str(), port, chunked_encoding, ssl, keepalive, min_interval);
 
-	std::printf("%s\n", buf);
-	pid_type r = async_run(buf);
-	if (r == 0) abort();
-	web_server_pid = r;
-	std::printf("%s launched\n", time_now_string().c_str());
-	std::this_thread::sleep_for(lt::milliseconds(1000));
-	wait_for_port(port);
-	return port;
+		std::printf("%s starting web_server on port %d...\n", time_now_string().c_str(), port);
+
+		std::printf("%s\n", buf);
+		pid_type r = async_run(buf);
+		if (r == 0) continue;
+		web_server_pid = r;
+		std::printf("%s launched\n", time_now_string().c_str());
+		std::this_thread::sleep_for(lt::milliseconds(1000));
+		wait_for_port(port);
+		return port;
+	}
+	abort();
 }
 
 void stop_web_server()
@@ -1125,7 +1210,7 @@ void stop_web_server()
 tcp::endpoint ep(char const* ip, int port)
 {
 	error_code ec;
-	tcp::endpoint ret(address::from_string(ip, ec), std::uint16_t(port));
+	tcp::endpoint ret(make_address(ip, ec), std::uint16_t(port));
 	TEST_CHECK(!ec);
 	return ret;
 }
@@ -1133,7 +1218,7 @@ tcp::endpoint ep(char const* ip, int port)
 udp::endpoint uep(char const* ip, int port)
 {
 	error_code ec;
-	udp::endpoint ret(address::from_string(ip, ec), std::uint16_t(port));
+	udp::endpoint ret(make_address(ip, ec), std::uint16_t(port));
 	TEST_CHECK(!ec);
 	return ret;
 }
@@ -1141,7 +1226,7 @@ udp::endpoint uep(char const* ip, int port)
 lt::address addr(char const* ip)
 {
 	lt::error_code ec;
-	auto ret = lt::address::from_string(ip, ec);
+	auto ret = lt::make_address(ip, ec);
 	TEST_CHECK(!ec);
 	return ret;
 }
@@ -1149,7 +1234,7 @@ lt::address addr(char const* ip)
 lt::address_v4 addr4(char const* ip)
 {
 	lt::error_code ec;
-	auto ret = lt::address_v4::from_string(ip, ec);
+	auto ret = lt::make_address_v4(ip, ec);
 	TEST_CHECK(!ec);
 	return ret;
 }
@@ -1157,7 +1242,7 @@ lt::address_v4 addr4(char const* ip)
 lt::address_v6 addr6(char const* ip)
 {
 	lt::error_code ec;
-	auto ret = lt::address_v6::from_string(ip, ec);
+	auto ret = lt::make_address_v6(ip, ec);
 	TEST_CHECK(!ec);
 	return ret;
 }
