@@ -1,6 +1,8 @@
 /*
 
-Copyright (c) 2011, Arvid Norberg
+Copyright (c) 2008-2009, 2012-2019, Arvid Norberg
+Copyright (c) 2016, Andrei Kurushin
+Copyright (c) 2018, Alden Torres
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -32,21 +34,23 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include "libtorrent/session.hpp"
 #include "libtorrent/session_settings.hpp"
-#include "libtorrent/hasher.hpp"
+#include "libtorrent/session_params.hpp"
 #include "libtorrent/alert_types.hpp"
 #include "libtorrent/ip_filter.hpp"
-#include "libtorrent/thread.hpp"
-#include <boost/tuple/tuple.hpp>
+#include "libtorrent/aux_/path.hpp"
+#include <tuple>
 #include <iostream>
 
 #include "test.hpp"
+#include "test_utils.hpp"
 #include "setup_transfer.hpp"
 #include "settings.hpp"
 
+namespace {
+
 void test_swarm()
 {
-	using namespace libtorrent;
-	namespace lt = libtorrent;
+	using namespace lt;
 
 	// these are declared before the session objects
 	// so that they are destructed last. This enables
@@ -67,14 +71,14 @@ void test_swarm()
 
 	pack.set_bool(settings_pack::allow_multiple_connections_per_ip, true);
 	pack.set_int(settings_pack::choking_algorithm, settings_pack::rate_based_choker);
-	pack.set_int(settings_pack::upload_rate_limit, rate_limit);
+	pack.set_int(settings_pack::upload_rate_limit, int(rate_limit));
 	pack.set_int(settings_pack::unchoke_slots_limit, 1);
 	pack.set_int(settings_pack::max_retry_port_bind, 900);
 	pack.set_str(settings_pack::listen_interfaces, "0.0.0.0:48010");
 	pack.set_bool(settings_pack::enable_natpmp, false);
 	pack.set_bool(settings_pack::enable_upnp, false);
 	pack.set_bool(settings_pack::enable_dht, false);
-#ifndef TORRENT_NO_DEPRECATE
+#if TORRENT_ABI_VERSION == 1
 	pack.set_bool(settings_pack::rate_limit_utp, true);
 #endif
 
@@ -83,8 +87,8 @@ void test_swarm()
 
 	lt::session ses1(pack);
 
-	pack.set_int(settings_pack::upload_rate_limit, rate_limit / 10);
-	pack.set_int(settings_pack::download_rate_limit, rate_limit / 5);
+	pack.set_int(settings_pack::upload_rate_limit, int(rate_limit / 10));
+	pack.set_int(settings_pack::download_rate_limit, int(rate_limit / 5));
 	pack.set_int(settings_pack::unchoke_slots_limit, 0);
 	pack.set_int(settings_pack::choking_algorithm, settings_pack::fixed_slots_choker);
 	pack.set_str(settings_pack::listen_interfaces, "0.0.0.0:49010");
@@ -99,11 +103,11 @@ void test_swarm()
 	torrent_handle tor2;
 	torrent_handle tor3;
 
-	boost::tie(tor1, tor2, tor3) = setup_transfer(&ses1, &ses2, &ses3, true, false, true, "_unchoke");
+	std::tie(tor1, tor2, tor3) = setup_transfer(&ses1, &ses2, &ses3, true, false, true, "_unchoke");
 
-	std::map<std::string, boost::int64_t> cnt = get_counters(ses1);
+	std::map<std::string, std::int64_t> cnt = get_counters(ses1);
 
-	fprintf(stderr, "allowed_upload_slots: %d\n", int(cnt["ses.num_unchoke_slots"]));
+	std::printf("allowed_upload_slots: %d\n", int(cnt["ses.num_unchoke_slots"]));
 	TEST_EQUAL(cnt["ses.num_unchoke_slots"], 1);
 	for (int i = 0; i < 200; ++i)
 	{
@@ -112,7 +116,7 @@ void test_swarm()
 		print_alerts(ses3, "ses3");
 
 		cnt = get_counters(ses1);
-		fprintf(stderr, "allowed unchoked: %d\n", int(cnt["ses.num_unchoke_slots"]));
+		std::printf("allowed unchoked: %d\n", int(cnt["ses.num_unchoke_slots"]));
 		if (cnt["ses.num_unchoke_slots"] >= 2) break;
 
 		torrent_status st1 = tor1.status();
@@ -121,7 +125,7 @@ void test_swarm()
 
 		print_ses_rate(i / 10.f, &st1, &st2, &st3);
 
-		test_sleep(100);
+		std::this_thread::sleep_for(lt::milliseconds(100));
 	}
 
 	TEST_CHECK(cnt["ses.num_unchoke_slots"] >= 2);
@@ -137,9 +141,11 @@ void test_swarm()
 	p3 = ses3.abort();
 }
 
+} // anonymous namespace
+
 TORRENT_TEST(auto_unchoke)
 {
-	using namespace libtorrent;
+	using namespace lt;
 
 	// in case the previous run was terminated
 	error_code ec;
@@ -157,4 +163,3 @@ TORRENT_TEST(auto_unchoke)
 	remove_all("./tmp2_unchoke", ec);
 	remove_all("./tmp3_unchoke", ec);
 }
-
