@@ -1,6 +1,7 @@
 /*
 
-Copyright (c) 2015, Arvid Norberg
+Copyright (c) 2015-2019, Arvid Norberg
+Copyright (c) 2018-2019, Alden Torres
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -52,7 +53,7 @@ TORRENT_TEST(init)
 	fs.add_file("torrent/6", 100000);
 	fs.add_file("torrent/7", 30);
 	fs.set_piece_length(piece_size);
-	fs.set_num_pieces((int(fs.total_size()) + piece_size - 1) / piece_size);
+	fs.set_num_pieces(aux::calc_num_pieces(fs));
 
 	for (auto const idx : fs.piece_range())
 	{
@@ -83,7 +84,7 @@ TORRENT_TEST(init2)
 	fs.add_file("torrent/1", 100000);
 	fs.add_file("torrent/2", 10);
 	fs.set_piece_length(piece_size);
-	fs.set_num_pieces((int(fs.total_size()) + piece_size - 1) / piece_size);
+	fs.set_num_pieces(aux::calc_num_pieces(fs));
 
 	for (auto const idx : fs.piece_range())
 	{
@@ -113,7 +114,7 @@ TORRENT_TEST(update_simple_sequential)
 	fs.add_file("torrent/2", 100);
 	fs.add_file("torrent/3", 45000);
 	fs.set_piece_length(piece_size);
-	fs.set_num_pieces((int(fs.total_size()) + piece_size - 1) / piece_size);
+	fs.set_num_pieces(aux::calc_num_pieces(fs));
 
 	piece_picker picker(4, fs.total_size() % 4, fs.num_pieces());
 
