@@ -1,6 +1,8 @@
 /*
 
-Copyright (c) 2013, Arvid Norberg
+Copyright (c) 2013, 2015-2017, 2019, Arvid Norberg
+Copyright (c) 2017, 2019, Steven Siloti
+Copyright (c) 2018, Alden Torres
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -33,6 +35,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <iostream>
 #include "test.hpp"
 #include "setup_transfer.hpp"
+#include "test_utils.hpp"
 
 #include "libtorrent/file_storage.hpp"
 #include "libtorrent/aux_/path.hpp"
@@ -49,7 +52,7 @@ void setup_test_storage(file_storage& st)
 	st.add_file(combine_path("test", combine_path("c", "b")), 40000);
 
 	st.set_piece_length(0x4000);
-	st.set_num_pieces((int(st.total_size()) + st.piece_length() - 1) / 0x4000);
+	st.set_num_pieces(aux::calc_num_pieces(st));
 
 	TEST_EQUAL(st.file_name(file_index_t{0}), "a");
 	TEST_EQUAL(st.file_name(file_index_t{1}), "b");
@@ -78,36 +81,40 @@ void setup_test_storage(file_storage& st)
 	TEST_EQUAL(st.num_pieces(), (100000 + 0x3fff) / 0x4000);
 }
 
+constexpr aux::path_index_t operator""_path(unsigned long long i)
+{ return aux::path_index_t(static_cast<std::uint32_t>(i)); }
+
 } // anonymous namespace
 
 TORRENT_TEST(coalesce_path)
 {
 	file_storage st;
+	st.set_piece_length(0x4000);
 	st.add_file(combine_path("test", "a"), 10000);
 	TEST_EQUAL(st.paths().size(), 1);
-	TEST_EQUAL(st.paths()[0], "");
+	TEST_EQUAL(st.paths()[0_path], "");
 	st.add_file(combine_path("test", "b"), 20000);
 	TEST_EQUAL(st.paths().size(), 1);
-	TEST_EQUAL(st.paths()[0], "");
+	TEST_EQUAL(st.paths()[0_path], "");
 	st.add_file(combine_path("test", combine_path("c", "a")), 30000);
 	TEST_EQUAL(st.paths().size(), 2);
-	TEST_EQUAL(st.paths()[0], "");
-	TEST_EQUAL(st.paths()[1], "c");
+	TEST_EQUAL(st.paths()[0_path], "");
+	TEST_EQUAL(st.paths()[1_path], "c");
 
 	// make sure that two files with the same path shares the path entry
 	st.add_file(combine_path("test", combine_path("c", "b")), 40000);
 	TEST_EQUAL(st.paths().size(), 2);
-	TEST_EQUAL(st.paths()[0], "");
-	TEST_EQUAL(st.paths()[1], "c");
+	TEST_EQUAL(st.paths()[0_path], "");
+	TEST_EQUAL(st.paths()[1_path], "c");
 
 	// cause pad files to be created, to make sure the pad files also share the
 	// same path entries
-	st.optimize(0, 1024, true);
+	st.canonicalize();
 
 	TEST_EQUAL(st.paths().size(), 3);
-	TEST_EQUAL(st.paths()[0], "");
-	TEST_EQUAL(st.paths()[1], "c");
-	TEST_EQUAL(st.paths()[2], ".pad");
+	TEST_EQUAL(st.paths()[0_path], "");
+	TEST_EQUAL(st.paths()[1_path], "c");
+	TEST_EQUAL(st.paths()[2_path], ".pad");
 }
 
 TORRENT_TEST(rename_file)
@@ -179,37 +186,32 @@ TORRENT_TEST(pointer_offset)
 {
 	// test applying pointer offset
 	file_storage st;
+	st.set_piece_length(16 * 1024);
 	char const filename[] = "test1fooba";
+	char const filehash[] = "01234567890123456789-----";
+	char const roothash[] = "01234567890123456789012345678912-----";
 
 	st.add_file_borrow({filename, 5}, combine_path("test-torrent-1", "test1")
-		, 10);
+		, 10, file_flags_t{}, filehash, 0, {}, roothash);
 
 	// test filename_ptr and filename_len
+#ifndef TORRENT_NO_DEPRECATE
 	TEST_EQUAL(st.file_name_ptr(file_index_t{0}), filename);
 	TEST_EQUAL(st.file_name_len(file_index_t{0}), 5);
+#endif
 	TEST_EQUAL(st.file_name(file_index_t{0}), string_view(filename, 5));
+	TEST_EQUAL(st.hash(file_index_t{0}), sha1_hash(filehash));
+	TEST_EQUAL(st.root(file_index_t{0}), sha256_hash(roothash));
 
 	TEST_EQUAL(st.file_path(file_index_t{0}, ""), combine_path("test-torrent-1", "test1"));
 	TEST_EQUAL(st.file_path(file_index_t{0}, "tmp"), combine_path("tmp"
 		, combine_path("test-torrent-1", "test1")));
-
-	// apply a pointer offset of 5 bytes. The name of the file should
-	// change to "fooba".
-
-	st.apply_pointer_offset(5);
-
-	TEST_EQUAL(st.file_path(file_index_t{0}, ""), combine_path("test-torrent-1", "fooba"));
-	TEST_EQUAL(st.file_path(file_index_t{0}, "tmp"), combine_path("tmp"
-		, combine_path("test-torrent-1", "fooba")));
-
-	// test filename_ptr and filename_len
-	TEST_EQUAL(st.file_name_ptr(file_index_t{0}), filename + 5);
-	TEST_EQUAL(st.file_name_len(file_index_t{0}), 5);
 }
 
 TORRENT_TEST(invalid_path1)
 {
 	file_storage st;
+	st.set_piece_length(16 * 1024);
 #ifdef TORRENT_WINDOWS
 	st.add_file_borrow({}, R"(+\\\()", 10);
 #else
@@ -223,6 +225,7 @@ TORRENT_TEST(invalid_path1)
 TORRENT_TEST(invalid_path2)
 {
 	file_storage st;
+	st.set_piece_length(16 * 1024);
 #ifdef TORRENT_WINDOWS
 	st.add_file_borrow({}, R"(+\\\+\\()", 10);
 #else
@@ -248,15 +251,15 @@ TORRENT_TEST(map_file)
 	// size: 4723
 
 	peer_request rq = fs.map_file(file_index_t{0}, 0, 10);
-	TEST_EQUAL(rq.piece, piece_index_t{0});
+	TEST_EQUAL(rq.piece, 0_piece);
 	TEST_EQUAL(rq.start, 0);
 	TEST_EQUAL(rq.length, 10);
 	rq = fs.map_file(file_index_t{5}, 0, 10);
-	TEST_EQUAL(rq.piece, piece_index_t{7});
+	TEST_EQUAL(rq.piece, 7_piece);
 	TEST_EQUAL(rq.start, 298);
 	TEST_EQUAL(rq.length, 10);
 	rq = fs.map_file(file_index_t{5}, 0, 1000);
-	TEST_EQUAL(rq.piece, piece_index_t{7});
+	TEST_EQUAL(rq.piece, 7_piece);
 	TEST_EQUAL(rq.start, 298);
 	TEST_EQUAL(rq.length, 841);
 }
@@ -270,103 +273,63 @@ TORRENT_TEST(file_path_hash)
 	fs.add_file(combine_path("temp_storage", "Foo"), 17);
 	fs.add_file(combine_path("temp_storage", "foo"), 612);
 
-	std::printf("path: %s\n", fs.file_path(file_index_t(0)).c_str());
-	std::printf("file: %s\n", fs.file_path(file_index_t(1)).c_str());
-	std::uint32_t file_hash0 = fs.file_path_hash(file_index_t(0), "a");
-	std::uint32_t file_hash1 = fs.file_path_hash(file_index_t(1), "a");
+	std::printf("path: %s\n", fs.file_path(0_file).c_str());
+	std::printf("file: %s\n", fs.file_path(1_file).c_str());
+	std::uint32_t file_hash0 = fs.file_path_hash(0_file, "a");
+	std::uint32_t file_hash1 = fs.file_path_hash(1_file, "a");
 	TEST_EQUAL(file_hash0, file_hash1);
 }
 
-// make sure that files whose size is a multiple by the alignment are picked
-// first, since that lets us keep placing files aligned
-TORRENT_TEST(optimize_aligned_sizes)
-{
-	file_storage fs;
-	fs.set_piece_length(512);
-	fs.add_file(combine_path("s", "1"), 1);
-	fs.add_file(combine_path("s", "2"), 40000);
-	fs.add_file(combine_path("s", "3"), 1024);
-
-	fs.optimize(512, 512, false);
-
-	// since the size of file 3 is a multiple of the alignment (512), it should
-	// be prioritized, to minimize the amount of padding.
-	// after that, we want to pick the largest file (2), and since file 1 is
-	// smaller than the pad-file limit (512) we won't pad it. Since tail_padding
-	// is false, we won't pad the tail of the torrent either
-
-	TEST_EQUAL(fs.num_files(), 3);
-
-	TEST_EQUAL(fs.file_size(file_index_t(0)), 1024);
-	TEST_EQUAL(fs.file_name(file_index_t(0)), "3");
-	TEST_EQUAL(fs.pad_file_at(file_index_t(0)), false);
-
-	TEST_EQUAL(fs.file_size(file_index_t(1)), 40000);
-	TEST_EQUAL(fs.file_name(file_index_t(1)), "2");
-	TEST_EQUAL(fs.pad_file_at(file_index_t(1)), false);
-
-	TEST_EQUAL(fs.file_size(file_index_t(2)), 1);
-	TEST_EQUAL(fs.file_name(file_index_t(2)), "1");
-	TEST_EQUAL(fs.pad_file_at(file_index_t(2)), false);
-}
-
-// make sure we pad the end of the torrent when tail_padding is specified
-TORRENT_TEST(optimize_tail_padding)
-{
-	file_storage fs;
-	fs.set_piece_length(512);
-	fs.add_file(combine_path("s", "1"), 700);
-
-	fs.optimize(512, 512, true);
-
-	// since the size of file 3 is a multiple of the alignment (512), it should
-	// be prioritized, to minimize the amount of padding.
-	// after that, we want to pick the largest file (2), and since file 1 is
-	// smaller than the pad-file limit (512) we won't pad it. Since tail_padding
-	// is false, we won't pad the tail of the torrent either
-
-	TEST_EQUAL(fs.num_files(), 2);
-
-	TEST_EQUAL(fs.file_size(file_index_t(0)), 700);
-	TEST_EQUAL(fs.file_name(file_index_t(0)), "1");
-	TEST_EQUAL(fs.pad_file_at(file_index_t(0)), false);
-
-	TEST_EQUAL(fs.file_size(file_index_t(1)), 1024 - 700);
-	TEST_EQUAL(fs.pad_file_at(file_index_t(1)), true);
-}
-
-
 // make sure we fill in padding with small files
-TORRENT_TEST(optimize_pad_fillers)
+TORRENT_TEST(canonicalize_pad)
 {
 	file_storage fs;
-	fs.set_piece_length(512);
+	fs.set_piece_length(0x4000);
+	fs.add_file(combine_path("s", "2"), 0x7000);
 	fs.add_file(combine_path("s", "1"), 1);
-	fs.add_file(combine_path("s", "2"), 1000);
-	fs.add_file(combine_path("s", "3"), 1001);
+	fs.add_file(combine_path("s", "3"), 0x7001);
 
-	fs.optimize(512, 512, false);
+	fs.canonicalize();
 
-	// first we pick the largest file, then we need to add padding, since file 1
-	// is smaller than the pad file limit, it won't be aligned anyway, so we
-	// place that as part of the padding
+	TEST_EQUAL(fs.num_files(), 5);
+
+	TEST_EQUAL(fs.file_size(0_file), 1);
+	TEST_EQUAL(fs.file_name(0_file), "1");
+	TEST_EQUAL(fs.pad_file_at(0_file), false);
+
+	TEST_EQUAL(fs.file_size(1_file), 0x4000 - 1);
+	TEST_EQUAL(fs.pad_file_at(1_file), true);
+
+	TEST_EQUAL(fs.file_size(2_file), 0x7000);
+	TEST_EQUAL(fs.file_name(2_file), "2");
+	TEST_EQUAL(fs.pad_file_at(2_file), false);
+
+	TEST_EQUAL(fs.file_size(3_file), 0x8000 - 0x7000);
+	TEST_EQUAL(fs.pad_file_at(3_file), true);
+
+	TEST_EQUAL(fs.file_size(4_file), 0x7001);
+	TEST_EQUAL(fs.file_name(4_file), "3");
+	TEST_EQUAL(fs.pad_file_at(4_file), false);
+}
+
+// make sure canonicalize sorts by path correctly
+TORRENT_TEST(canonicalize_path)
+{
+	file_storage fs;
+	fs.set_piece_length(0x4000);
+	fs.add_file(combine_path("b", combine_path("2", "a")), 0x4000);
+	fs.add_file(combine_path("b", combine_path("1", "a")), 0x4000);
+	fs.add_file(combine_path("b", combine_path("3", "a")), 0x4000);
+	fs.add_file(combine_path("b", "11"), 0x4000);
+
+	fs.canonicalize();
 
 	TEST_EQUAL(fs.num_files(), 4);
 
-	TEST_EQUAL(fs.file_size(file_index_t(0)), 1001);
-	TEST_EQUAL(fs.file_name(file_index_t(0)), "3");
-	TEST_EQUAL(fs.pad_file_at(file_index_t(0)), false);
-
-	TEST_EQUAL(fs.file_size(file_index_t(1)), 1);
-	TEST_EQUAL(fs.file_name(file_index_t(1)), "1");
-	TEST_EQUAL(fs.pad_file_at(file_index_t(1)), false);
-
-	TEST_EQUAL(fs.file_size(file_index_t(2)), 1024 - (1001 + 1));
-	TEST_EQUAL(fs.pad_file_at(file_index_t(2)), true);
-
-	TEST_EQUAL(fs.file_size(file_index_t(3)), 1000);
-	TEST_EQUAL(fs.file_name(file_index_t(3)), "2");
-	TEST_EQUAL(fs.pad_file_at(file_index_t(3)), false);
+	TEST_EQUAL(fs.file_path(0_file), combine_path("b", combine_path("1", "a")));
+	TEST_EQUAL(fs.file_path(1_file), combine_path("b", "11"));
+	TEST_EQUAL(fs.file_path(2_file), combine_path("b", combine_path("2", "a")));
+	TEST_EQUAL(fs.file_path(3_file), combine_path("b", combine_path("3", "a")));
 }
 
 TORRENT_TEST(piece_range_exclusive)
@@ -377,16 +340,16 @@ TORRENT_TEST(piece_range_exclusive)
 	fs.add_file(combine_path("temp_storage", "0"), piece_size);
 	fs.add_file(combine_path("temp_storage", "1"), piece_size * 4 + 1);
 	fs.add_file(combine_path("temp_storage", "2"), piece_size * 4 - 1);
-	fs.set_num_pieces(int((fs.total_size() + piece_size - 1) / piece_size));
+	fs.set_num_pieces(aux::calc_num_pieces(fs));
 	//        +---+---+---+---+---+---+---+---+---+
 	// pieces | 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |
 	//        +---+---+---+---+---+---+---+---+---+
 	// files  | 0 |        1       |        2     |
 	//        +---+----------------+--------------+
 
-	TEST_CHECK(aux::file_piece_range_exclusive(fs, file_index_t(0)) == std::make_tuple(piece_index_t(0), piece_index_t(1)));
-	TEST_CHECK(aux::file_piece_range_exclusive(fs, file_index_t(1)) == std::make_tuple(piece_index_t(1), piece_index_t(5)));
-	TEST_CHECK(aux::file_piece_range_exclusive(fs, file_index_t(2)) == std::make_tuple(piece_index_t(6), piece_index_t(9)));
+	TEST_CHECK(aux::file_piece_range_exclusive(fs, 0_file) == std::make_tuple(0_piece, 1_piece));
+	TEST_CHECK(aux::file_piece_range_exclusive(fs, 1_file) == std::make_tuple(1_piece, 5_piece));
+	TEST_CHECK(aux::file_piece_range_exclusive(fs, 2_file) == std::make_tuple(6_piece, 9_piece));
 }
 
 TORRENT_TEST(piece_range_inclusive)
@@ -397,16 +360,16 @@ TORRENT_TEST(piece_range_inclusive)
 	fs.add_file(combine_path("temp_storage", "0"), piece_size);
 	fs.add_file(combine_path("temp_storage", "1"), piece_size * 4 + 1);
 	fs.add_file(combine_path("temp_storage", "2"), piece_size * 4 - 1);
-	fs.set_num_pieces(int((fs.total_size() + piece_size - 1) / piece_size));
+	fs.set_num_pieces(aux::calc_num_pieces(fs));
 	//        +---+---+---+---+---+---+---+---+---+
 	// pieces | 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |
 	//        +---+---+---+---+---+---+---+---+---+
 	// files  | 0 |        1       |        2     |
 	//        +---+----------------+--------------+
 
-	TEST_CHECK(aux::file_piece_range_inclusive(fs, file_index_t(0)) == std::make_tuple(piece_index_t(0), piece_index_t(1)));
-	TEST_CHECK(aux::file_piece_range_inclusive(fs, file_index_t(1)) == std::make_tuple(piece_index_t(1), piece_index_t(6)));
-	TEST_CHECK(aux::file_piece_range_inclusive(fs, file_index_t(2)) == std::make_tuple(piece_index_t(5), piece_index_t(9)));
+	TEST_CHECK(aux::file_piece_range_inclusive(fs, 0_file) == std::make_tuple(0_piece, 1_piece));
+	TEST_CHECK(aux::file_piece_range_inclusive(fs, 1_file) == std::make_tuple(1_piece, 6_piece));
+	TEST_CHECK(aux::file_piece_range_inclusive(fs, 2_file) == std::make_tuple(5_piece, 9_piece));
 }
 
 TORRENT_TEST(piece_range)
@@ -416,126 +379,18 @@ TORRENT_TEST(piece_range)
 	fs.set_piece_length(piece_size);
 	fs.add_file(combine_path("temp_storage", "0"), piece_size * 3);
 	fs.add_file(combine_path("temp_storage", "1"), piece_size * 3 + 0x30);
-	fs.set_num_pieces(int((fs.total_size() + piece_size - 1) / piece_size));
+	fs.set_num_pieces(aux::calc_num_pieces(fs));
 	//        +---+---+---+---+---+---+---+
 	// pieces | 0 | 1 | 2 | 3 | 4 | 5 | 6 |
 	//        +---+---+---+---+---+---+---+
 	// files  |      0    |      1     |
 	//        +---+-------+------------+
 
-	TEST_CHECK(aux::file_piece_range_inclusive(fs, file_index_t(0)) == std::make_tuple(piece_index_t(0), piece_index_t(3)));
-	TEST_CHECK(aux::file_piece_range_inclusive(fs, file_index_t(1)) == std::make_tuple(piece_index_t(3), piece_index_t(7)));
+	TEST_CHECK(aux::file_piece_range_inclusive(fs, 0_file) == std::make_tuple(0_piece, 3_piece));
+	TEST_CHECK(aux::file_piece_range_inclusive(fs, 1_file) == std::make_tuple(3_piece, 7_piece));
 
-	TEST_CHECK(aux::file_piece_range_exclusive(fs, file_index_t(0)) == std::make_tuple(piece_index_t(0), piece_index_t(3)));
-	TEST_CHECK(aux::file_piece_range_exclusive(fs, file_index_t(1)) == std::make_tuple(piece_index_t(3), piece_index_t(7)));
-}
-
-namespace {
-
-void test_optimize(std::vector<int> file_sizes
-	, int const alignment
-	, int const pad_file_limit
-	, bool const tail_padding
-	, std::vector<int> const expected_order)
-{
-	file_storage fs;
-	int i = 0;
-	for (int s : file_sizes)
-	{
-		fs.add_file(combine_path("test", std::to_string(i++)), s);
-	}
-	fs.optimize(pad_file_limit, alignment, tail_padding);
-
-	TEST_EQUAL(fs.num_files(), int(expected_order.size()));
-	if (fs.num_files() != int(expected_order.size())) return;
-
-	std::cout << "{ ";
-	for (auto const idx : fs.file_range())
-	{
-		if (fs.file_flags(idx) & file_storage::flag_pad_file) std::cout << "*";
-		std::cout << fs.file_size(idx) << " ";
-	}
-	std::cout << "}\n";
-
-	file_index_t idx{0};
-	int num_pad_files = 0;
-	for (int expect : expected_order)
-	{
-		if (expect == -1)
-		{
-			TEST_CHECK(fs.file_flags(idx) & file_storage::flag_pad_file);
-			TEST_EQUAL(fs.file_name(idx), std::to_string(num_pad_files++));
-		}
-		else
-		{
-			TEST_EQUAL(fs.file_name(idx), std::to_string(expect));
-			TEST_EQUAL(fs.file_size(idx), file_sizes[std::size_t(expect)]);
-		}
-		++idx;
-	}
-}
-
-} // anonymous namespace
-
-TORRENT_TEST(optimize_order_large_first)
-{
-	test_optimize({1000, 3000, 10000}, 1024, 1024, false, {2, -1, 1, 0});
-}
-
-TORRENT_TEST(optimize_tail_padding2)
-{
-	// when tail padding is enabled, a pad file is added at the end
-	test_optimize({2000}, 1024, 1024, true, {0, -1});
-}
-
-TORRENT_TEST(optimize_tail_padding3)
-{
-	// when tail padding is enabled, a pad file is added at the end, even if the
-	// file is smaller than the alignment, as long as pad_file_limit is 0 *(which
-	// means files are aligned unconditionally)
-	test_optimize({1000}, 1024, 0, true, {0, -1});
-}
-
-TORRENT_TEST(optimize_tail_padding_small_files)
-{
-	// files smaller than the pad file limit are not tail-padded
-	test_optimize({1000, 1, 2}, 1024, 50, true, {0, -1, 2, 1});
-}
-
-TORRENT_TEST(optimize_tail_padding_small_files2)
-{
-	// files larger than the pad file limit are not tail-padded
-	test_optimize({1000, 1, 2}, 1024, 0, true, {0, -1, 2, -1, 1, -1});
-}
-
-TORRENT_TEST(optimize_prioritize_aligned_size)
-{
-	// file 0 of size 1024 will be chosen over the larger file, since it won't
-	// affect the alignment of the next file
-	test_optimize({1024, 3000, 10}, 1024, 1024, false, {0, 1, 2});
-}
-
-TORRENT_TEST(optimize_fill_with_small_files)
-{
-	// fill in space that otherwise would just be a pad file with other small
-	// files.
-	test_optimize({2000, 5000, 48, 120}, 1024, 1024, false, {1, 3, 0, 2});
-}
-
-TORRENT_TEST(optimize_pad_all)
-{
-	// when pad_size_limit is 0, every file is padded to alignment, regardless of
-	// how big it is
-	// the empty file is first, since it doesn't affect alignment of the next
-	// file
-	test_optimize({48, 1, 0, 5000}, 1024, 0, false, {2, 3, -1, 0, -1, 1});
-}
-
-TORRENT_TEST(optimize_pad_all_with_tail)
-{
-	// when pad_size_limit is 0, every file is padded to alignment, regardless of
-	// how big it is, also with tail-padding enabled
-	test_optimize({48, 1, 0, 5000}, 1024, 0, true, {2, 3, -1, 0, -1, 1, -1});
+	TEST_CHECK(aux::file_piece_range_exclusive(fs, 0_file) == std::make_tuple(0_piece, 3_piece));
+	TEST_CHECK(aux::file_piece_range_exclusive(fs, 1_file) == std::make_tuple(3_piece, 7_piece));
 }
 
 TORRENT_TEST(piece_size_last_piece)
@@ -543,8 +398,8 @@ TORRENT_TEST(piece_size_last_piece)
 	file_storage fs;
 	fs.set_piece_length(1024);
 	fs.add_file("0", 100);
-	fs.set_num_pieces(int((fs.total_size() + 1023) / 1024));
-	TEST_EQUAL(fs.piece_size(piece_index_t{0}), 100);
+	fs.set_num_pieces(aux::calc_num_pieces(fs));
+	TEST_EQUAL(fs.piece_size(0_piece), 100);
 }
 
 TORRENT_TEST(piece_size_middle_piece)
@@ -552,9 +407,9 @@ TORRENT_TEST(piece_size_middle_piece)
 	file_storage fs;
 	fs.set_piece_length(1024);
 	fs.add_file("0", 2000);
-	fs.set_num_pieces(int((fs.total_size() + 1023) / 1024));
-	TEST_EQUAL(fs.piece_size(piece_index_t{0}), 1024);
-	TEST_EQUAL(fs.piece_size(piece_index_t{1}), 2000 - 1024);
+	fs.set_num_pieces(aux::calc_num_pieces(fs));
+	TEST_EQUAL(fs.piece_size(0_piece), 1024);
+	TEST_EQUAL(fs.piece_size(1_piece), 2000 - 1024);
 }
 
 TORRENT_TEST(file_index_at_offset)
@@ -582,11 +437,11 @@ TORRENT_TEST(map_block_start)
 	fs.add_file("test/2", 3);
 	fs.add_file("test/3", 4);
 	fs.add_file("test/4", 5);
-	fs.set_num_pieces(int((fs.total_size() + 1023) / 1024));
+	fs.set_num_pieces(aux::calc_num_pieces(fs));
 	int len = 0;
 	for (int f : {0, 1, 2, 2, 3, 3, 3, 4, 4, 4, 4, 5, 5, 5, 5, 5})
 	{
-		std::vector<file_slice> const map = fs.map_block(piece_index_t{0}, 0, len);
+		std::vector<file_slice> const map = fs.map_block(0_piece, 0, len);
 		TEST_EQUAL(int(map.size()), f);
 		file_index_t file_index{0};
 		std::int64_t actual_len = 0;
@@ -610,11 +465,11 @@ TORRENT_TEST(map_block_mid)
 	fs.add_file("test/2", 3);
 	fs.add_file("test/3", 4);
 	fs.add_file("test/4", 5);
-	fs.set_num_pieces(int((fs.total_size() + 1023) / 1024));
+	fs.set_num_pieces(aux::calc_num_pieces(fs));
 	int offset = 0;
 	for (int f : {0, 1, 1, 2, 2, 2, 3, 3, 3, 3, 4, 4, 4, 4, 4})
 	{
-		std::vector<file_slice> const map = fs.map_block(piece_index_t{0}, offset, 1);
+		std::vector<file_slice> const map = fs.map_block(0_piece, offset, 1);
 		TEST_EQUAL(int(map.size()), 1);
 		auto const& file = map[0];
 		TEST_EQUAL(file.file_index, file_index_t{f});
@@ -775,6 +630,428 @@ TORRENT_TEST(query_symlinks)
 	TEST_CHECK(ret3 != ret4);
 }
 
+TORRENT_TEST(query_symlinks2)
+{
+	file_storage fs;
+	fs.set_piece_length(1024);
+	fs.add_file("test/0", 10);
+	fs.add_file("test/1", 10);
+	fs.add_file("test/2", 10);
+	fs.add_file("test/3", 10);
+
+	TEST_CHECK(fs.symlink(file_index_t{0}).empty());
+	TEST_CHECK(fs.symlink(file_index_t{1}).empty());
+	TEST_CHECK(fs.symlink(file_index_t{2}).empty());
+	TEST_CHECK(fs.symlink(file_index_t{3}).empty());
+}
+
+TORRENT_TEST(files_equal)
+{
+	file_storage fs1;
+	fs1.set_piece_length(0x4000);
+	fs1.add_file("test/0", 1);
+	fs1.add_file("test/1", 2);
+
+	file_storage fs2;
+	fs2.set_piece_length(0x4000);
+	fs2.add_file("test/0", 1);
+	fs2.add_file("test/1", 2);
+
+	TEST_CHECK(lt::aux::files_equal(fs1, fs2));
+}
+
+TORRENT_TEST(files_equal_num_files)
+{
+	file_storage fs1;
+	fs1.set_piece_length(0x4000);
+	fs1.add_file("test/0", 1);
+	fs1.add_file("test/1", 2);
+
+	file_storage fs2;
+	fs2.set_piece_length(0x4000);
+	fs2.add_file("test/0", 3);
+
+	TEST_CHECK(!lt::aux::files_equal(fs1, fs2));
+}
+
+TORRENT_TEST(files_equal_size)
+{
+	file_storage fs1;
+	fs1.set_piece_length(0x4000);
+	fs1.add_file("test/0", 2);
+	fs1.add_file("test/1", 1);
+
+	file_storage fs2;
+	fs2.set_piece_length(0x4000);
+	fs2.add_file("test/0", 1);
+	fs2.add_file("test/1", 2);
+
+	TEST_CHECK(!lt::aux::files_equal(fs1, fs2));
+}
+
+TORRENT_TEST(files_equal_name)
+{
+	file_storage fs1;
+	fs1.set_piece_length(0x4000);
+	fs1.add_file("test/1", 1);
+	fs1.add_file("test/0", 2);
+
+	file_storage fs2;
+	fs2.set_piece_length(0x4000);
+	fs2.add_file("test/0", 1);
+	fs2.add_file("test/1", 2);
+
+	TEST_CHECK(!lt::aux::files_equal(fs1, fs2));
+}
+
+TORRENT_TEST(files_equal_flags)
+{
+	file_storage fs1;
+	fs1.set_piece_length(0x4000);
+	fs1.add_file("test/0", 1);
+	fs1.add_file("test/1", 2, file_storage::flag_hidden);
+
+	file_storage fs2;
+	fs2.set_piece_length(0x4000);
+	fs2.add_file("test/0", 1);
+	fs2.add_file("test/1", 2);
+
+	TEST_CHECK(!lt::aux::files_equal(fs1, fs2));
+}
+
+TORRENT_TEST(files_equal_mtime)
+{
+	file_storage fs1;
+	fs1.set_piece_length(0x4000);
+	fs1.add_file("test/0", 1, {}, 1234);
+	fs1.add_file("test/1", 2, {}, 1235);
+
+	file_storage fs2;
+	fs2.set_piece_length(0x4000);
+	fs2.add_file("test/0", 1, {}, 1234);
+	fs2.add_file("test/1", 2, {}, 1234);
+
+	TEST_CHECK(!lt::aux::files_equal(fs1, fs2));
+}
+
+TORRENT_TEST(files_equal_piece_size)
+{
+	file_storage fs1;
+	fs1.set_piece_length(0x8000);
+	fs1.add_file("test/0", 1);
+	fs1.add_file("test/1", 2);
+
+	file_storage fs2;
+	fs2.set_piece_length(0x4000);
+	fs2.add_file("test/0", 1);
+	fs2.add_file("test/1", 2);
+
+	TEST_CHECK(!lt::aux::files_equal(fs1, fs2));
+}
+
+TORRENT_TEST(files_equal_symlink)
+{
+	file_storage fs1;
+	fs1.set_piece_length(0x4000);
+	fs1.add_file("test/0", 1);
+	fs1.add_file("test/1", 2, file_storage::flag_symlink, 0, "test/0");
+
+	file_storage fs2;
+	fs2.set_piece_length(0x4000);
+	fs2.add_file("test/0", 1);
+	fs2.add_file("test/1", 2, file_storage::flag_symlink, 0, "test/1");
+
+	TEST_CHECK(!lt::aux::files_equal(fs1, fs2));
+}
+
+std::int64_t const int_max = std::numeric_limits<int>::max();
+
+TORRENT_TEST(large_files)
+{
+	file_storage fs1;
+	fs1.set_piece_length(0x4000);
+	TEST_THROW(fs1.add_file("test/0", int_max / 2 * lt::default_block_size + 1));
+
+	error_code ec;
+	fs1.add_file(ec, "test/0", int_max * lt::default_block_size + 1);
+	TEST_EQUAL(ec, make_error_code(boost::system::errc::file_too_large));
+
+	// should not throw
+	TEST_NOTHROW(fs1.add_file("test/0", int_max / 2 * lt::default_block_size));
+}
+
+TORRENT_TEST(large_offset)
+{
+	file_storage fs1;
+	fs1.set_piece_length(0x4000);
+	for (int i = 0; i < 16; ++i)
+		fs1.add_file(("test/" + std::to_string(i)).c_str(), int_max / 2 * lt::default_block_size);
+
+	// this exceeds the 2^48-1 limit
+	TEST_THROW(fs1.add_file("test/16", 262144));
+
+	error_code ec;
+	fs1.add_file(ec, "test/8", 262144);
+	TEST_EQUAL(ec, make_error_code(errors::torrent_invalid_length));
+
+	// this should be OK, but just
+	fs1.add_file("test/8", 262143);
+}
+
+TORRENT_TEST(large_filename)
+{
+	file_storage fs1;
+	fs1.set_piece_length(0x4000);
+	// yes, this creates an invalid string_view, as it claims to be larger than
+	// the allocation. This should be OK though as the test for size never
+	// actually looks at the string
+	TEST_THROW(fs1.add_file_borrow(string_view("0", 1 << 12), "test/path/", 10));
+
+	error_code ec;
+	fs1.add_file_borrow(ec, string_view("0", 1 << 12), "test/path/", 10);
+	TEST_EQUAL(ec, make_error_code(boost::system::errc::filename_too_long));
+}
+
+TORRENT_TEST(piece_size2)
+{
+	file_storage fs;
+	fs.set_piece_length(0x8000);
+	// passing in a root hash (the last argument) makes it follow v2 rules, to
+	// add pad files
+	fs.add_file("test/0", 0x5000, {}, 0, {}, "01234567890123456789012345678901");
+
+	fs.set_num_pieces(aux::calc_num_pieces(fs));
+	TEST_EQUAL(fs.num_pieces(), 1);
+	TEST_EQUAL(fs.piece_size2(0_piece), 0x5000);
+
+	fs.add_file("test/1", 0x2000, {}, 0, {}, "01234567890123456789012345678901");
+	fs.add_file("test/2", 0x8000, {}, 0, {}, "01234567890123456789012345678901");
+
+	fs.set_num_pieces(aux::calc_num_pieces(fs));
+	TEST_EQUAL(fs.num_pieces(), 3);
+	TEST_EQUAL(fs.piece_size2(2_piece), 0x8000);
+
+	fs.add_file("test/3", 8, {}, 0, {}, "01234567890123456789012345678901");
+
+	fs.set_num_pieces(aux::calc_num_pieces(fs));
+	TEST_EQUAL(fs.num_pieces(), 4);
+	TEST_EQUAL(fs.piece_size2(0_piece), 0x5000);
+	TEST_EQUAL(fs.piece_size2(1_piece), 0x2000);
+	TEST_EQUAL(fs.piece_size2(2_piece), 0x8000);
+	TEST_EQUAL(fs.piece_size2(3_piece), 8);
+
+	fs.add_file("test/4", 0x8001, {}, 0, {}, "01234567890123456789012345678901");
+
+	fs.set_num_pieces(aux::calc_num_pieces(fs));
+	TEST_EQUAL(fs.num_pieces(), 6);
+
+	TEST_EQUAL(fs.piece_size2(0_piece), 0x5000);
+	TEST_EQUAL(fs.piece_size2(1_piece), 0x2000);
+	TEST_EQUAL(fs.piece_size2(2_piece), 0x8000);
+	TEST_EQUAL(fs.piece_size2(3_piece), 8);
+	TEST_EQUAL(fs.piece_size2(4_piece), 0x8000);
+	TEST_EQUAL(fs.piece_size2(5_piece), 1);
+}
+
+TORRENT_TEST(file_num_blocks)
+{
+	file_storage fs;
+	fs.set_piece_length(0x8000);
+	fs.add_file("test/0", 0x5000, {}, 0, {}, "01234567890123456789012345678901");
+	fs.add_file("test/1", 0x2000, {}, 0, {}, "01234567890123456789012345678901");
+	fs.add_file("test/2", 0x8000, {}, 0, {}, "01234567890123456789012345678901");
+	fs.add_file("test/3", 0x8001, {}, 0, {}, "01234567890123456789012345678901");
+	fs.add_file("test/4", 1, {}, 0, {}, "01234567890123456789012345678901");
+
+	// generally the number of blocks in a file is:
+	// (file_size + lt::default_block_size - 1) / lt::default_block_size
+
+	TEST_EQUAL(fs.file_num_blocks(file_index_t{0}), 2);
+	// pad file at index 1
+	TEST_EQUAL(fs.file_num_blocks(file_index_t{2}), 1);
+	// pad file at index 3
+	TEST_EQUAL(fs.file_num_blocks(file_index_t{4}), 2);
+	TEST_EQUAL(fs.file_num_blocks(file_index_t{5}), 3);
+	// pad file at index 6
+	TEST_EQUAL(fs.file_num_blocks(file_index_t{7}), 1);
+}
+
+TORRENT_TEST(file_num_pieces)
+{
+	file_storage fs;
+	fs.set_piece_length(0x8000);
+	fs.add_file("test/0", 0x5000, {}, 0, {}, "01234567890123456789012345678901");
+	fs.add_file("test/1", 0x2000, {}, 0, {}, "01234567890123456789012345678901");
+	fs.add_file("test/2", 0x8000, {}, 0, {}, "01234567890123456789012345678901");
+	fs.add_file("test/3", 0x8001, {}, 0, {}, "01234567890123456789012345678901");
+	fs.add_file("test/4", 1, {}, 0, {}, "01234567890123456789012345678901");
+
+	// generally the number of blocks in a file is:
+	// (file_size + lt::default_block_size - 1) / lt::default_block_size
+
+	TEST_EQUAL(fs.file_num_pieces(file_index_t{0}), 1);
+	// pad file at index 1
+	TEST_EQUAL(fs.file_num_pieces(file_index_t{2}), 1);
+	// pad file at index 3
+	TEST_EQUAL(fs.file_num_pieces(file_index_t{4}), 1);
+	TEST_EQUAL(fs.file_num_pieces(file_index_t{5}), 2);
+	// pad file at index 6
+	TEST_EQUAL(fs.file_num_pieces(file_index_t{7}), 1);
+}
+
+namespace {
+int first_piece_node(int piece_size, int file_size)
+{
+	file_storage fs;
+	fs.set_piece_length(piece_size);
+	fs.add_file("test/0", file_size, {}, 0, {}, "01234567890123456789012345678901");
+	fs.set_num_pieces(aux::calc_num_pieces(fs));
+	return fs.file_first_piece_node(file_index_t{0});
+}
+
+int first_block_node(int file_size)
+{
+	file_storage fs;
+	fs.set_piece_length(0x10000);
+	fs.add_file("test/0", file_size, {}, 0, {}, "01234567890123456789012345678901");
+	fs.set_num_pieces(aux::calc_num_pieces(fs));
+	return fs.file_first_block_node(file_index_t{0});
+}
+}
+
+TORRENT_TEST(file_first_piece_node)
+{
+	// the size of the merkle tree is implied by the size of the file.
+	// 0x500000 / 0x10000 = 80 pieces
+	// a merkle tree must have a power of 2 number of leaves, so that's 128,
+	// thats 7 layers
+	TEST_EQUAL(first_piece_node(0x10000, 0x500000), 127);
+	TEST_EQUAL(first_piece_node(0x8000, 0x500000), 255);
+	TEST_EQUAL(first_piece_node(0x4000, 0x500000), 511);
+	TEST_EQUAL(first_piece_node(0x2000, 0x500000), 1023);
+	TEST_EQUAL(first_piece_node(0x1000, 0x500000), 2047);
+
+	// also test boundary cases around exact power of two file size
+	// technically piece size is not allowed to be less than 16kB
+	TEST_EQUAL(first_piece_node(0x1000, 0x7fffff), 2047);
+	TEST_EQUAL(first_piece_node(0x1000, 0x800000), 2047);
+	TEST_EQUAL(first_piece_node(0x1000, 0x800001), 4095);
+
+	TEST_EQUAL(first_piece_node(0x1000, 0x7fff), 7);
+	TEST_EQUAL(first_piece_node(0x1000, 0x8000), 7);
+	TEST_EQUAL(first_piece_node(0x1000, 0x8001), 15);
+
+	// edge case of file smaller than one block
+	TEST_EQUAL(first_piece_node(0x1000, 0x1000), 0);
+
+	// edge case of file smaller than one piece
+	TEST_EQUAL(first_piece_node(0x4000, 0x1000), 0);
+}
+
+TORRENT_TEST(file_first_block_node)
+{
+	// the full merkle tree, all the way down to blocks, does not depend on the
+	// piece size. Blocks are always 0x4000 bytes.
+
+	// there must be an even power of two number of leaves, e.g.
+	// file size 0x500000 / 0x4000 = 320 blocks -> 512 leaves
+	TEST_EQUAL(first_block_node(0x500000), 511);
+
+	// edge case of file smaller than one block
+	TEST_EQUAL(first_block_node(0x1000), 0);
+
+	// even power-of-two boundary condition
+	TEST_EQUAL(first_block_node(0x7fffff), 511);
+	TEST_EQUAL(first_block_node(0x800000), 511);
+	TEST_EQUAL(first_block_node(0x800001), 1023);
+}
+
+TORRENT_TEST(mismatching_file_hash1)
+{
+	file_storage st;
+	st.set_piece_length(0x4000);
+
+	error_code ec;
+	st.add_file(ec, combine_path("test", "a"), 10000);
+	TEST_CHECK(!ec);
+	st.add_file(ec, combine_path("test", "B"), 10000, {}, 0, {}, "abababababababababababababababab");
+	TEST_CHECK(ec);
+}
+
+TORRENT_TEST(mismatching_file_hash2)
+{
+	file_storage st;
+	st.set_piece_length(0x4000);
+
+	error_code ec;
+	st.add_file(ec, combine_path("test", "B"), 10000, {}, 0, {}, "abababababababababababababababab");
+	TEST_CHECK(!ec);
+	st.add_file(ec, combine_path("test", "a"), 10000);
+	TEST_CHECK(ec);
+}
+
+TORRENT_TEST(v2_detection_1)
+{
+	file_storage fs;
+	fs.set_piece_length(0x8000);
+	// passing in a root hash (the last argument) makes it follow v2 rules, to
+	// add pad files
+	fs.add_file("test/0", 0x5000, {}, 0, "symlink-test-1");
+	fs.add_file("test/1", 0x5000, {}, 0, "symlink-test-2");
+
+	fs.add_file("test/2", 0x2000, {}, 0, {}, "01234567890123456789012345678901");
+	// it's an error to add a v1 file to a v2 torrent
+	TEST_THROW(fs.add_file("test/3", 0x2000));
+}
+
+TORRENT_TEST(v2_detection_2)
+{
+	file_storage fs;
+	fs.set_piece_length(0x8000);
+	// passing in a root hash (the last argument) makes it follow v2 rules, to
+	// add pad files
+	fs.add_file("test/0", 0x5000, {}, 0, "symlink-test-1");
+	fs.add_file("test/1", 0x5000, {}, 0, "symlink-test-2");
+
+	fs.add_file("test/2", 0x2000);
+
+	// it's an error to add a v1 file to a v2 torrent
+	TEST_THROW(fs.add_file("test/3", 0x2000, {}, 0, {}, "01234567890123456789012345678901"));
+}
+
+TORRENT_TEST(blocks_in_piece2)
+{
+	static std::map<int, int> const piece_sizes = {
+		{0x3fff, 1},
+		{0x4000, 1},
+		{0x4001, 2},
+	};
+
+	for (auto t : piece_sizes)
+	{
+		file_storage fs;
+		fs.set_piece_length(0x8000);
+		fs.add_file("test/0", t.first, {}, 0, {}, "01234567890123456789012345678901");
+		fs.set_num_pieces(aux::calc_num_pieces(fs));
+		TEST_EQUAL(fs.blocks_in_piece2(0_piece), t.second);
+	}
+}
+
+TORRENT_TEST(file_index_for_root)
+{
+	file_storage fs;
+	fs.set_piece_length(0x8000);
+	fs.add_file("test/0", 0x8000, {}, 0, {}, "11111111111111111111111111111111");
+	fs.add_file("test/1", 0x8000, {}, 0, {}, "22222222222222222222222222222222");
+	fs.add_file("test/2", 0x8000, {}, 0, {}, "33333333333333333333333333333333");
+	fs.add_file("test/3", 0x8000, {}, 0, {}, "44444444444444444444444444444444");
+
+	TEST_EQUAL(fs.file_index_for_root(sha256_hash("11111111111111111111111111111111")), file_index_t{0});
+	TEST_EQUAL(fs.file_index_for_root(sha256_hash("22222222222222222222222222222222")), file_index_t{1});
+	TEST_EQUAL(fs.file_index_for_root(sha256_hash("33333333333333333333333333333333")), file_index_t{2});
+	TEST_EQUAL(fs.file_index_for_root(sha256_hash("44444444444444444444444444444444")), file_index_t{3});
+	TEST_EQUAL(fs.file_index_for_root(sha256_hash("55555555555555555555555555555555")), file_index_t{-1});
+}
+
 // TODO: test file attributes
 // TODO: test symlinks
-// TODO: test reorder_file (make sure internal_file_entry::swap() is used)
