@@ -1,6 +1,7 @@
 /*
 
-Copyright (c) 2013, Arvid Norberg
+Copyright (c) 2014-2019, Arvid Norberg
+Copyright (c) 2016, 2018, Alden Torres
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -170,6 +171,7 @@ torrent_state init_state()
 {
 	torrent_state st;
 	st.is_finished = false;
+	st.is_paused = false;
 	st.max_peerlist_size = 1000;
 	st.allow_multiple_connections_per_ip = false;
 	st.port = 9999;
@@ -964,38 +966,6 @@ TORRENT_TEST(new_peer_size_limit)
 		, 5);
 }
 
-TORRENT_TEST(clear_peers)
-{
-	torrent_state st = init_state();
-	st.max_peerlist_size = 5;
-	mock_torrent t(&st);
-	peer_list p(allocator);
-	t.m_p = &p;
-
-	torrent_peer* peer1 = add_peer(p, st, ep("10.0.0.1", 8080));
-	TEST_CHECK(peer1);
-	p.set_seed(peer1, true);
-
-	torrent_peer* peer2 = add_peer(p, st, ep("10.0.0.2", 8080));
-	TEST_CHECK(peer2);
-
-	torrent_peer* peer3 = add_peer(p, st, ep("10.0.0.3", 8080));
-	TEST_CHECK(peer3);
-
-	p.connect_one_peer(1, &st);
-
-	TEST_EQUAL(p.num_peers(), 3);
-	TEST_EQUAL(p.num_candidate_cache(), 2);
-	TEST_EQUAL(p.num_connect_candidates(), 3);
-	TEST_EQUAL(p.num_seeds(), 1);
-
-	p.clear();
-	TEST_EQUAL(p.num_peers(), 0);
-	TEST_EQUAL(p.num_candidate_cache(), 0);
-	TEST_EQUAL(p.num_connect_candidates(), 0);
-	TEST_EQUAL(p.num_seeds(), 0);
-}
-
 // TODO: test erasing peers
 // TODO: test update_peer_port with allow_multiple_connections_per_ip and without
 // TODO: test add i2p peers
