@@ -1,6 +1,9 @@
 /*
 
-Copyright (c) 2008-2013, Arvid Norberg
+Copyright (c) 2018, Steven Siloti
+Copyright (c) 2013-2020, Arvid Norberg
+Copyright (c) 2016-2018, Alden Torres
+Copyright (c) 2018, d-komarov
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -32,6 +35,7 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include "libtorrent/session.hpp"
 #include "libtorrent/session_settings.hpp"
+#include "libtorrent/session_params.hpp"
 #include "libtorrent/alert_types.hpp"
 #include "libtorrent/bencode.hpp"
 #include "libtorrent/time.hpp"
@@ -47,7 +51,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "test_utils.hpp"
 #include "setup_transfer.hpp"
 #include "settings.hpp"
-#include <fstream>
 #include <iostream>
 
 using namespace lt;
@@ -121,7 +124,7 @@ void test_transfer(settings_pack const& sett, bool test_deprecated = false)
 
 	error_code ec;
 	create_directory("tmp1_priority", ec);
-	std::ofstream file("tmp1_priority/temporary");
+	ofstream file("tmp1_priority/temporary");
 	std::shared_ptr<torrent_info> t = ::create_torrent(&file, "temporary", 16 * 1024, 13, false);
 	file.close();
 
@@ -423,7 +426,7 @@ TORRENT_TEST(no_metadata_prioritize_files)
 	add_torrent_params addp;
 	addp.flags &= ~torrent_flags::paused;
 	addp.flags &= ~torrent_flags::auto_managed;
-	addp.info_hash = sha1_hash("abababababababababab");
+	addp.info_hashes.v1 = sha1_hash("abababababababababab");
 	addp.save_path = ".";
 	torrent_handle h = ses.add_torrent(addp);
 
@@ -450,17 +453,17 @@ TORRENT_TEST(no_metadata_file_prio)
 	add_torrent_params addp;
 	addp.flags &= ~torrent_flags::paused;
 	addp.flags &= ~torrent_flags::auto_managed;
-	addp.info_hash = sha1_hash("abababababababababab");
+	addp.info_hashes.v1 = sha1_hash("abababababababababab");
 	addp.save_path = ".";
 	torrent_handle h = ses.add_torrent(addp);
 
-	h.file_priority(file_index_t(0), 0_pri);
+	h.file_priority(0_file, 0_pri);
 	// TODO 2: this should wait for an alert instead of just sleeping
 	std::this_thread::sleep_for(lt::milliseconds(100));
-	TEST_EQUAL(h.file_priority(file_index_t(0)), 0_pri);
-	h.file_priority(file_index_t(0), 1_pri);
+	TEST_EQUAL(h.file_priority(0_file), 0_pri);
+	h.file_priority(0_file, 1_pri);
 	std::this_thread::sleep_for(lt::milliseconds(100));
-	TEST_EQUAL(h.file_priority(file_index_t(0)), 1_pri);
+	TEST_EQUAL(h.file_priority(0_file), 1_pri);
 
 	ses.remove_torrent(h);
 }
@@ -472,15 +475,15 @@ TORRENT_TEST(no_metadata_piece_prio)
 	add_torrent_params addp;
 	addp.flags &= ~torrent_flags::paused;
 	addp.flags &= ~torrent_flags::auto_managed;
-	addp.info_hash = sha1_hash("abababababababababab");
+	addp.info_hashes.v1 = sha1_hash("abababababababababab");
 	addp.save_path = ".";
 	torrent_handle h = ses.add_torrent(addp);
 
 	// you can't set piece priorities before the metadata has been downloaded
-	h.piece_priority(piece_index_t(2), 0_pri);
-	TEST_EQUAL(h.piece_priority(piece_index_t(2)), 4_pri);
-	h.piece_priority(piece_index_t(2), 1_pri);
-	TEST_EQUAL(h.piece_priority(piece_index_t(2)), 4_pri);
+	h.piece_priority(2_piece, 0_pri);
+	TEST_EQUAL(h.piece_priority(2_piece), 4_pri);
+	h.piece_priority(2_piece, 1_pri);
+	TEST_EQUAL(h.piece_priority(2_piece), 4_pri);
 
 	ses.remove_torrent(h);
 }
@@ -520,7 +523,7 @@ TORRENT_TEST(export_file_while_seed)
 
 	error_code ec;
 	create_directory("tmp2_priority", ec);
-	std::ofstream file("tmp2_priority/temporary");
+	ofstream file("tmp2_priority/temporary");
 	auto t = ::create_torrent(&file, "temporary", 16 * 1024, 13, false);
 	file.close();
 
@@ -558,8 +561,9 @@ TORRENT_TEST(export_file_while_seed)
 
 	TEST_CHECK(!exists("temporary"));
 
-	for (int i = 0; i < 10; ++i)
+	for (int i = 0; i < 40; ++i)
 	{
+		print_alerts(ses, "ses", true, true, &on_alert);
 		if (h.status().is_seeding) break;
 		std::this_thread::sleep_for(lt::milliseconds(100));
 	}
@@ -568,8 +572,9 @@ TORRENT_TEST(export_file_while_seed)
 	// this should cause the file to be exported
 	h.file_priority(file_index_t{0}, lt::low_priority);
 
-	for (int i = 0; i < 10; ++i)
+	for (int i = 0; i < 20; ++i)
 	{
+		print_alerts(ses, "ses", true, true, &on_alert);
 		if (h.file_priority(file_index_t{0}) == lt::low_priority) break;
 		std::this_thread::sleep_for(lt::milliseconds(100));
 	}
@@ -593,11 +598,11 @@ TORRENT_TEST(test_piece_priority_after_resume)
 		lt::session ses(settings());
 		torrent_handle h = ses.add_torrent(p);
 
-		TEST_EQUAL(h.piece_priority(piece_index_t{0}), prio);
+		TEST_EQUAL(h.piece_priority(0_piece), prio);
 
 		using prio_vec = std::vector<std::pair<lt::piece_index_t, lt::download_priority_t>>;
-		h.prioritize_pieces(prio_vec{{piece_index_t{0}, new_prio}});
-		TEST_EQUAL(h.piece_priority(piece_index_t{0}), new_prio);
+		h.prioritize_pieces(prio_vec{{0_piece, new_prio}});
+		TEST_EQUAL(h.piece_priority(0_piece), new_prio);
 
 		ses.pause();
 		h.save_resume_data();
@@ -614,7 +619,7 @@ TORRENT_TEST(test_piece_priority_after_resume)
 		lt::session ses(settings());
 		torrent_handle h = ses.add_torrent(p);
 
-		TEST_EQUAL(h.piece_priority(piece_index_t{0}), new_prio);
+		TEST_EQUAL(h.piece_priority(0_piece), new_prio);
 	}
 }
 
