@@ -1,6 +1,8 @@
 /*
 
-Copyright (c) 2013, Arvid Norberg
+Copyright (c) 2013, 2015-2020, Arvid Norberg
+Copyright (c) 2017, Steven Siloti
+Copyright (c) 2018, Alden Torres
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -31,7 +33,9 @@ POSSIBILITY OF SUCH DAMAGE.
 */
 
 #include "libtorrent/session.hpp"
+#include "libtorrent/session_params.hpp"
 #include "test.hpp"
+#include "test_utils.hpp"
 #include "setup_transfer.hpp"
 #include "settings.hpp"
 #include "libtorrent/create_torrent.hpp"
@@ -69,7 +73,6 @@ void test_read_piece(int flags)
 		, ec.value(), ec.message().c_str());
 
 	file_storage fs;
-	std::srand(10);
 	int piece_size = 0x4000;
 
 	static std::array<const int, 2> const file_sizes{{ 100000, 10000 }};
@@ -77,7 +80,7 @@ void test_read_piece(int flags)
 	create_random_files(combine_path("tmp1_read_piece", "test_torrent"), file_sizes);
 
 	add_files(fs, combine_path("tmp1_read_piece", "test_torrent"));
-	lt::create_torrent t(fs, piece_size, 0x4000);
+	lt::create_torrent t(fs, piece_size);
 
 	// calculate the hash for all pieces
 	set_piece_hashes(t, "tmp1_read_piece", ec);
@@ -89,7 +92,7 @@ void test_read_piece(int flags)
 	auto ti = std::make_shared<torrent_info>(buf, ec, from_span);
 
 	std::printf("generated torrent: %s tmp1_read_piece/test_torrent\n"
-		, aux::to_hex(ti->info_hash()).c_str());
+		, aux::to_hex(ti->info_hashes().v1).c_str());
 
 	settings_pack sett = settings();
 	sett.set_str(settings_pack::listen_interfaces, "0.0.0.0:48000");
@@ -115,11 +118,11 @@ void test_read_piece(int flags)
 
 	if (flags & time_critical)
 	{
-		tor1.set_piece_deadline(piece_index_t(1), 0, torrent_handle::alert_when_available);
+		tor1.set_piece_deadline(1_piece, 0, torrent_handle::alert_when_available);
 	}
 	else
 	{
-		tor1.read_piece(piece_index_t(1));
+		tor1.read_piece(1_piece);
 	}
 
 	a = wait_for_alert(ses, read_piece_alert::alert_type, "ses");
@@ -131,7 +134,7 @@ void test_read_piece(int flags)
 		TEST_CHECK(rp);
 		if (rp)
 		{
-			TEST_EQUAL(rp->piece, piece_index_t(1));
+			TEST_EQUAL(rp->piece, 1_piece);
 		}
 	}
 
