@@ -1,6 +1,9 @@
 /*
 
-Copyright (c) 2014, Arvid Norberg
+Copyright (c) 2014-2020, Arvid Norberg
+Copyright (c) 2016, 2018, Alden Torres
+Copyright (c) 2017, Steven Siloti
+Copyright (c) 2020, AllSeeingEyeTolledEweSew
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -31,6 +34,7 @@ POSSIBILITY OF SUCH DAMAGE.
 */
 
 #include "libtorrent/session.hpp"
+#include "libtorrent/session_params.hpp"
 #include "libtorrent/add_torrent_params.hpp"
 #include "libtorrent/torrent_info.hpp"
 #include "libtorrent/random.hpp"
@@ -46,15 +50,12 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "setup_transfer.hpp"
 #include "test_utils.hpp"
 
-#include <iostream>
 #include "test.hpp"
 #include "test_utils.hpp"
 #include "settings.hpp"
 #include "setup_transfer.hpp"
 #include "settings.hpp"
 
-#include <fstream>
-
 #ifdef TORRENT_WINDOWS
 #define SEP "\\"
 #else
@@ -81,7 +82,7 @@ std::vector<char> generate_resume_data(torrent_info* ti
 
 	rd["file-format"] = "libtorrent resume file";
 	rd["file-version"] = 1;
-	rd["info-hash"] = ti->info_hash().to_string();
+	rd["info-hash"] = ti->info_hashes().v1.to_string();
 	rd["blocks per piece"] = std::max(1, ti->piece_length() / 0x4000);
 	rd["pieces"] = std::string(std::size_t(ti->num_pieces()), '\x01');
 
@@ -97,10 +98,9 @@ std::vector<char> generate_resume_data(torrent_info* ti
 	rd["super_seeding"] = 0;
 	rd["added_time"] = 1347;
 	rd["completed_time"] = 1348;
-	rd["last_download"] = time(nullptr) - 1350;
-	rd["last_upload"] = time(nullptr) - 1351;
+	rd["last_download"] = 2;
+	rd["last_upload"] = 3;
 	rd["finished_time"] = 1352;
-	rd["last_seen_complete"] = 1353;
 	if (file_priorities && file_priorities[0])
 	{
 		entry::list_type& file_prio = rd["file_priority"].list();
@@ -192,12 +192,13 @@ torrent_handle test_resume_flags(lt::session& ses
 
 	torrent_handle h = ses.add_torrent(p);
 	torrent_status s = h.status();
-	TEST_EQUAL(s.info_hash, ti->info_hash());
+	TEST_EQUAL(s.info_hashes, ti->info_hashes());
 	return h;
 }
 
-void default_tests(torrent_status const& s)
+void default_tests(torrent_status const& s, lt::time_point const time_now)
 {
+	TORRENT_UNUSED(time_now);
 	// allow some slack in the time stamps since they are reported as
 	// relative times. If the computer is busy while running the unit test
 	// or running under valgrind it may take several seconds
@@ -205,20 +206,16 @@ void default_tests(torrent_status const& s)
 	TEST_CHECK(s.active_time >= 1339);
 	TEST_CHECK(s.active_time < 1339 + 10);
 
-	std::cout << "time since download: " << s.time_since_download << std::endl;
-	TEST_CHECK(s.time_since_download >= 1350 - 3);
-	TEST_CHECK(s.time_since_download <= 1350 + 2);
+	auto const now = duration_cast<seconds>(time_now.time_since_epoch()).count();
+	TEST_CHECK(s.time_since_download >= now - 2);
+	TEST_CHECK(s.time_since_upload >= now - 3);
 
-	std::cout << "time since upload: " << s.time_since_upload << std::endl;
-	TEST_CHECK(s.time_since_upload >= 1351 - 3);
-	TEST_CHECK(s.time_since_upload <= 1351 + 2);
-
-	TEST_CHECK(s.finished_time < 1352 + 2);
-	TEST_CHECK(s.finished_time >= 1352);
+	TEST_CHECK(s.time_since_download < now - 2 + 10);
+	TEST_CHECK(s.time_since_upload < now - 3 + 10);
 #endif
 
 	using lt::seconds;
-	TEST_CHECK(s.finished_duration < seconds(1352 + 2));
+	TEST_CHECK(s.finished_duration< seconds(1352 + 2));
 	TEST_CHECK(s.seeding_duration < seconds(1340 + 2));
 	TEST_CHECK(s.active_duration >= seconds(1339));
 	TEST_CHECK(s.active_duration < seconds(1339 + 10));
@@ -227,18 +224,6 @@ void default_tests(torrent_status const& s)
 	TEST_CHECK(s.added_time >= 1347);
 	TEST_CHECK(s.completed_time < 1348 + 2);
 	TEST_CHECK(s.completed_time >= 1348);
-
-	auto const now = lt::clock_type::now();
-	std::cout << "now: " << now.time_since_epoch().count() << std::endl;
-	std::cout << "last_download: " << s.last_download.time_since_epoch().count() << std::endl;
-	TEST_CHECK(s.last_download <= now - lt::seconds(1350 - 3));
-	TEST_CHECK(s.last_download >= now - lt::seconds(1350 + 2));
-
-	std::cout << "last_upload: " << s.last_upload.time_since_epoch().count() << std::endl;
-	TEST_CHECK(s.last_upload <= now - lt::seconds(1351 - 3));
-	TEST_CHECK(s.last_upload >= now - lt::seconds(1351 + 2));
-
-	TEST_EQUAL(s.last_seen_complete, 1353);
 }
 
 void test_piece_priorities(bool test_deprecated = false)
@@ -250,8 +235,8 @@ void test_piece_priorities(bool test_deprecated = false)
 	p.save_path = ".";
 	torrent_handle h = ses.add_torrent(p);
 
-	h.piece_priority(piece_index_t(0), 0_pri);
-	h.piece_priority(piece_index_t(ti->num_pieces()-1), 0_pri);
+	h.piece_priority(0_piece, 0_pri);
+	h.piece_priority(ti->last_piece(), 0_pri);
 
 	h.save_resume_data();
 	alert const* a = wait_for_alert(ses, save_resume_data_alert::alert_type);
@@ -288,9 +273,9 @@ void test_piece_priorities(bool test_deprecated = false)
 	// now, make sure the piece priorities are loaded correctly
 	h = ses.add_torrent(p);
 
-	TEST_EQUAL(h.piece_priority(piece_index_t(0)), 0_pri);
-	TEST_EQUAL(h.piece_priority(piece_index_t(1)), 4_pri);
-	TEST_EQUAL(h.piece_priority(piece_index_t(ti->num_pieces()-1)), 0_pri);
+	TEST_EQUAL(h.piece_priority(0_piece), 0_pri);
+	TEST_EQUAL(h.piece_priority(1_piece), 4_pri);
+	TEST_EQUAL(h.piece_priority(ti->last_piece()), 0_pri);
 }
 
 } // anonymous namespace
@@ -454,9 +439,9 @@ TORRENT_TEST(piece_slots)
 	{
 		std::vector<char> a(128 * 1024 * 8);
 		std::vector<char> b(128 * 1024);
-		std::ofstream("add_torrent_params_test" SEP "test_resume" SEP "tmp1").write(a.data(), std::streamsize(a.size()));
-		std::ofstream("add_torrent_params_test" SEP "test_resume" SEP "tmp2").write(b.data(), std::streamsize(b.size()));
-		std::ofstream("add_torrent_params_test" SEP "test_resume" SEP "tmp3").write(b.data(), std::streamsize(b.size()));
+		ofstream("add_torrent_params_test" SEP "test_resume" SEP "tmp1").write(a.data(), std::streamsize(a.size()));
+		ofstream("add_torrent_params_test" SEP "test_resume" SEP "tmp2").write(b.data(), std::streamsize(b.size()));
+		ofstream("add_torrent_params_test" SEP "test_resume" SEP "tmp3").write(b.data(), std::streamsize(b.size()));
 	}
 
 	add_torrent_params p;
@@ -464,8 +449,8 @@ TORRENT_TEST(piece_slots)
 	p.save_path = "add_torrent_params_test";
 
 	p.have_pieces.resize(2);
-	p.have_pieces.set_bit(piece_index_t{0});
-	p.have_pieces.set_bit(piece_index_t{1});
+	p.have_pieces.set_bit(0_piece);
+	p.have_pieces.set_bit(1_piece);
 
 	lt::session ses(settings());
 	torrent_handle h = ses.add_torrent(p);
@@ -474,13 +459,13 @@ TORRENT_TEST(piece_slots)
 
 	torrent_status s = h.status();
 	print_alerts(ses, "ses");
-	TEST_EQUAL(s.info_hash, ti->info_hash());
+	TEST_EQUAL(s.info_hashes, ti->info_hashes());
 	TEST_EQUAL(s.pieces.size(), ti->num_pieces());
 	TEST_CHECK(s.pieces.size() >= 4);
-	TEST_EQUAL(s.pieces[piece_index_t{0}], true);
-	TEST_EQUAL(s.pieces[piece_index_t{1}], true);
-	TEST_EQUAL(s.pieces[piece_index_t{2}], false);
-	TEST_EQUAL(s.pieces[piece_index_t{3}], false);
+	TEST_EQUAL(s.pieces[0_piece], true);
+	TEST_EQUAL(s.pieces[1_piece], true);
+	TEST_EQUAL(s.pieces[2_piece], false);
+	TEST_EQUAL(s.pieces[3_piece], false);
 
 	// now save resume data and make sure the pieces are preserved correctly
 	h.save_resume_data();
@@ -494,10 +479,10 @@ TORRENT_TEST(piece_slots)
 		auto const& pieces = ra->params.have_pieces;
 		TEST_EQUAL(int(pieces.size()), ti->num_pieces());
 
-		TEST_EQUAL(pieces[piece_index_t{0}], true);
-		TEST_EQUAL(pieces[piece_index_t{1}], true);
-		TEST_EQUAL(pieces[piece_index_t{2}], false);
-		TEST_EQUAL(pieces[piece_index_t{3}], false);
+		TEST_EQUAL(pieces[0_piece], true);
+		TEST_EQUAL(pieces[1_piece], true);
+		TEST_EQUAL(pieces[2_piece], false);
+		TEST_EQUAL(pieces[3_piece], false);
 	}
 }
 
@@ -513,9 +498,9 @@ void test_piece_slots_seed(settings_pack const& sett)
 	{
 		std::vector<char> a(128 * 1024 * 8);
 		std::vector<char> b(128 * 1024);
-		std::ofstream("add_torrent_params_test" SEP "test_resume" SEP "tmp1").write(a.data(), std::streamsize(a.size()));
-		std::ofstream("add_torrent_params_test" SEP "test_resume" SEP "tmp2").write(b.data(), std::streamsize(b.size()));
-		std::ofstream("add_torrent_params_test" SEP "test_resume" SEP "tmp3").write(b.data(), std::streamsize(b.size()));
+		ofstream("add_torrent_params_test" SEP "test_resume" SEP "tmp1").write(a.data(), std::streamsize(a.size()));
+		ofstream("add_torrent_params_test" SEP "test_resume" SEP "tmp2").write(b.data(), std::streamsize(b.size()));
+		ofstream("add_torrent_params_test" SEP "test_resume" SEP "tmp3").write(b.data(), std::streamsize(b.size()));
 	}
 
 	add_torrent_params p;
@@ -531,7 +516,7 @@ void test_piece_slots_seed(settings_pack const& sett)
 
 	torrent_status s = h.status();
 	print_alerts(ses, "ses");
-	TEST_EQUAL(s.info_hash, ti->info_hash());
+	TEST_EQUAL(s.info_hashes, ti->info_hashes());
 	TEST_EQUAL(s.pieces.size(), ti->num_pieces());
 	for (auto const i : ti->piece_range())
 	{
@@ -770,8 +755,9 @@ TORRENT_TEST(plain_deprecated)
 {
 	lt::session ses(settings());
 
+	auto const now = lt::clock_type::now();
 	torrent_status s = test_resume_flags(ses, {}, "", "", true).status();
-	default_tests(s);
+	default_tests(s, now);
 #ifdef TORRENT_WINDOWS
 	TEST_EQUAL(s.save_path, "c:\\add_torrent_params save_path");
 #else
@@ -785,9 +771,10 @@ TORRENT_TEST(plain_deprecated)
 TORRENT_TEST(use_resume_save_path_deprecated)
 {
 	lt::session ses(settings());
+	auto const now = lt::clock_type::now();
 	torrent_status s = test_resume_flags(ses
 		, torrent_flags::use_resume_save_path, "", "", true).status();
-	default_tests(s);
+	default_tests(s, now);
 #ifdef TORRENT_WINDOWS
 	TEST_EQUAL(s.save_path, "c:\\resume_data save_path");
 #else
@@ -801,11 +788,12 @@ TORRENT_TEST(use_resume_save_path_deprecated)
 TORRENT_TEST(override_resume_data_deprecated)
 {
 	lt::session ses(settings());
+	auto const now = lt::clock_type::now();
 	torrent_status s = test_resume_flags(ses
 		, torrent_flags::override_resume_data
 		| torrent_flags::paused, "", "", true).status();
 
-	default_tests(s);
+	default_tests(s, now);
 #ifdef TORRENT_WINDOWS
 	TEST_EQUAL(s.save_path, "c:\\add_torrent_params save_path");
 #else
@@ -819,9 +807,10 @@ TORRENT_TEST(override_resume_data_deprecated)
 TORRENT_TEST(seed_mode_deprecated)
 {
 	lt::session ses(settings());
+	auto const now = lt::clock_type::now();
 	torrent_status s = test_resume_flags(ses, torrent_flags::override_resume_data
 		| torrent_flags::seed_mode, "", "", true).status();
-	default_tests(s);
+	default_tests(s, now);
 	TEST_EQUAL(s.flags & flags_mask, torrent_flags::seed_mode);
 	TEST_EQUAL(s.connections_limit, 2);
 	TEST_EQUAL(s.uploads_limit, 1);
@@ -830,9 +819,10 @@ TORRENT_TEST(seed_mode_deprecated)
 TORRENT_TEST(upload_mode_deprecated)
 {
 	lt::session ses(settings());
+	auto const now = lt::clock_type::now();
 	torrent_status s = test_resume_flags(ses
 		, torrent_flags::upload_mode, "", "", true).status();
-	default_tests(s);
+	default_tests(s, now);
 #ifdef TORRENT_WINDOWS
 	TEST_EQUAL(s.save_path, "c:\\add_torrent_params save_path");
 #else
@@ -847,10 +837,11 @@ TORRENT_TEST(upload_mode_deprecated)
 TORRENT_TEST(share_mode_deprecated)
 {
 	lt::session ses(settings());
+	auto const now = lt::clock_type::now();
 	torrent_status s = test_resume_flags(ses
 		, torrent_flags::override_resume_data
 		| torrent_flags::share_mode, "", "", true).status();
-	default_tests(s);
+	default_tests(s, now);
 #ifdef TORRENT_WINDOWS
 	TEST_EQUAL(s.save_path, "c:\\add_torrent_params save_path");
 #else
@@ -866,9 +857,10 @@ TORRENT_TEST(auto_managed_deprecated)
 {
 	lt::session ses(settings());
 	// resume data overrides the auto-managed flag
+	auto const now = lt::clock_type::now();
 	torrent_status s = test_resume_flags(ses
 		, torrent_flags::auto_managed, "", "", true).status();
-	default_tests(s);
+	default_tests(s, now);
 #ifdef TORRENT_WINDOWS
 	TEST_EQUAL(s.save_path, "c:\\add_torrent_params save_path");
 #else
@@ -883,8 +875,9 @@ TORRENT_TEST(paused_deprecated)
 {
 	lt::session ses(settings());
 	// resume data overrides the paused flag
+	auto const now = lt::clock_type::now();
 	torrent_status s = test_resume_flags(ses, torrent_flags::paused, "", "", true).status();
-	default_tests(s);
+	default_tests(s, now);
 #ifdef TORRENT_WINDOWS
 	TEST_EQUAL(s.save_path, "c:\\add_torrent_params save_path");
 #else
@@ -998,13 +991,16 @@ void test_zero_file_prio(bool test_deprecated = false, bool mix_prios = false)
 
 	rd["file-format"] = "libtorrent resume file";
 	rd["file-version"] = 1;
-	rd["info-hash"] = ti->info_hash().to_string();
+	rd["info-hash"] = ti->info_hashes().v1.to_string();
 	rd["blocks per piece"] = std::max(1, ti->piece_length() / 0x4000);
 
 	// set file priorities to 0
 	rd["file_priority"] = entry::list_type(100, entry(0));
 
-	rd["pieces"] = std::string(std::size_t(ti->num_pieces()), '\x01');
+	if (mix_prios)
+		rd["pieces"] = std::string(std::size_t(ti->num_pieces()), '\x01');
+	else
+		rd["pieces"] = std::string(std::size_t(ti->num_pieces()), '\x00');
 
 	// but set the piece priorities to 1. these take precedence
 	if (mix_prios)
@@ -1028,7 +1024,19 @@ void test_zero_file_prio(bool test_deprecated = false, bool mix_prios = false)
 
 	torrent_handle h = ses.add_torrent(p);
 
-	torrent_status s = h.status();
+	lt::torrent_status s = h.status();
+	for (int i = 0; i < 40; ++i)
+	{
+		if (s.state != torrent_status::checking_files
+			&& s.state != torrent_status::checking_resume_data)
+			break;
+		std::this_thread::sleep_for(lt::milliseconds(100));
+		s = h.status();
+	}
+
+	// Once a torrent becomes seed, any piece- and file priorities are
+	// forgotten and all bytes are considered "wanted". So, whether we "want"
+	// all bytes here or not, depends on whether we're a seed
 	if (mix_prios)
 	{
 		TEST_EQUAL(s.total_wanted, ti->total_size());
@@ -1061,9 +1069,8 @@ TORRENT_TEST(backwards_compatible_resume_info_dict)
 	entry rd;
 	rd["file-format"] = "libtorrent resume file";
 	rd["name"] = ti->name();
-	rd["info-hash"] = ti->info_hash();
-	auto metainfo = ti->metadata();
-	rd["info"] = bdecode({metainfo.get(), ti->metadata_size()});
+	rd["info-hash"] = ti->info_hashes().v1;
+	rd["info"] = bdecode(ti->info_section());
 	std::vector<char> resume_data;
 	bencode(back_inserter(resume_data), rd);
 
@@ -1074,11 +1081,54 @@ TORRENT_TEST(backwards_compatible_resume_info_dict)
 	session ses;
 	torrent_handle h = ses.add_torrent(atp);
 	auto torrent = h.torrent_file();
-	TEST_CHECK(torrent->info_hash() == ti->info_hash());
+	TEST_CHECK(torrent->info_hashes() == ti->info_hashes());
 	torrent_status s = h.status();
 }
 #endif
 
+TORRENT_TEST(merkle_trees)
+{
+	lt::add_torrent_params p;
+	p.ti = generate_torrent();
+	p.save_path = ".";
+
+	std::vector<std::vector<char>> piece_layers;
+
+	for (file_index_t const i : p.ti->files().file_range())
+	{
+		auto const pspan = p.ti->piece_layer(i);
+		piece_layers.emplace_back(pspan.begin(), pspan.end());
+	}
+
+	lt::session ses(settings());
+	auto h = ses.add_torrent(p);
+
+	h.save_resume_data();
+
+	save_resume_data_alert const* a = alert_cast<save_resume_data_alert>(
+		wait_for_alert(ses, save_resume_data_alert::alert_type
+		, "merkle_trees"));
+
+	TEST_CHECK(a);
+	if (a == nullptr) return;
+
+	TEST_EQUAL(a->params.merkle_trees.size(), 3);
+	TEST_EQUAL(a->params.merkle_tree_mask.size(), 3);
+
+	auto const pl = h.piece_layers();
+	TEST_CHECK(pl.size() == piece_layers.size());
+
+	for (file_index_t const i : p.ti->files().file_range())
+	{
+		auto const& one_layer = pl[std::size_t(int(i))];
+		TEST_CHECK(one_layer.size() == piece_layers[std::size_t(int(i))].size() / lt::sha256_hash::size());
+		for (int piece = 0; piece < int(one_layer.size()); ++piece)
+			TEST_CHECK(one_layer[std::size_t(piece)] == lt::sha256_hash(piece_layers[std::size_t(int(i))].data() + piece * lt::sha256_hash::size()));
+		auto const& m = a->params.merkle_tree_mask[i];
+		TEST_CHECK(std::count(m.begin(), m.end(), true) == int(a->params.merkle_trees[i].size()));
+	}
+}
+
 TORRENT_TEST(resume_info_dict)
 {
 	// make sure the "info" dictionary is picked up correctly from the
@@ -1088,15 +1138,14 @@ TORRENT_TEST(resume_info_dict)
 	entry rd;
 	rd["file-format"] = "libtorrent resume file";
 	rd["name"] = ti->name();
-	rd["info-hash"] = ti->info_hash();
-	auto metainfo = ti->metadata();
-	rd["info"] = bdecode({metainfo.get(), ti->metadata_size()});
+	rd["info-hash"] = ti->info_hashes().v1;
+	rd["info"] = bdecode(ti->info_section());
 	std::vector<char> resume_data;
 	bencode(back_inserter(resume_data), rd);
 
 	error_code ec;
 	add_torrent_params atp = read_resume_data(resume_data, ec);
-	TEST_CHECK(atp.ti->info_hash() == ti->info_hash());
+	TEST_CHECK(atp.ti->info_hashes() == ti->info_hashes());
 }
 
 TORRENT_TEST(zero_file_prio)
@@ -1149,16 +1198,15 @@ void test_seed_mode(test_mode_t const flags)
 
 	if (flags & test_mode::extended_files)
 	{
-		error_code ec;
-		file("test_resume" SEP "tmp2", open_mode::read_write, ec).set_size(128 * 1024 + 10, ec);
-		TEST_CHECK(!ec);
+		int const ret = ::truncate("test_resume" SEP "tmp2", 128 * 1024 + 10);
+		TEST_EQUAL(ret, 0);
 	}
 
 	entry rd;
 
 	rd["file-format"] = "libtorrent resume file";
 	rd["file-version"] = 1;
-	rd["info-hash"] = ti->info_hash().to_string();
+	rd["info-hash"] = ti->info_hashes().v1.to_string();
 	rd["blocks per piece"] = std::max(1, ti->piece_length() / 0x4000);
 
 	if (flags & test_mode::file_prio)
@@ -1389,7 +1437,7 @@ TORRENT_TEST(seed_mode_load_peers)
 	p.ti = ti;
 	p.save_path = ".";
 	p.flags |= torrent_flags::seed_mode;
-	p.peers.push_back(tcp::endpoint(address::from_string("1.2.3.4"), 12345));
+	p.peers.push_back(tcp::endpoint(make_address("1.2.3.4"), 12345));
 
 	torrent_handle h = ses.add_torrent(p);
 
@@ -1406,7 +1454,7 @@ TORRENT_TEST(seed_mode_load_peers)
 
 	auto const& peers = a->params.peers;
 	TEST_EQUAL(peers.size(), 1);
-	TEST_CHECK(peers[0] == tcp::endpoint(address::from_string("1.2.3.4"), 12345));
+	TEST_CHECK(peers[0] == tcp::endpoint(make_address("1.2.3.4"), 12345));
 }
 
 TORRENT_TEST(resume_save_load)
@@ -1503,8 +1551,9 @@ TORRENT_TEST(plain)
 {
 	lt::session ses(settings());
 
+	auto const now = lt::clock_type::now();
 	torrent_status s = test_resume_flags(ses).status();
-	default_tests(s);
+	default_tests(s, now);
 #ifdef TORRENT_WINDOWS
 	TEST_EQUAL(s.save_path, "c:\\add_torrent_params save_path");
 #else
@@ -1518,9 +1567,10 @@ TORRENT_TEST(plain)
 TORRENT_TEST(seed_mode)
 {
 	lt::session ses(settings());
+	auto const now = lt::clock_type::now();
 	torrent_status s = test_resume_flags(ses
 		, torrent_flags::seed_mode).status();
-	default_tests(s);
+	default_tests(s, now);
 	TEST_EQUAL(s.flags & flags_mask, torrent_flags::seed_mode);
 	TEST_EQUAL(s.connections_limit, 1345);
 	TEST_EQUAL(s.uploads_limit, 1346);
@@ -1529,14 +1579,10 @@ TORRENT_TEST(seed_mode)
 TORRENT_TEST(seed_mode_no_verify_files)
 {
 	lt::session ses(settings());
+	auto const now = lt::clock_type::now();
 	torrent_status s = test_resume_flags(ses
 		, torrent_flags::seed_mode | torrent_flags::no_verify_files).status();
-	default_tests(s);
-#ifdef TORRENT_WINDOWS
-	TEST_EQUAL(s.save_path, "c:\\add_torrent_params save_path");
-#else
-	TEST_EQUAL(s.save_path, "/add_torrent_params save_path");
-#endif
+	default_tests(s, now);
 	// note taht torrent_flags::no_verify_files is NOT set here
 	TEST_EQUAL(s.flags & flags_mask, torrent_flags::seed_mode);
 	TEST_EQUAL(s.connections_limit, 1345);
@@ -1546,8 +1592,9 @@ TORRENT_TEST(seed_mode_no_verify_files)
 TORRENT_TEST(upload_mode)
 {
 	lt::session ses(settings());
+	auto const now = lt::clock_type::now();
 	torrent_status s = test_resume_flags(ses, torrent_flags::upload_mode).status();
-	default_tests(s);
+	default_tests(s, now);
 #ifdef TORRENT_WINDOWS
 	TEST_EQUAL(s.save_path, "c:\\add_torrent_params save_path");
 #else
@@ -1562,9 +1609,10 @@ TORRENT_TEST(upload_mode)
 TORRENT_TEST(share_mode)
 {
 	lt::session ses(settings());
+	auto const now = lt::clock_type::now();
 	torrent_status s = test_resume_flags(ses
 		, torrent_flags::share_mode).status();
-	default_tests(s);
+	default_tests(s, now);
 #ifdef TORRENT_WINDOWS
 	TEST_EQUAL(s.save_path, "c:\\add_torrent_params save_path");
 #else
@@ -1580,14 +1628,17 @@ TORRENT_TEST(auto_managed)
 {
 	lt::session ses(settings());
 	// resume data overrides the auto-managed flag
+	auto const now = lt::clock_type::now();
 	torrent_status s = test_resume_flags(ses, torrent_flags::auto_managed).status();
-	default_tests(s);
+	default_tests(s, now);
 #ifdef TORRENT_WINDOWS
 	TEST_EQUAL(s.save_path, "c:\\add_torrent_params save_path");
 #else
 	TEST_EQUAL(s.save_path, "/add_torrent_params save_path");
 #endif
-	TEST_EQUAL(s.flags & flags_mask, torrent_flags::auto_managed);
+	// auto managed torrents may have been paused by the time we get here, so
+	// filter out that flag
+	TEST_EQUAL((s.flags & flags_mask) & ~torrent_flags::paused, torrent_flags::auto_managed);
 	TEST_EQUAL(s.connections_limit, 1345);
 	TEST_EQUAL(s.uploads_limit, 1346);
 }
@@ -1596,8 +1647,9 @@ TORRENT_TEST(paused)
 {
 	lt::session ses(settings());
 	// resume data overrides the paused flag
+	auto const now = lt::clock_type::now();
 	torrent_status s = test_resume_flags(ses, torrent_flags::paused).status();
-	default_tests(s);
+	default_tests(s, now);
 #ifdef TORRENT_WINDOWS
 	TEST_EQUAL(s.save_path, "c:\\add_torrent_params save_path");
 #else
@@ -1617,7 +1669,7 @@ TORRENT_TEST(no_metadata)
 	lt::session ses(settings());
 
 	add_torrent_params p;
-	p.info_hash = sha1_hash("abababababababababab");
+	p.info_hashes.v1 = sha1_hash("abababababababababab");
 	p.save_path = ".";
 	p.name = "foobar";
 	torrent_handle h = ses.add_torrent(p);
@@ -1629,7 +1681,7 @@ TORRENT_TEST(no_metadata)
 	if (ra)
 	{
 		auto const& atp = ra->params;
-		TEST_EQUAL(atp.info_hash, p.info_hash);
+		TEST_EQUAL(atp.info_hashes, p.info_hashes);
 		TEST_EQUAL(atp.name, "foobar");
 	}
 }
@@ -1641,7 +1693,7 @@ void test_unfinished_pieces(Fun f)
 	std::shared_ptr<torrent_info> ti = generate_torrent(true, true);
 
 	add_torrent_params p;
-	p.info_hash = ti->info_hash();
+	p.info_hashes = ti->info_hashes();
 	p.have_pieces.resize(ti->num_pieces(), true);
 	p.ti = ti;
 	p.save_path = ".";
@@ -1651,7 +1703,7 @@ void test_unfinished_pieces(Fun f)
 	lt::session ses(settings());
 	torrent_handle h = ses.add_torrent(p);
 	torrent_status s = h.status();
-	TEST_EQUAL(s.info_hash, ti->info_hash());
+	TEST_EQUAL(s.info_hashes, ti->info_hashes());
 
 	if (s.state == torrent_status::seeding) return;
 
@@ -1668,7 +1720,7 @@ void test_unfinished_pieces(Fun f)
 	TEST_EQUAL(s.state, torrent_status::seeding);
 }
 
-TORRENT_TEST(unfinished_pieces_purse_seed)
+TORRENT_TEST(unfinished_pieces_pure_seed)
 {
 	test_unfinished_pieces([](torrent_info const&, add_torrent_params&){});
 }
@@ -1687,8 +1739,8 @@ TORRENT_TEST(unfinished_pieces_finished)
 	// all downloaded gets checked and turn into "have".
 	test_unfinished_pieces([](torrent_info const& ti, add_torrent_params& atp)
 	{
-		atp.have_pieces.clear_bit(piece_index_t{0});
-		atp.unfinished_pieces[lt::piece_index_t{0}].resize(ti.piece_length() / 0x4000, true);
+		atp.have_pieces.clear_bit(0_piece);
+		atp.unfinished_pieces[0_piece].resize(ti.piece_length() / 0x4000, true);
 	});
 }
 
@@ -1708,11 +1760,20 @@ TORRENT_TEST(unfinished_pieces_all_finished)
 	});
 }
 
+#if TORRENT_HAVE_MMAP || TORRENT_HAVE_MAP_VIEW_OF_FILE
+// this test relies on (and tests) the disk I/O being asynchronous. Since the
+// posix_disk_io isn't, this test won't pass
 TORRENT_TEST(resume_data_have_pieces)
 {
+	if (sizeof(void*) < 8)
+	{
+		// disable this test when disk I/O is not async
+		return;
+	}
+
 	file_storage fs;
 	fs.add_file("tmp1", 128 * 1024 * 8);
-	lt::create_torrent t(fs, 128 * 1024, 6);
+	lt::create_torrent t(fs, 128 * 1024);
 
 	TEST_CHECK(t.num_pieces() > 0);
 
@@ -1734,17 +1795,17 @@ TORRENT_TEST(resume_data_have_pieces)
 	atp.save_path = ".";
 	auto h = ses.add_torrent(atp);
 	wait_for_downloading(ses, "");
-	h.add_piece(piece_index_t{0}, std::move(piece_data));
-	lt::torrent_status s = h.status(torrent_handle::query_pieces);
+	h.add_piece(0_piece, piece_data.data());
 
-	ses.pause();
 	h.save_resume_data();
+	ses.pause();
 
 	auto const* rs = static_cast<save_resume_data_alert const*>(
 		wait_for_alert(ses, save_resume_data_alert::alert_type));
 	TEST_CHECK(rs != nullptr);
 	TEST_EQUAL(rs->params.unfinished_pieces.size(), 1);
 }
+#endif
 
 // See https://github.com/arvidn/libtorrent/issues/5174
 TORRENT_TEST(removed)
