@@ -1,6 +1,11 @@
 /*
 
-Copyright (c) 2008, Arvid Norberg
+Copyright (c) 2008-2009, 2013-2020, Arvid Norberg
+Copyright (c) 2016, 2018-2019, Alden Torres
+Copyright (c) 2017, Falcosc
+Copyright (c) 2017, Steven Siloti
+Copyright (c) 2018, d-komarov
+Copyright (c) 2020, AllSeeingEyeTolledEweSew
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -41,6 +46,8 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/extensions.hpp"
 #include "libtorrent/aux_/path.hpp" // for combine_path, current_working_directory
 #include "libtorrent/magnet_uri.hpp"
+#include "libtorrent/span.hpp"
+#include "libtorrent/session_params.hpp"
 #include "libtorrent/random.hpp"
 #include "settings.hpp"
 #include <tuple>
@@ -80,7 +87,7 @@ void test_running_torrent(std::shared_ptr<torrent_info> info, std::int64_t file_
 {
 	settings_pack pack = settings();
 	pack.set_int(settings_pack::alert_mask, alert_category::piece_progress | alert_category::storage);
-	pack.set_str(settings_pack::listen_interfaces, test_listen_interface());
+	pack.set_str(settings_pack::listen_interfaces, "0.0.0.0:48130");
 	pack.set_int(settings_pack::max_retry_port_bind, 10);
 	lt::session ses(pack);
 
@@ -113,7 +120,7 @@ void test_running_torrent(std::shared_ptr<torrent_info> info, std::int64_t file_
 	TEST_EQUAL(st.total_wanted_done, 0);
 
 	aux::vector<download_priority_t, file_index_t> prio(std::size_t(info->num_files()), 1_pri);
-	prio[file_index_t(0)] = 0_pri;
+	prio[0_file] = 0_pri;
 	TEST_CHECK(prioritize_files(h, prio));
 	st = h.status();
 
@@ -134,13 +141,13 @@ void test_running_torrent(std::shared_ptr<torrent_info> info, std::int64_t file_
 
 	if (info->num_pieces() > 0)
 	{
-		h.piece_priority(piece_index_t(0), 1_pri);
+		h.piece_priority(0_piece, 1_pri);
 		st = h.status();
-		TEST_CHECK(st.pieces.size() > 0 && st.pieces[piece_index_t(0)] == false);
+		TEST_CHECK(st.pieces.size() > 0 && st.pieces[0_piece] == false);
 		std::vector<char> piece(std::size_t(info->piece_length()));
 		for (int i = 0; i < int(piece.size()); ++i)
 			piece[std::size_t(i)] = (i % 26) + 'A';
-		h.add_piece(piece_index_t(0), &piece[0], torrent_handle::overwrite_existing);
+		h.add_piece(0_piece, &piece[0], torrent_handle::overwrite_existing);
 
 		// wait until the piece is done writing and hashing
 		wait_for_alert(ses, piece_finished_alert::alert_type, "piece_finished_alert");
@@ -148,7 +155,7 @@ void test_running_torrent(std::shared_ptr<torrent_info> info, std::int64_t file_
 		TEST_CHECK(st.pieces.size() > 0);
 
 		std::cout << "reading piece 0" << std::endl;
-		h.read_piece(piece_index_t(0));
+		h.read_piece(0_piece);
 		alert const* a = wait_for_alert(ses, read_piece_alert::alert_type, "read_piece");
 		TEST_CHECK(a);
 		read_piece_alert const* rpa = alert_cast<read_piece_alert>(a);
@@ -157,17 +164,17 @@ void test_running_torrent(std::shared_ptr<torrent_info> info, std::int64_t file_
 		{
 			std::cout << "SUCCEEDED!" << std::endl;
 			TEST_CHECK(std::memcmp(&piece[0], rpa->buffer.get()
-				, std::size_t(info->piece_size(piece_index_t(0)))) == 0);
-			TEST_CHECK(rpa->size == info->piece_size(piece_index_t(0)));
-			TEST_CHECK(rpa->piece == piece_index_t(0));
-			TEST_CHECK(hasher(piece).final() == info->hash_for_piece(piece_index_t(0)));
+				, std::size_t(info->piece_size(0_piece))) == 0);
+			TEST_CHECK(rpa->size == info->piece_size(0_piece));
+			TEST_CHECK(rpa->piece == 0_piece);
+			TEST_CHECK(hasher(piece).final() == info->hash_for_piece(0_piece));
 		}
 	}
 
 	TEST_CHECK(h.get_file_priorities() == prio);
 }
 
-void test_large_piece_size(std::int64_t const size)
+void test_large_piece_size(int const size)
 {
 	entry torrent;
 	entry& info = torrent["info"];
@@ -179,15 +186,7 @@ void test_large_piece_size(std::int64_t const size)
 	std::vector<char> buf;
 	bencode(std::back_inserter(buf), torrent);
 	add_torrent_params atp;
-	try
-	{
-		atp.ti = std::make_shared<torrent_info>(buf, from_span);
-	}
-	catch (lt::system_error const& e)
-	{
-		TEST_CHECK(e.code() == error_code(lt::errors::invalid_piece_size));
-		return;
-	}
+	atp.ti = std::make_shared<torrent_info>(buf, from_span);
 	atp.save_path = ".";
 
 	lt::session ses;
@@ -220,31 +219,32 @@ TORRENT_TEST(long_names)
 
 TORRENT_TEST(large_piece_size)
 {
-	test_large_piece_size(0xfffc000 + 0x4000);
-	test_large_piece_size(0x10000000);
-	test_large_piece_size(0x20000000);
-	test_large_piece_size(0x40000000);
-	test_large_piece_size(0x80000000);
-	test_large_piece_size(0x100000000);
+	test_large_piece_size(32768 * 16 * 1024);
+	test_large_piece_size(65535 * 16 * 1024);
 }
 
 TORRENT_TEST(total_wanted)
 {
 	file_storage fs;
 
-	fs.add_file("test_torrent_dir4/tmp1", 1024);
-	fs.add_file("test_torrent_dir4/tmp2", 1024);
-	fs.add_file("test_torrent_dir4/tmp3", 1024);
-	fs.add_file("test_torrent_dir4/tmp4", 1024);
+	fs.add_file("test_torrent_dir4/tmp1", default_block_size);
+	fs.add_file("test_torrent_dir4/tmp2", default_block_size);
+	fs.add_file("test_torrent_dir4/tmp3", default_block_size);
+	fs.add_file("test_torrent_dir4/tmp4", default_block_size);
+
+	lt::create_torrent t(fs, default_block_size);
+	t.set_hash(0_piece, sha1_hash::max());
+	t.set_hash(1_piece, sha1_hash::max());
+	t.set_hash(2_piece, sha1_hash::max());
+	t.set_hash(3_piece, sha1_hash::max());
 
-	lt::create_torrent t(fs, 1024);
 	std::vector<char> tmp;
 	bencode(std::back_inserter(tmp), t.generate());
 	auto info = std::make_shared<torrent_info>(tmp, from_span);
 
 	settings_pack pack = settings();
 	pack.set_int(settings_pack::alert_mask, alert_category::storage);
-	pack.set_str(settings_pack::listen_interfaces, test_listen_interface());
+	pack.set_str(settings_pack::listen_interfaces, "0.0.0.0:48130");
 	pack.set_int(settings_pack::max_retry_port_bind, 10);
 	lt::session ses(pack);
 
@@ -253,20 +253,20 @@ TORRENT_TEST(total_wanted)
 	p.save_path = ".";
 
 	// we just want 1 out of 4 files, 1024 out of 4096 bytes
-	p.file_priorities.resize(4, 0_pri);
-	p.file_priorities[1] = 1_pri;
+	p.file_priorities.resize(8, 0_pri);
+	p.file_priorities[2] = 1_pri;
 
 	torrent_handle h = ses.add_torrent(std::move(p));
 
 	torrent_status st = h.status();
-	TEST_EQUAL(st.total_wanted, 1024);
+	TEST_EQUAL(st.total_wanted, default_block_size);
 	TEST_EQUAL(st.total_wanted_done, 0);
 
 	// make sure that selecting and unseleting a file quickly still end up with
 	// the last set priority
-	h.file_priority(file_index_t{1}, default_priority);
-	h.file_priority(file_index_t{1}, dont_download);
-	TEST_CHECK(wait_priority(h, aux::vector<download_priority_t, file_index_t>(static_cast<std::size_t>(fs.num_files()))));
+	h.file_priority(file_index_t{2}, default_priority);
+	h.file_priority(file_index_t{2}, dont_download);
+	TEST_CHECK(wait_priority(h, aux::vector<download_priority_t, file_index_t>(static_cast<std::size_t>(fs.num_files()), dont_download)));
 	TEST_EQUAL(h.status({}).total_wanted, 0);
 }
 
@@ -277,19 +277,20 @@ TORRENT_TEST(added_peers)
 	fs.add_file("test_torrent_dir4/tmp1", 1024);
 
 	lt::create_torrent t(fs, 1024);
+	t.set_hash(0_piece, sha1_hash::max());
 	std::vector<char> tmp;
 	bencode(std::back_inserter(tmp), t.generate());
 	auto info = std::make_shared<torrent_info>(tmp, from_span);
 
 	settings_pack pack = settings();
-	pack.set_str(settings_pack::listen_interfaces, test_listen_interface());
+	pack.set_str(settings_pack::listen_interfaces, "0.0.0.0:48130");
 	pack.set_int(settings_pack::max_retry_port_bind, 10);
 	lt::session ses(pack);
 
 	add_torrent_params p = parse_magnet_uri(
 		"magnet:?xt=urn:btih:abababababababababababababababababababab&x.pe=127.0.0.1:48081&x.pe=127.0.0.2:48082");
 	p.ti = info;
-	p.info_hash.clear();
+	p.info_hashes = info_hash_t{};
 	p.save_path = ".";
 
 	torrent_handle h = ses.add_torrent(std::move(p));
@@ -308,6 +309,7 @@ TORRENT_TEST(mismatching_info_hash)
 	file_storage fs;
 	fs.add_file("test_torrent_dir4/tmp1", 1024);
 	lt::create_torrent t(fs, 1024);
+	t.set_hash(0_piece, sha1_hash::max());
 	std::vector<char> tmp;
 	bencode(std::back_inserter(tmp), t.generate());
 	auto info = std::make_shared<torrent_info>(tmp, from_span);
@@ -317,7 +319,7 @@ TORRENT_TEST(mismatching_info_hash)
 
 	// this info-hash is definitely different from the one in `info`, this
 	// should trigger a failure
-	p.info_hash = lt::sha1_hash("01010101010101010101");
+	p.info_hashes = lt::info_hash_t(lt::sha1_hash("01010101010101010101"));
 	p.save_path = ".";
 
 	lt::session ses(settings());
@@ -332,6 +334,7 @@ TORRENT_TEST(exceed_file_prio)
 	file_storage fs;
 	fs.add_file("test_torrent_dir4/tmp1", 1024);
 	lt::create_torrent t(fs, 1024);
+	t.set_hash(0_piece, sha1_hash::max());
 	std::vector<char> tmp;
 	bencode(std::back_inserter(tmp), t.generate());
 	auto info = std::make_shared<torrent_info>(tmp, from_span);
@@ -343,7 +346,6 @@ TORRENT_TEST(exceed_file_prio)
 	p.save_path = ".";
 
 	lt::session ses(settings());
-	error_code ec;
 	torrent_handle h = ses.add_torrent(std::move(p));
 	auto const prios = h.get_file_priorities();
 	TEST_CHECK(prios.size() == 1);
@@ -354,6 +356,7 @@ TORRENT_TEST(exceed_piece_prio)
 	file_storage fs;
 	fs.add_file("test_torrent_dir4/tmp1", 1024);
 	lt::create_torrent t(fs, 1024);
+	t.set_hash(0_piece, sha1_hash::max());
 	std::vector<char> tmp;
 	bencode(std::back_inserter(tmp), t.generate());
 	auto info = std::make_shared<torrent_info>(tmp, from_span);
@@ -366,7 +369,6 @@ TORRENT_TEST(exceed_piece_prio)
 	p.save_path = ".";
 
 	lt::session ses(settings());
-	error_code ec;
 	torrent_handle h = ses.add_torrent(std::move(p));
 	auto const prios = h.get_piece_priorities();
 	TEST_CHECK(prios.size() == num_pieces);
@@ -375,12 +377,11 @@ TORRENT_TEST(exceed_piece_prio)
 TORRENT_TEST(exceed_piece_prio_magnet)
 {
 	add_torrent_params p;
-	p.info_hash = sha1_hash("abababababababababab");
+	p.info_hashes = lt::info_hash_t(lt::sha1_hash("01010101010101010101"));
 	p.piece_priorities.resize(9999, lt::low_priority);
 	p.save_path = ".";
 
 	lt::session ses(settings());
-	error_code ec;
 	torrent_handle h = ses.add_torrent(std::move(p));
 	auto const prios = h.get_piece_priorities();
 	TEST_CHECK(prios.empty());
@@ -412,8 +413,7 @@ TORRENT_TEST(torrent)
 			t.set_hash(i, ph);
 
 		std::vector<char> tmp;
-		std::back_insert_iterator<std::vector<char>> out(tmp);
-		bencode(out, t.generate());
+		bencode(std::back_inserter(tmp), t.generate());
 		error_code ec;
 		auto info = std::make_shared<torrent_info>(tmp, std::ref(ec), from_span);
 		TEST_CHECK(info->num_pieces() > 0);
@@ -424,10 +424,10 @@ TORRENT_TEST(torrent)
 	{
 		file_storage fs;
 
-		fs.add_file("test_torrent_dir2/tmp1", 1024);
-		lt::create_torrent t(fs, 1024, 6);
+		fs.add_file("test_torrent_dir2/tmp1", default_block_size);
+		lt::create_torrent t(fs, default_block_size);
 
-		std::vector<char> piece(1024);
+		std::vector<char> piece(default_block_size);
 		for (int i = 0; i < int(piece.size()); ++i)
 			piece[std::size_t(i)] = (i % 26) + 'A';
 
@@ -437,11 +437,12 @@ TORRENT_TEST(torrent)
 		for (auto const i : fs.piece_range())
 			t.set_hash(i, ph);
 
+		t.set_hash2(file_index_t{ 0 }, piece_index_t::diff_type{ 0 }, lt::hasher256(piece).final());
+
 		std::vector<char> tmp;
-		std::back_insert_iterator<std::vector<char>> out(tmp);
-		bencode(out, t.generate());
+		bencode(std::back_inserter(tmp), t.generate());
 		auto info = std::make_shared<torrent_info>(tmp, from_span);
-		test_running_torrent(info, 1024);
+		test_running_torrent(info, default_block_size);
 	}
 }
 
@@ -453,7 +454,7 @@ struct plugin_creator
 	explicit plugin_creator(int& c) : m_called(c) {}
 
 	std::shared_ptr<lt::torrent_plugin>
-	operator()(torrent_handle const&, void*)
+	operator()(torrent_handle const&, client_data_t)
 	{
 		++m_called;
 		return std::make_shared<test_plugin>();
@@ -467,7 +468,7 @@ TORRENT_TEST(duplicate_is_not_error)
 	file_storage fs;
 
 	fs.add_file("test_torrent_dir2/tmp1", 1024);
-	lt::create_torrent t(fs, 128 * 1024, 6);
+	lt::create_torrent t(fs, 128 * 1024);
 
 	std::vector<char> piece(128 * 1024);
 	for (int i = 0; i < int(piece.size()); ++i)
@@ -480,8 +481,7 @@ TORRENT_TEST(duplicate_is_not_error)
 		t.set_hash(i, ph);
 
 	std::vector<char> tmp;
-	std::back_insert_iterator<std::vector<char>> out(tmp);
-	bencode(out, t.generate());
+	bencode(std::back_inserter(tmp), t.generate());
 
 	int called = 0;
 	plugin_creator creator(called);
@@ -534,54 +534,25 @@ TORRENT_TEST(rename_file)
 
 	fs.add_file("test3/tmp1", 20);
 	fs.add_file("test3/tmp2", 20);
-	lt::create_torrent t(fs, 128 * 1024, 6);
+	lt::create_torrent t(fs, 128 * 1024, lt::create_torrent::v1_only);
+	t.set_hash(0_piece, sha1_hash::max());
 
 	std::vector<char> tmp;
-	std::back_insert_iterator<std::vector<char>> out(tmp);
-	bencode(out, t.generate());
+	bencode(std::back_inserter(tmp), t.generate());
 	auto info = std::make_shared<torrent_info>(tmp, from_span);
 
-	TEST_EQUAL(info->files().file_path(file_index_t(0)), combine_path("test3","tmp1"));
+	TEST_EQUAL(info->files().file_path(0_file), combine_path("test3","tmp1"));
 
 	// move "test3/tmp1" -> "tmp1"
-	info->rename_file(file_index_t(0), "tmp1");
+	info->rename_file(0_file, "tmp1");
 
-	TEST_EQUAL(info->files().file_path(file_index_t(0)), "tmp1");
+	TEST_EQUAL(info->files().file_path(0_file), "tmp1");
 }
 
-#if TORRENT_ABI_VERSION == 1
-TORRENT_TEST(async_load_deprecated)
-{
-	settings_pack pack = settings();
-	lt::session ses(pack);
-
-	add_torrent_params p;
-	p.flags &= ~torrent_flags::paused;
-	p.flags &= ~torrent_flags::auto_managed;
-	std::string dir = parent_path(current_working_directory());
-
-	p.url = "file://" + combine_path(combine_path(dir, "test_torrents"), "base.torrent");
-	p.save_path = ".";
-	ses.async_add_torrent(std::move(p));
-
-	alert const* a = wait_for_alert(ses, add_torrent_alert::alert_type);
-	TEST_CHECK(a);
-	if (a == nullptr) return;
-	auto const* ta = alert_cast<add_torrent_alert const>(a);
-	TEST_CHECK(ta);
-	if (ta == nullptr) return;
-	TEST_CHECK(!ta->error);
-	TEST_CHECK(ta->params.ti->name() == "temp");
-}
-#endif
-
 TORRENT_TEST(torrent_status)
 {
 	TEST_EQUAL(static_cast<int>(torrent_status::error_file_none), -1);
-#if TORRENT_ABI_VERSION == 1
-	TEST_EQUAL(static_cast<int>(torrent_status::error_file_url), -2);
 	TEST_EQUAL(static_cast<int>(torrent_status::error_file_metadata), -4);
-#endif
 	TEST_EQUAL(static_cast<int>(torrent_status::error_file_ssl_ctx), -3);
 	TEST_EQUAL(static_cast<int>(torrent_status::error_file_exception), -5);
 }
@@ -602,7 +573,8 @@ void test_queue(add_torrent_params)
 		std::stringstream file_path;
 		file_path << "test_torrent_dir4/queue" << i;
 		fs.add_file(file_path.str(), 1024);
-		lt::create_torrent t(fs, 128 * 1024, 6);
+		lt::create_torrent t(fs, 128 * 1024);
+		t.set_hash(0_piece, sha1_hash::max());
 
 		std::vector<char> buf;
 		bencode(std::back_inserter(buf), t.generate());
@@ -736,8 +708,8 @@ TORRENT_TEST(test_have_piece_no_metadata)
 	torrent_handle h = ses.add_torrent(p);
 
 	TEST_EQUAL(h.have_piece(piece_index_t{-1}), false);
-	TEST_EQUAL(h.have_piece(piece_index_t{0}), false);
-	TEST_EQUAL(h.have_piece(piece_index_t{100}), false);
+	TEST_EQUAL(h.have_piece(0_piece), false);
+	TEST_EQUAL(h.have_piece(100_piece), false);
 }
 
 TORRENT_TEST(test_have_piece_out_of_range)
@@ -747,14 +719,18 @@ TORRENT_TEST(test_have_piece_out_of_range)
 	add_torrent_params p;
 	static std::array<const int, 2> const file_sizes{{100000, 100000}};
 	int const piece_size = 0x8000;
-	p.ti = make_torrent(file_sizes, piece_size);
-	p.save_path = "save_path";
+	lt::file_storage fs;
+	fs.set_piece_length(piece_size);
+	create_random_files(".", file_sizes, &fs);
+	p.ti = make_torrent(fs);
+
+	p.save_path = ".";
 	p.flags |= torrent_flags::seed_mode;
 	torrent_handle h = ses.add_torrent(p);
 
 	TEST_EQUAL(h.have_piece(piece_index_t{-1}), false);
-	TEST_EQUAL(h.have_piece(piece_index_t{0}), true);
-	TEST_EQUAL(h.have_piece(piece_index_t{100}), false);
+	TEST_EQUAL(h.have_piece(0_piece), true);
+	TEST_EQUAL(h.have_piece(100_piece), false);
 }
 
 TORRENT_TEST(test_read_piece_no_metadata)
@@ -805,7 +781,7 @@ file_storage test_fs()
 	file_storage fs;
 	fs.set_piece_length(piece_size);
 	fs.add_file("temp", 99999999999);
-	fs.set_num_pieces(int((fs.total_size() + piece_size - 1) / piece_size));
+	fs.set_num_pieces(aux::calc_num_pieces(fs));
 	return fs;
 }
 }
@@ -857,7 +833,7 @@ TORRENT_TEST(symlinks_restore)
 	p.flags &= ~lt::torrent_flags::paused;
 	p.save_path = work_dir;
 	settings_pack pack = settings();
-	pack.set_int(libtorrent::settings_pack::alert_mask, libtorrent::alert_category::status | libtorrent::alert_category::error);
+	pack.set_int(lt::settings_pack::alert_mask, lt::alert_category::status | lt::alert_category::error);
 	lt::session ses(std::move(pack));
 	ses.add_torrent(p);
 
@@ -865,7 +841,7 @@ TORRENT_TEST(symlinks_restore)
 
 	std::string const f = combine_path(combine_path(work_dir, "Some.framework"), "SDL2");
 	TEST_CHECK(aux::get_file_attributes(f) & file_storage::flag_symlink);
-	TEST_CHECK(aux::get_symlink_path(f) == "Versions/A/SDL2");
+	TEST_EQUAL(aux::get_symlink_path(f), "Versions/A/SDL2");
 }
 #endif
 
@@ -873,7 +849,7 @@ TORRENT_TEST(redundant_add_piece)
 {
 	file_storage fs;
 	fs.add_file("tmp1", 128 * 1024 * 8);
-	lt::create_torrent t(fs, 128 * 1024, 6);
+	lt::create_torrent t(fs, 128 * 1024);
 
 	TEST_CHECK(t.num_pieces() > 0);
 
@@ -896,10 +872,10 @@ TORRENT_TEST(redundant_add_piece)
 	auto h = ses.add_torrent(atp);
 	wait_for_downloading(ses, "");
 
-	h.add_piece(piece_index_t{0}, piece_data);
-	h.set_piece_deadline(piece_index_t{0}, 0, torrent_handle::alert_when_available);
+	h.add_piece(0_piece, piece_data.data());
+	h.set_piece_deadline(0_piece, 0, torrent_handle::alert_when_available);
 	h.prioritize_pieces(std::vector<lt::download_priority_t>(std::size_t(ti->num_pieces()), lt::dont_download));
-	h.add_piece(piece_index_t{0}, std::move(piece_data));
+	h.add_piece(0_piece, piece_data.data());
 	std::this_thread::sleep_for(lt::seconds(2));
 }
 
