@@ -1,6 +1,10 @@
 /*
 
-Copyright (c) 2012, Arvid Norberg
+Copyright (c) 2013-2020, Arvid Norberg
+Copyright (c) 2016, 2018, Alden Torres
+Copyright (c) 2017, Pavel Pimenov
+Copyright (c) 2017-2019, Steven Siloti
+Copyright (c) 2019, Andrei Kurushin
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -32,16 +36,16 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include "test.hpp"
 #include "setup_transfer.hpp" // for load_file
+#include "test_utils.hpp"
 #include "libtorrent/file_storage.hpp"
-#include "libtorrent/load_torrent.hpp"
 #include "libtorrent/aux_/path.hpp"
 #include "libtorrent/torrent_info.hpp"
 #include "libtorrent/create_torrent.hpp"
 #include "libtorrent/announce_entry.hpp"
+#include "libtorrent/disk_interface.hpp" // for default_block_size
 #include "libtorrent/aux_/escape_string.hpp" // for convert_path_to_posix
+#include "libtorrent/piece_picker.hpp"
 #include "libtorrent/hex.hpp" // to_hex
-#include "libtorrent/write_resume_data.hpp" // write_torrent_file
-#include "libtorrent/alert_types.hpp"
 
 #include <iostream>
 
@@ -57,9 +61,8 @@ TORRENT_TEST(mutable_torrents)
 	lt::create_torrent t(fs, 0x4000);
 
 	// calculate the hash for all pieces
-	sha1_hash ph;
 	for (auto const i : fs.piece_range())
-		t.set_hash(i, ph);
+		t.set_hash(i, sha1_hash::max());
 
 	t.add_collection("collection1");
 	t.add_collection("collection2");
@@ -67,11 +70,9 @@ TORRENT_TEST(mutable_torrents)
 	t.add_similar_torrent(sha1_hash("abababababababababab"));
 	t.add_similar_torrent(sha1_hash("babababababababababa"));
 
-	std::vector<char> tmp;
-	std::back_insert_iterator<std::vector<char>> out(tmp);
-
 	entry tor = t.generate();
-	bencode(out, tor);
+	std::vector<char> tmp;
+	bencode(std::back_inserter(tmp), tor);
 
 	torrent_info ti(tmp, from_span);
 
@@ -92,21 +93,43 @@ namespace {
 
 struct test_torrent_t
 {
+	test_torrent_t(char const* f, std::function<void(torrent_info*)> t = {}) // NOLINT
+		: file(f), test(std::move(t)) {}
+
 	char const* file;
+	std::function<void(torrent_info*)> test;
 };
 
 using namespace lt;
 
-static test_torrent_t test_torrents[] =
+#ifdef TORRENT_WINDOWS
+#define SEPARATOR "\\"
+#else
+#define SEPARATOR "/"
+#endif
+
+static test_torrent_t const test_torrents[] =
 {
-	{ "base.torrent" },
+	{ "base.torrent"},
 	{ "empty_path.torrent" },
 	{ "parent_path.torrent" },
 	{ "hidden_parent_path.torrent" },
 	{ "single_multi_file.torrent" },
-	{ "slash_path.torrent" },
-	{ "slash_path2.torrent" },
-	{ "slash_path3.torrent" },
+	{ "slash_path.torrent", [](torrent_info const* ti) {
+			TEST_EQUAL(ti->num_files(), 1);
+			TEST_EQUAL(ti->files().file_path(file_index_t{0}), "temp" SEPARATOR "bar");
+		}
+	},
+	{ "slash_path2.torrent", [](torrent_info const* ti) {
+			TEST_EQUAL(ti->num_files(), 1);
+			TEST_EQUAL(ti->files().file_path(file_index_t{0}), "temp" SEPARATOR "abc....def" SEPARATOR "bar");
+		}
+	},
+	{ "slash_path3.torrent", [](torrent_info const* ti) {
+			TEST_EQUAL(ti->num_files(), 1);
+			TEST_EQUAL(ti->files().file_path(file_index_t{0}), "temp....abc");
+		}
+	},
 	{ "backslash_path.torrent" },
 	{ "url_list.torrent" },
 	{ "url_list2.torrent" },
@@ -114,31 +137,204 @@ static test_torrent_t test_torrents[] =
 	{ "httpseed.torrent" },
 	{ "empty_httpseed.torrent" },
 	{ "long_name.torrent" },
-	{ "whitespace_url.torrent" },
-	{ "duplicate_files.torrent" },
-	{ "pad_file.torrent" },
-	{ "creation_date.torrent" },
-	{ "no_creation_date.torrent" },
-	{ "url_seed.torrent" },
-	{ "url_seed_multi.torrent" },
-	{ "url_seed_multi_single_file.torrent" },
-	{ "url_seed_multi_space.torrent" },
-	{ "url_seed_multi_space_nolist.torrent" },
-	{ "root_hash.torrent" },
+	{ "whitespace_url.torrent", [](torrent_info const* ti) {
+			// make sure we trimmed the url
+			TEST_CHECK(ti->trackers().size() > 0);
+			if (ti->trackers().size() > 0)
+				TEST_CHECK(ti->trackers()[0].url == "udp://test.com/announce");
+		}
+	},
+	{ "duplicate_files.torrent", [](torrent_info const* ti) {
+			// make sure we disambiguated the files
+			TEST_EQUAL(ti->num_files(), 2);
+			TEST_CHECK(ti->files().file_path(file_index_t{0}) == combine_path(combine_path("temp", "foo"), "bar.txt"));
+			TEST_CHECK(ti->files().file_path(file_index_t{1}) == combine_path(combine_path("temp", "foo"), "bar.1.txt"));
+		}
+	},
+	{ "pad_file.torrent", [](torrent_info const* ti) {
+			TEST_EQUAL(ti->num_files(), 2);
+			TEST_EQUAL(bool(ti->files().file_flags(file_index_t{0}) & file_storage::flag_pad_file), false);
+			TEST_EQUAL(bool(ti->files().file_flags(file_index_t{1}) & file_storage::flag_pad_file), true);
+		}
+	},
+	{ "creation_date.torrent", [](torrent_info const* ti) {
+			TEST_EQUAL(ti->creation_date(), 1234567);
+		}
+	},
+	{ "no_creation_date.torrent", [](torrent_info const* ti) {
+			TEST_CHECK(!ti->creation_date());
+		}
+	},
+	{ "url_seed.torrent", [](torrent_info const* ti) {
+			TEST_EQUAL(ti->web_seeds().size(), 1);
+			TEST_EQUAL(ti->web_seeds()[0].url, "http://test.com/file");
+#if TORRENT_ABI_VERSION == 1
+			TEST_EQUAL(ti->http_seeds().size(), 0);
+			TEST_EQUAL(ti->url_seeds().size(), 1);
+			TEST_EQUAL(ti->url_seeds()[0], "http://test.com/file");
+#endif
+		}
+	},
+	{ "url_seed_multi.torrent", [](torrent_info const* ti) {
+			TEST_EQUAL(ti->web_seeds().size(), 1);
+			TEST_EQUAL(ti->web_seeds()[0].url, "http://test.com/file/");
+#if TORRENT_ABI_VERSION == 1
+			TEST_EQUAL(ti->http_seeds().size(), 0);
+			TEST_EQUAL(ti->url_seeds().size(), 1);
+			TEST_EQUAL(ti->url_seeds()[0], "http://test.com/file/");
+#endif
+		}
+	},
+	{ "url_seed_multi_single_file.torrent", [](torrent_info const* ti) {
+			TEST_EQUAL(ti->web_seeds().size(), 1);
+			TEST_EQUAL(ti->web_seeds()[0].url, "http://test.com/file/temp/foo/bar.txt");
+		}
+	},
+	{ "url_seed_multi_space.torrent", [](torrent_info const* ti) {
+			TEST_EQUAL(ti->web_seeds().size(), 1);
+			TEST_EQUAL(ti->web_seeds()[0].url, "http://test.com/test%20file/foo%20bar/");
+#if TORRENT_ABI_VERSION == 1
+			TEST_EQUAL(ti->http_seeds().size(), 0);
+			TEST_EQUAL(ti->url_seeds().size(), 1);
+			TEST_EQUAL(ti->url_seeds()[0], "http://test.com/test%20file/foo%20bar/");
+#endif
+		}
+	},
+	{ "url_seed_multi_space_nolist.torrent", [](torrent_info const* ti) {
+			TEST_EQUAL(ti->web_seeds().size(), 1);
+			TEST_EQUAL(ti->web_seeds()[0].url, "http://test.com/test%20file/foo%20bar/");
+#if TORRENT_ABI_VERSION == 1
+			TEST_EQUAL(ti->http_seeds().size(), 0);
+			TEST_EQUAL(ti->url_seeds().size(), 1);
+			TEST_EQUAL(ti->url_seeds()[0], "http://test.com/test%20file/foo%20bar/");
+#endif
+		}
+	},
 	{ "empty_path_multi.torrent" },
-	{ "duplicate_web_seeds.torrent" },
-	{ "invalid_name2.torrent" },
-	{ "invalid_name3.torrent" },
-	{ "symlink1.torrent" },
-	{ "symlink2.torrent" },
+	{ "duplicate_web_seeds.torrent", [](torrent_info const* ti) {
+			TEST_EQUAL(ti->web_seeds().size(), 3);
+		}
+	},
+	{ "invalid_name2.torrent", [](torrent_info const* ti) {
+			// if, after all invalid characters are removed from the name, it ends up
+			// being empty, it's set to the info-hash. Some torrents also have an empty name
+			// in which case it's also set to the info-hash
+			TEST_EQUAL(ti->name(), "b61560c2918f463768cd122b6d2fdd47b77bdb35");
+		}
+	},
+	{ "invalid_name3.torrent", [](torrent_info const* ti) {
+			// windows does not allow trailing spaces in filenames
+#ifdef TORRENT_WINDOWS
+			TEST_EQUAL(ti->name(), "foobar");
+#else
+			TEST_EQUAL(ti->name(), "foobar ");
+#endif
+		}
+	},
+	{ "symlink1.torrent", [](torrent_info const* ti) {
+			TEST_EQUAL(ti->num_files(), 2);
+			TEST_EQUAL(ti->files().symlink(file_index_t{1}), "temp" SEPARATOR "a" SEPARATOR "b" SEPARATOR "bar");
+		}
+	},
+	{ "symlink2.torrent", [](torrent_info const* ti) {
+			TEST_EQUAL(ti->num_files(), 5);
+			TEST_EQUAL(ti->files().symlink(file_index_t{0}), "Some.framework" SEPARATOR "Versions" SEPARATOR "A" SEPARATOR "SDL2");
+			TEST_EQUAL(ti->files().symlink(file_index_t{4}), "Some.framework" SEPARATOR "Versions" SEPARATOR "A");
+		}
+	},
 	{ "unordered.torrent" },
-	{ "symlink_zero_size.torrent" },
-	{ "pad_file_no_path.torrent" },
+	{ "symlink_zero_size.torrent", [](torrent_info const* ti) {
+			TEST_EQUAL(ti->num_files(), 2);
+			TEST_EQUAL(ti->files().symlink(1_file), "temp" SEPARATOR "a" SEPARATOR "b" SEPARATOR "bar");
+		}
+	},
+	{ "pad_file_no_path.torrent", [](torrent_info const* ti) {
+			TEST_EQUAL(ti->num_files(), 2);
+			TEST_EQUAL(ti->files().file_path(file_index_t{1}), combine_path(".pad", "2124"));
+		}
+	},
 	{ "large.torrent" },
-	{ "absolute_filename.torrent" },
-	{ "invalid_filename.torrent" },
-	{ "invalid_filename2.torrent" },
-	{ "overlapping_symlinks.torrent" },
+	{ "absolute_filename.torrent", [](torrent_info const* ti) {
+			TEST_EQUAL(ti->num_files(), 2);
+			TEST_EQUAL(ti->files().file_path(file_index_t{0}), combine_path("temp", "abcde"));
+			TEST_EQUAL(ti->files().file_path(file_index_t{1}), combine_path("temp", "foobar"));
+		}
+	},
+	{ "invalid_filename.torrent", [](torrent_info const* ti) {
+			TEST_EQUAL(ti->num_files(), 2);
+		}
+	},
+	{ "invalid_filename2.torrent", [](torrent_info const* ti) {
+			TEST_EQUAL(ti->num_files(), 3);
+		}
+	},
+	{ "overlapping_symlinks.torrent", [](torrent_info const* ti) {
+			TEST_CHECK(ti->num_files() > 3);
+			TEST_EQUAL(ti->files().symlink(file_index_t{0}), "SDL2.framework" SEPARATOR "Versions" SEPARATOR "Current" SEPARATOR "Headers");
+			TEST_EQUAL(ti->files().symlink(file_index_t{1}), "SDL2.framework" SEPARATOR "Versions" SEPARATOR "Current" SEPARATOR "Resources");
+			TEST_EQUAL(ti->files().symlink(file_index_t{2}), "SDL2.framework" SEPARATOR "Versions" SEPARATOR "Current" SEPARATOR "SDL2");
+		}
+	},
+	{ "v2.torrent", [](torrent_info const* ti) {
+			TEST_EQUAL(ti->num_files(), 1);
+			TEST_EQUAL(ti->files().file_path(file_index_t{ 0 }), "test64K"_sv);
+			TEST_EQUAL(ti->files().file_size(file_index_t{ 0 }), 65536);
+			TEST_EQUAL(aux::to_hex(ti->files().root(file_index_t{ 0 })), "60aae9c7b428f87e0713e88229e18f0adf12cd7b22a0dd8a92bb2485eb7af242"_sv);
+			TEST_EQUAL(ti->info_hashes().has_v1(), true);
+			TEST_EQUAL(ti->info_hashes().has_v2(), true);
+			TEST_EQUAL(aux::to_hex(ti->info_hashes().v2), "597b180c1a170a585dfc5e85d834d69013ceda174b8f357d5bb1a0ca509faf0a"_sv);
+			TEST_CHECK(ti->v2());
+			TEST_CHECK(ti->v1());
+		}
+	},
+	{ "v2_multipiece_file.torrent", [](torrent_info const* ti) {
+			TEST_EQUAL(ti->num_files(), 1);
+			TEST_EQUAL(ti->files().file_path(file_index_t{ 0 }), "test1MB"_sv);
+			TEST_EQUAL(ti->files().file_size(file_index_t{ 0 }), 1048576);
+			TEST_EQUAL(aux::to_hex(ti->files().root(file_index_t{ 0 })), "515ea9181744b817744ded9d2e8e9dc6a8450c0b0c52e24b5077f302ffbd9008"_sv);
+			TEST_EQUAL(ti->info_hashes().has_v1(), true);
+			TEST_EQUAL(ti->info_hashes().has_v2(), true);
+			TEST_EQUAL(aux::to_hex(ti->info_hashes().v2), "108ac2c3718ce722e6896edc56c4afa98f1d711ecaace7aad74fca418ebd03de"_sv);
+		}
+	},
+	{ "v2_only.torrent", [](torrent_info const* ti) {
+			TEST_EQUAL(ti->num_files(), 1);
+			TEST_EQUAL(ti->files().file_path(file_index_t{ 0 }), "test1MB"_sv);
+			TEST_EQUAL(ti->files().file_size(file_index_t{ 0 }), 1048576);
+			TEST_EQUAL(aux::to_hex(ti->files().root(file_index_t{ 0 })), "515ea9181744b817744ded9d2e8e9dc6a8450c0b0c52e24b5077f302ffbd9008"_sv);
+			TEST_EQUAL(ti->info_hashes().has_v1(), false);
+			TEST_EQUAL(ti->info_hashes().has_v2(), true);
+			TEST_EQUAL(aux::to_hex(ti->info_hashes().v2), "95e04d0c4bad94ab206efa884666fd89777dbe4f7bd9945af1829037a85c6192"_sv);
+			TEST_CHECK(ti->v2());
+			TEST_CHECK(!ti->v1());
+		}
+	},
+	{ "v2_invalid_filename.torrent", [](torrent_info const* ti) {
+			TEST_EQUAL(ti->num_files(), 1);
+			TEST_EQUAL(ti->files().file_path(file_index_t{0}), "_estMB"_sv);
+		}
+	},
+	{ "v2_multiple_files.torrent", [](torrent_info* ti) {
+			TEST_EQUAL(ti->v2_piece_hashes_verified(), true);
+			TEST_EQUAL(ti->num_files(), 4);
+			TEST_CHECK(ti->v2());
+			ti->free_piece_layers();
+			TEST_CHECK(ti->v2());
+			TEST_EQUAL(ti->v2_piece_hashes_verified(), false);
+		}
+	},
+	{ "v2_symlinks.torrent", [](torrent_info const* ti) {
+			TEST_CHECK(ti->num_files() > 3);
+			TEST_EQUAL(ti->files().symlink(0_file), "SDL2.framework" SEPARATOR "Versions" SEPARATOR "Current" SEPARATOR "Headers");
+			TEST_EQUAL(ti->files().symlink(1_file), "SDL2.framework" SEPARATOR "Versions" SEPARATOR "Current" SEPARATOR "Resources");
+			TEST_EQUAL(ti->files().symlink(2_file), "SDL2.framework" SEPARATOR "Versions" SEPARATOR "Current" SEPARATOR "SDL2");
+		}
+	},
+	{ "v2_hybrid.torrent", [](torrent_info const* ti) {
+			TEST_CHECK(ti->info_hashes().has_v1());
+			TEST_CHECK(ti->info_hashes().has_v2());
+		}
+	},
 };
 
 struct test_failing_torrent_t
@@ -163,31 +359,44 @@ test_failing_torrent_t test_error_torrents[] =
 	{ "missing_path_list.torrent", errors::torrent_missing_name },
 	{ "invalid_pieces.torrent", errors::torrent_missing_pieces },
 	{ "unaligned_pieces.torrent", errors::torrent_invalid_hashes },
-	{ "invalid_root_hash.torrent", errors::torrent_invalid_hashes },
-	{ "invalid_root_hash2.torrent", errors::torrent_missing_pieces },
-	{ "invalid_merkle.torrent", errors::no_files_in_torrent},
 	{ "invalid_file_size.torrent", errors::torrent_invalid_length },
 	{ "invalid_symlink.torrent", errors::torrent_invalid_name },
 	{ "many_pieces.torrent", errors::too_many_pieces_in_torrent },
 	{ "no_files.torrent", errors::no_files_in_torrent},
 	{ "zero.torrent", errors::torrent_invalid_length},
 	{ "zero2.torrent", errors::torrent_invalid_length},
+	{ "v2_mismatching_metadata.torrent", errors::torrent_inconsistent_files},
+	{ "v2_no_power2_piece.torrent", errors::torrent_missing_piece_length},
+	{ "v2_invalid_file.torrent", errors::torrent_file_parse_failed},
+	{ "v2_deep_recursion.torrent", errors::torrent_file_parse_failed},
+	{ "v2_non_multiple_piece_layer.torrent", errors::torrent_missing_piece_layer},
+	{ "v2_piece_layer_invalid_file_hash.torrent", errors::torrent_missing_piece_layer},
+	{ "v2_invalid_piece_layer.torrent", errors::torrent_missing_piece_layer},
+	{ "v2_invalid_piece_layer_size.torrent", errors::torrent_invalid_piece_layer},
+	{ "v2_bad_file_alignment.torrent", errors::torrent_inconsistent_files},
+	{ "v2_unordered_files.torrent", errors::invalid_bencoding},
+	{ "v2_overlong_integer.torrent", errors::invalid_bencoding},
+	{ "v2_missing_file_root_invalid_symlink.torrent", errors::torrent_missing_pieces_root},
+	{ "v2_large_file.torrent", errors::torrent_invalid_length},
+	{ "v2_no_piece_layers.torrent", errors::torrent_missing_piece_layer},
+	{ "v2_large_offset.torrent", errors::too_many_pieces_in_torrent},
+	{ "v2_piece_size.torrent", errors::torrent_missing_piece_length},
+	{ "v2_invalid_pad_file.torrent", errors::torrent_invalid_pad_file},
+	{ "v2_zero_root.torrent", errors::torrent_missing_pieces_root},
+	{ "v2_zero_root_small.torrent", errors::torrent_missing_pieces_root},
 };
 
 } // anonymous namespace
 
 // TODO: test remap_files
-// TODO: merkle torrents. specifically torrent_info::add_merkle_nodes and torrent with "root hash"
 // TODO: torrent with 'p' (padfile) attribute
 // TODO: torrent with 'h' (hidden) attribute
 // TODO: torrent with 'x' (executable) attribute
 // TODO: torrent with 'l' (symlink) attribute
-// TODO: creating a merkle torrent (torrent_info::build_merkle_list)
 // TODO: torrent with multiple trackers in multiple tiers, making sure we
 // shuffle them (how do you test shuffling?, load it multiple times and make
 // sure it's in different order at least once)
 // TODO: torrents with a zero-length name
-// TODO: torrents with a merkle tree and add_merkle_nodes
 // TODO: torrent with a non-dictionary info-section
 // TODO: torrents with DHT nodes
 // TODO: torrent with url-list as a single string
@@ -195,10 +404,24 @@ test_failing_torrent_t test_error_torrents[] =
 // TODO: torrent with a comment
 // TODO: torrent with an SSL cert
 // TODO: torrent with attributes (executable and hidden)
-// TODO: torrent_info::add_tracker
 // TODO: torrent_info constructor that takes an invalid bencoded buffer
 // TODO: verify_encoding with a string that triggers character replacement
 
+TORRENT_TEST(add_tracker)
+{
+	torrent_info ti(info_hash_t(sha1_hash("                   ")));
+	TEST_EQUAL(ti.trackers().size(), 0);
+
+	ti.add_tracker("http://test.com/announce");
+	TEST_EQUAL(ti.trackers().size(), 1);
+
+	announce_entry ae = ti.trackers()[0];
+	TEST_EQUAL(ae.url, "http://test.com/announce");
+
+	ti.clear_trackers();
+	TEST_EQUAL(ti.trackers().size(), 0);
+}
+
 TORRENT_TEST(url_list_and_httpseeds)
 {
 	entry info;
@@ -224,7 +447,7 @@ TORRENT_TEST(url_list_and_httpseeds)
 
 TORRENT_TEST(add_url_seed)
 {
-	torrent_info ti(sha1_hash("                   "));
+	torrent_info ti(info_hash_t(sha1_hash("                   ")));
 	TEST_EQUAL(ti.web_seeds().size(), 0);
 
 	ti.add_url_seed("http://test.com");
@@ -237,7 +460,7 @@ TORRENT_TEST(add_url_seed)
 
 TORRENT_TEST(add_http_seed)
 {
-	torrent_info ti(sha1_hash("                   "));
+	torrent_info ti(info_hash_t(sha1_hash("                   ")));
 	TEST_EQUAL(ti.web_seeds().size(), 0);
 
 	ti.add_http_seed("http://test.com");
@@ -250,7 +473,7 @@ TORRENT_TEST(add_http_seed)
 
 TORRENT_TEST(set_web_seeds)
 {
-	torrent_info ti(sha1_hash("                   "));
+	torrent_info ti(info_hash_t(sha1_hash("                   ")));
 	TEST_EQUAL(ti.web_seeds().size(), 0);
 
 	std::vector<web_seed_entry> seeds;
@@ -265,14 +488,10 @@ TORRENT_TEST(set_web_seeds)
 	TEST_CHECK(ti.web_seeds() == seeds);
 }
 
-#ifdef TORRENT_WINDOWS
-#define SEPARATOR "\\"
-#else
-#define SEPARATOR "/"
-#endif
-
 TORRENT_TEST(sanitize_path_truncate)
 {
+	using lt::aux::sanitize_append_path_element;
+
 	std::string path;
 	sanitize_append_path_element(path,
 		"abcdefghi_abcdefghi_abcdefghi_abcdefghi_abcdefghi_"
@@ -301,6 +520,8 @@ TORRENT_TEST(sanitize_path_truncate)
 
 TORRENT_TEST(sanitize_path_truncate_utf)
 {
+	using lt::aux::sanitize_append_path_element;
+
 	std::string path;
 	// msvc doesn't like unicode string literals, so we encode it as UTF-8 explicitly
 	sanitize_append_path_element(path,
@@ -320,6 +541,7 @@ TORRENT_TEST(sanitize_path_truncate_utf)
 TORRENT_TEST(sanitize_path_trailing_dots)
 {
 	std::string path;
+	using lt::aux::sanitize_append_path_element;
 	sanitize_append_path_element(path, "a");
 	sanitize_append_path_element(path, "abc...");
 	sanitize_append_path_element(path, "c");
@@ -357,6 +579,7 @@ TORRENT_TEST(sanitize_path_trailing_dots)
 
 TORRENT_TEST(sanitize_path_trailing_spaces)
 {
+	using lt::aux::sanitize_append_path_element;
 	std::string path;
 	sanitize_append_path_element(path, "a");
 	sanitize_append_path_element(path, "abc   ");
@@ -386,6 +609,7 @@ TORRENT_TEST(sanitize_path_trailing_spaces)
 
 TORRENT_TEST(sanitize_path)
 {
+	using lt::aux::sanitize_append_path_element;
 	std::string path;
 	sanitize_append_path_element(path, "\0\0\xed\0\x80");
 	TEST_EQUAL(path, "_");
@@ -573,6 +797,7 @@ TORRENT_TEST(sanitize_path)
 
 TORRENT_TEST(sanitize_path_zeroes)
 {
+	using lt::aux::sanitize_append_path_element;
 	std::string path;
 	sanitize_append_path_element(path, "\0foo");
 	TEST_EQUAL(path, "_");
@@ -584,6 +809,7 @@ TORRENT_TEST(sanitize_path_zeroes)
 
 TORRENT_TEST(sanitize_path_colon)
 {
+	using lt::aux::sanitize_append_path_element;
 	std::string path;
 	sanitize_append_path_element(path, "foo:bar");
 #ifdef TORRENT_WINDOWS
@@ -595,6 +821,8 @@ TORRENT_TEST(sanitize_path_colon)
 
 TORRENT_TEST(verify_encoding)
 {
+	using aux::verify_encoding;
+
 	// verify_encoding
 	std::string test = "\b?filename=4";
 	TEST_CHECK(verify_encoding(test));
@@ -724,7 +952,7 @@ TORRENT_TEST(parse_torrents)
 	TEST_EQUAL(ti3.name(), "test2..test3.......test4");
 
 	std::string root_dir = parent_path(current_working_directory());
-	for (auto const t : test_torrents)
+	for (auto const& t : test_torrents)
 	{
 		std::printf("loading %s\n", t.file);
 		std::string filename = combine_path(combine_path(root_dir, "test_torrents")
@@ -735,153 +963,25 @@ TORRENT_TEST(parse_torrents)
 		if (ec) std::printf(" loading(\"%s\") -> failed %s\n", filename.c_str()
 			, ec.message().c_str());
 
-		add_torrent_params atp = load_torrent_file(filename);
-		TEST_CHECK(atp.info_hash == ti->info_hash());
+		// construct a piece_picker to get some more test coverage. Perhaps
+		// loading the torrent is fine, but if we can't construct a piece_picker
+		// for it, it's still no good.
+		int const block_size = std::min(ti->piece_length(), default_block_size);
+		int const blocks_per_piece
+			= (ti->piece_length() + block_size - 1) / block_size;
+		int const blocks_in_last_piece
+			= ((ti->total_size() % ti->piece_length())
+			+ block_size - 1) / block_size;
+		piece_picker pp(blocks_per_piece, blocks_in_last_piece, ti->num_pieces());
 
-		if (t.file == "whitespace_url.torrent"_sv)
-		{
-			// make sure we trimmed the url
-			TEST_CHECK(ti->trackers().size() > 0);
-			if (ti->trackers().size() > 0)
-				TEST_CHECK(ti->trackers()[0].url == "udp://test.com/announce");
-		}
-		else if (t.file == "duplicate_files.torrent"_sv)
-		{
-			// make sure we disambiguated the files
-			TEST_EQUAL(ti->num_files(), 2);
-			TEST_CHECK(ti->files().file_path(file_index_t{0}) == combine_path(combine_path("temp", "foo"), "bar.txt"));
-			TEST_CHECK(ti->files().file_path(file_index_t{1}) == combine_path(combine_path("temp", "foo"), "bar.1.txt"));
-		}
-		else if (t.file == "pad_file.torrent"_sv)
-		{
-			TEST_EQUAL(ti->num_files(), 2);
-			TEST_EQUAL(bool(ti->files().file_flags(file_index_t{0}) & file_storage::flag_pad_file), false);
-			TEST_EQUAL(bool(ti->files().file_flags(file_index_t{1}) & file_storage::flag_pad_file), true);
-		}
-		else if (t.file == "creation_date.torrent"_sv)
-		{
-			TEST_EQUAL(ti->creation_date(), 1234567);
-		}
-		else if (t.file == "duplicate_web_seeds.torrent"_sv)
-		{
-			TEST_EQUAL(ti->web_seeds().size(), 3);
-		}
-		else if (t.file == "no_creation_date.torrent"_sv)
-		{
-			TEST_CHECK(!ti->creation_date());
-		}
-		else if (t.file == "url_seed.torrent"_sv)
-		{
-			TEST_EQUAL(ti->web_seeds().size(), 1);
-			TEST_EQUAL(ti->web_seeds()[0].url, "http://test.com/file");
-#if TORRENT_ABI_VERSION == 1
-			TEST_EQUAL(ti->http_seeds().size(), 0);
-			TEST_EQUAL(ti->url_seeds().size(), 1);
-			TEST_EQUAL(ti->url_seeds()[0], "http://test.com/file");
-#endif
-		}
-		else if (t.file == "url_seed_multi.torrent"_sv)
-		{
-			TEST_EQUAL(ti->web_seeds().size(), 1);
-			TEST_EQUAL(ti->web_seeds()[0].url, "http://test.com/file/");
-#if TORRENT_ABI_VERSION == 1
-			TEST_EQUAL(ti->http_seeds().size(), 0);
-			TEST_EQUAL(ti->url_seeds().size(), 1);
-			TEST_EQUAL(ti->url_seeds()[0], "http://test.com/file/");
-#endif
-		}
-		else if (t.file == "url_seed_multi_single_file.torrent"_sv)
-		{
-			TEST_EQUAL(ti->web_seeds().size(), 1);
-			TEST_EQUAL(ti->web_seeds()[0].url, "http://test.com/file/temp/foo/bar.txt");
-		}
-		else if (t.file == "url_seed_multi_space.torrent"_sv
-			|| t.file == "url_seed_multi_space_nolist.torrent"_sv)
-		{
-			TEST_EQUAL(ti->web_seeds().size(), 1);
-			TEST_EQUAL(ti->web_seeds()[0].url, "http://test.com/test%20file/foo%20bar/");
-#if TORRENT_ABI_VERSION == 1
-			TEST_EQUAL(ti->http_seeds().size(), 0);
-			TEST_EQUAL(ti->url_seeds().size(), 1);
-			TEST_EQUAL(ti->url_seeds()[0], "http://test.com/test%20file/foo%20bar/");
-#endif
-		}
-		else if (t.file == "invalid_name2.torrent"_sv)
-		{
-			// if, after all invalid characters are removed from the name, it ends up
-			// being empty, it's set to the info-hash. Some torrents also have an empty name
-			// in which case it's also set to the info-hash
-			TEST_EQUAL(ti->name(), "b61560c2918f463768cd122b6d2fdd47b77bdb35");
-		}
-		else if (t.file == "invalid_name3.torrent"_sv)
-		{
-			// windows does not allow trailing spaces in filenames
-#ifdef TORRENT_WINDOWS
-			TEST_EQUAL(ti->name(), "foobar");
-#else
-			TEST_EQUAL(ti->name(), "foobar ");
-#endif
-		}
-		else if (t.file == "symlink1.torrent"_sv)
-		{
-			TEST_EQUAL(ti->num_files(), 2);
-			TEST_EQUAL(ti->files().symlink(file_index_t{1}), "temp" SEPARATOR "a" SEPARATOR "b" SEPARATOR "bar");
-		}
-		else if (t.file == "symlink2.torrent"_sv)
-		{
-			TEST_EQUAL(ti->num_files(), 5);
-			TEST_EQUAL(ti->files().symlink(file_index_t{0}), "Some.framework" SEPARATOR "Versions" SEPARATOR "A" SEPARATOR "SDL2");
-			TEST_EQUAL(ti->files().symlink(file_index_t{4}), "Some.framework" SEPARATOR "Versions" SEPARATOR "A");
-		}
-		else if (t.file == "slash_path.torrent"_sv)
-		{
-			TEST_EQUAL(ti->num_files(), 1);
-			TEST_EQUAL(ti->files().file_path(file_index_t{0}), "temp" SEPARATOR "bar");
-		}
-		else if (t.file == "slash_path2.torrent"_sv)
-		{
-			TEST_EQUAL(ti->num_files(), 1);
-			TEST_EQUAL(ti->files().file_path(file_index_t{0}), "temp" SEPARATOR "abc....def" SEPARATOR "bar");
-		}
-		else if (t.file == "slash_path3.torrent"_sv)
-		{
-			TEST_EQUAL(ti->num_files(), 1);
-			TEST_EQUAL(ti->files().file_path(file_index_t{0}), "temp....abc");
-		}
-		else if (t.file == "symlink_zero_size.torrent"_sv)
-		{
-			TEST_EQUAL(ti->num_files(), 2);
-			TEST_EQUAL(ti->files().symlink(file_index_t(1)), "temp" SEPARATOR "a" SEPARATOR "b" SEPARATOR "bar");
-		}
-		else if (t.file == "pad_file_no_path.torrent"_sv)
-		{
-			TEST_EQUAL(ti->num_files(), 2);
-			TEST_EQUAL(ti->files().file_path(file_index_t{1}), combine_path(".pad", "0"));
-		}
-		else if (t.file == "absolute_filename.torrent"_sv)
-		{
-			TEST_EQUAL(ti->num_files(), 2);
-			TEST_EQUAL(ti->files().file_path(file_index_t{0}), combine_path("temp", "abcde"));
-			TEST_EQUAL(ti->files().file_path(file_index_t{1}), combine_path("temp", "foobar"));
-		}
-		else if (t.file == "invalid_filename.torrent"_sv)
-		{
-			TEST_EQUAL(ti->num_files(), 2);
-		}
-		else if (t.file == "invalid_filename2.torrent"_sv)
-		{
-			TEST_EQUAL(ti->num_files(), 3);
-		}
-		else if (t.file == "overlapping_symlinks.torrent"_sv)
-		{
-			TEST_CHECK(ti->num_files() > 3);
-			TEST_EQUAL(ti->files().symlink(file_index_t{0}), "SDL2.framework" SEPARATOR "Versions" SEPARATOR "Current" SEPARATOR "Headers");
-			TEST_EQUAL(ti->files().symlink(file_index_t{1}), "SDL2.framework" SEPARATOR "Versions" SEPARATOR "Current" SEPARATOR "Resources");
-			TEST_EQUAL(ti->files().symlink(file_index_t{2}), "SDL2.framework" SEPARATOR "Versions" SEPARATOR "Current" SEPARATOR "SDL2");
-		}
+		TEST_CHECK(ti->piece_length() < std::numeric_limits<int>::max() / 2);
+		TEST_EQUAL(ti->v1(), ti->info_hashes().has_v1());
+		TEST_EQUAL(ti->v2(), ti->info_hashes().has_v2());
+
+		if (t.test) t.test(ti.get());
 
 		file_storage const& fs = ti->files();
-		for (file_index_t idx{0}; idx != file_index_t(fs.num_files()); ++idx)
+		for (file_index_t const idx : fs.file_range())
 		{
 			piece_index_t const first = ti->map_file(idx, 0, 0).piece;
 			piece_index_t const last = ti->map_file(idx, std::max(fs.file_size(idx)-1, std::int64_t(0)), 0).piece;
@@ -901,30 +1001,26 @@ TORRENT_TEST(parse_torrents)
 				, flags & file_storage::flag_symlink ? fs.symlink(idx).c_str() : "");
 		}
 	}
+}
 
+TORRENT_TEST(parse_invalid_torrents)
+{
+	std::string const root_dir = parent_path(current_working_directory());
 	for (auto const& e : test_error_torrents)
 	{
 		error_code ec;
 		std::printf("loading %s\n", e.file);
-		std::string const filename = combine_path(
-			combine_path(root_dir, "test_torrents"), e.file);
-		auto ti = std::make_shared<torrent_info>(filename, ec);
+		std::vector<char> data;
+		std::string const filename = combine_path(combine_path(root_dir, "test_torrents")
+			, e.file);
+		TEST_CHECK(load_file(filename, data, ec) == 0);
+		TEST_CHECK(!ec);
+
+		auto ti = std::make_shared<torrent_info>(bdecode(data, 1000), ec);
 		std::printf("E:        \"%s\"\nexpected: \"%s\"\n", ec.message().c_str()
 			, e.error.message().c_str());
-		TEST_CHECK(ec.message() == e.error.message());
+		TEST_EQUAL(ec.message(), e.error.message());
 		TEST_EQUAL(ti->is_valid(), false);
-
-		try
-		{
-			add_torrent_params atp = load_torrent_file(filename);
-			TEST_CHECK(false);
-		}
-		catch (system_error const& err)
-		{
-			std::printf("E:        \"%s\"\nexpected: \"%s\"\n", err.code().message().c_str()
-				, e.error.message().c_str());
-			TEST_EQUAL(err.code().message(), e.error.message());
-		}
 	}
 }
 
@@ -1031,12 +1127,12 @@ void test_resolve_duplicates(aux::vector<file_t, file_index_t> const& test)
 	file_storage fs;
 	for (auto const& f : test) fs.add_file(f.filename, f.size, f.flags);
 
-	lt::create_torrent t(fs, 0x4000);
+	// This test creates torrents with duplicate (identical) filenames, which
+	// isn't supported by v2 torrents, so we can only test this with v1 torrents
+	lt::create_torrent t(fs, 0x4000, create_torrent::v1_only);
 
-	// calculate the hash for all pieces
-	sha1_hash ph;
 	for (auto const i : fs.piece_range())
-		t.set_hash(i, ph);
+		t.set_hash(i, sha1_hash::max());
 
 	std::vector<char> tmp;
 	std::back_insert_iterator<std::vector<char>> out(tmp);
@@ -1118,7 +1214,6 @@ TORRENT_TEST(copy)
 
 	// copy the torrent_info object
 	std::shared_ptr<torrent_info> b = std::make_shared<torrent_info>(*a);
-
 	a.reset();
 
 	TEST_EQUAL(b->num_files(), 3);
@@ -1151,119 +1246,49 @@ TORRENT_TEST(copy_ptr)
 	TEST_EQUAL(b->val, 4);
 }
 
-TORRENT_TEST(write_torrent_file_session_roundtrip)
+TORRENT_TEST(torrent_info_with_hashes_roundtrip)
 {
-	std::string const root_dir = combine_path(parent_path(current_working_directory()), "test_torrents");
+	std::string const root_dir = parent_path(current_working_directory());
+	std::string const filename = combine_path(combine_path(root_dir, "test_torrents"), "v2_only.torrent");
 
-	auto const files = {
-		"base.torrent",
-		"empty_path.torrent",
-		"parent_path.torrent",
-		"hidden_parent_path.torrent",
-		"single_multi_file.torrent",
-		"slash_path.torrent",
-		"slash_path2.torrent",
-		"slash_path3.torrent",
-		"backslash_path.torrent",
-		"long_name.torrent",
-		"duplicate_files.torrent",
-		"pad_file.torrent",
-		"creation_date.torrent",
-		"no_creation_date.torrent",
-		"url_seed.torrent",
-		"url_seed_multi_single_file.torrent",
-		"empty_path_multi.torrent",
-		"invalid_name2.torrent",
-		"invalid_name3.torrent",
-		"symlink1.torrent",
-		"symlink2.torrent",
-		"unordered.torrent",
-		"symlink_zero_size.torrent",
-		"pad_file_no_path.torrent",
-		"large.torrent",
-		"absolute_filename.torrent",
-		"invalid_filename.torrent",
-		"invalid_filename2.torrent",
-		"overlapping_symlinks.torrent",
-	};
+	error_code ec;
+	std::vector<char> data;
+	TEST_CHECK(load_file(filename, data, ec) == 0);
+
+	auto ti = std::make_shared<torrent_info>(data, ec, from_span);
+	TEST_CHECK(!ec);
+	if (ec) std::printf(" loading(\"%s\") -> failed %s\n", filename.c_str()
+		, ec.message().c_str());
+
+	TEST_CHECK(ti->v2());
+	TEST_CHECK(!ti->v1());
+	TEST_EQUAL(ti->v2_piece_hashes_verified(), true);
+
+	add_torrent_params atp;
+	atp.ti = ti;
+	atp.save_path = ".";
+
+	session ses;
+	torrent_handle h = ses.add_torrent(atp);
+
+	TEST_CHECK(ti->v2());
+	TEST_CHECK(!ti->v1());
 
-	for (auto const& name : files)
 	{
-		std::string const filename = combine_path(root_dir, name);
-
-		std::printf("loading(\"%s\")\n", name);
-		error_code ec;
-		std::vector<char> data;
-		TEST_CHECK(load_file(filename, data, ec) == 0);
-
-		// apparently the test files have a garbage character at the end
-		// on windows, it can even be two bytes. Perhaps if git checks them out
-		// as text files and adds newline
-		if (!data.empty() && data.back() != 'e')
-			data.resize(data.size() - 1);
-
-		if (!data.empty() && data.back() != 'e')
-			data.resize(data.size() - 1);
-
-		auto ti = std::make_shared<torrent_info>(data, ec, from_span);
-		TEST_CHECK(!ec);
-		if (ec) std::printf(" -> failed %s\n", ec.message().c_str());
-
-		add_torrent_params atp;
-		atp.ti = ti;
-		atp.save_path = ".";
-
-		session ses;
-		torrent_handle h = ses.add_torrent(atp);
-
-		h.save_resume_data(torrent_handle::save_info_dict);
-		alert const* a = wait_for_alert(ses, save_resume_data_alert::alert_type);
-
-		TORRENT_ASSERT(a);
-		{
-			auto const& p = static_cast<save_resume_data_alert const*>(a)->params;
-			entry e = write_torrent_file(p, write_flags::include_dht_nodes);
-			std::vector<char> out_buffer;
-			bencode(std::back_inserter(out_buffer), e);
-
-			TEST_CHECK(out_buffer == write_torrent_file_buf(p, write_flags::include_dht_nodes));
-
-			if (out_buffer != data)
-			{
-				std::cout << "GOT:\n";
-				for (char b : out_buffer)
-					std::cout << (std::isprint(std::uint8_t(b)) ? b : '.');
-				std::cout << '\n';
-
-				std::cout << "EXPECTED:\n";
-				for (char b : data)
-					std::cout << (std::isprint(std::uint8_t(b)) ? b : '.');
-				std::cout << '\n';
-			}
-			TEST_CHECK(out_buffer == data);
-		}
-
-		{
-			add_torrent_params p = load_torrent_file(filename);
-			entry e = write_torrent_file(p, write_flags::include_dht_nodes);
-			std::vector<char> out_buffer;
-			bencode(std::back_inserter(out_buffer), e);
-
-			TEST_CHECK(out_buffer == write_torrent_file_buf(p, write_flags::include_dht_nodes));
-
-			if (out_buffer != data)
-			{
-				std::cout << "GOT:\n";
-				for (char b : out_buffer)
-					std::cout << (std::isprint(std::uint8_t(b)) ? b : '.');
-				std::cout << '\n';
-
-				std::cout << "EXPECTED:\n";
-				for (char b : data)
-					std::cout << (std::isprint(std::uint8_t(b)) ? b : '.');
-				std::cout << '\n';
-			}
-			TEST_CHECK(out_buffer == data);
-		}
+		auto ti2 = h.torrent_file();
+		TEST_CHECK(ti2->v2());
+		TEST_CHECK(!ti2->v1());
+		TEST_EQUAL(ti2->v2_piece_hashes_verified(), false);
 	}
+
+	ti = h.torrent_file_with_hashes();
+
+	TEST_CHECK(ti->v2());
+	TEST_CHECK(!ti->v1());
+	TEST_EQUAL(ti->v2_piece_hashes_verified(), true);
+
+	std::vector<char> out_buffer = serialize(*ti);
+
+	TEST_EQUAL(out_buffer, data);
 }
+
