@@ -1,6 +1,11 @@
 /*
 
-Copyright (c) 2010, Arvid Norberg
+Copyright (c) 2011-2020, Arvid Norberg
+Copyright (c) 2015, Mikhail Titov
+Copyright (c) 2016, terry zhao
+Copyright (c) 2016-2018, Alden Torres
+Copyright (c) 2017-2018, Steven Siloti
+Copyright (c) 2018, d-komarov
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -31,14 +36,15 @@ POSSIBILITY OF SUCH DAMAGE.
 */
 
 #include "test.hpp"
-#include "setup_transfer.hpp"
+#include "setup_transfer.hpp"  // for supports_ipv6
+#include "test_utils.hpp"
 #include "udp_tracker.hpp"
 #include "settings.hpp"
 #include "libtorrent/alert.hpp"
 #include "libtorrent/peer_info.hpp" // for peer_list_entry
-#include "libtorrent/broadcast_socket.hpp" // for supports_ipv6
 #include "libtorrent/alert_types.hpp"
 #include "libtorrent/session.hpp"
+#include "libtorrent/session_params.hpp"
 #include "libtorrent/error_code.hpp"
 #include "libtorrent/tracker_manager.hpp"
 #include "libtorrent/http_tracker_connection.hpp" // for parse_tracker_response
@@ -48,8 +54,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/aux_/path.hpp"
 #include "libtorrent/socket_io.hpp"
 
-#include <fstream>
-
 using namespace lt;
 
 // TODO: test scrape requests
@@ -71,7 +75,7 @@ TORRENT_TEST(parse_hostname_peers)
 		"7:peer id20:bbbbabaababababababa2:ip12:another_host4:porti1001eeee";
 	error_code ec;
 	tracker_response resp = parse_tracker_response(response
-		, ec, false, sha1_hash());
+		, ec, {}, sha1_hash());
 
 	TEST_EQUAL(ec, error_code());
 	TEST_EQUAL(resp.peers.size(), 2);
@@ -95,7 +99,7 @@ TORRENT_TEST(parse_peers4)
 		"\x09\x08\x07\x06\x20\x10" "e";
 	error_code ec;
 	tracker_response resp = parse_tracker_response(response
-		, ec, false, sha1_hash());
+		, ec, {}, sha1_hash());
 
 	TEST_EQUAL(ec, error_code());
 	TEST_EQUAL(resp.peers4.size(), 2);
@@ -180,7 +184,7 @@ TORRENT_TEST(parse_interval)
 	char const response[] = "d8:intervali1042e12:min intervali10e5:peers0:e";
 	error_code ec;
 	tracker_response resp = parse_tracker_response(response
-		, ec, false, sha1_hash());
+		, ec, {}, sha1_hash());
 
 	TEST_EQUAL(ec, error_code());
 	TEST_EQUAL(resp.peers.size(), 0);
@@ -194,7 +198,7 @@ TORRENT_TEST(parse_warning)
 	char const response[] = "d5:peers0:15:warning message12:test messagee";
 	error_code ec;
 	tracker_response resp = parse_tracker_response(response
-		, ec, false, sha1_hash());
+		, ec, {}, sha1_hash());
 
 	TEST_EQUAL(ec, error_code());
 	TEST_EQUAL(resp.peers.size(), 0);
@@ -206,7 +210,7 @@ TORRENT_TEST(parse_failure_reason)
 	char const response[] = "d5:peers0:14:failure reason12:test messagee";
 	error_code ec;
 	tracker_response resp = parse_tracker_response(response
-		, ec, false, sha1_hash());
+		, ec, {}, sha1_hash());
 
 	TEST_EQUAL(ec, errors::tracker_failure);
 	TEST_EQUAL(resp.peers.size(), 0);
@@ -219,7 +223,7 @@ TORRENT_TEST(parse_scrape_response)
 		"8:completei1e10:incompletei2e10:downloadedi3e11:downloadersi6eeee";
 	error_code ec;
 	tracker_response resp = parse_tracker_response(response
-		, ec, true, sha1_hash("aaaaaaaaaaaaaaaaaaaa"));
+		, ec, tracker_request::scrape_request, sha1_hash("aaaaaaaaaaaaaaaaaaaa"));
 
 	TEST_EQUAL(ec, error_code());
 	TEST_EQUAL(resp.complete, 1);
@@ -234,7 +238,7 @@ TORRENT_TEST(parse_scrape_response_with_zero)
 		"8:completei4e10:incompletei5e10:downloadedi6eeee";
 	error_code ec;
 	tracker_response resp = parse_tracker_response(response
-		, ec, true, sha1_hash("aaa\0aaaaaaaaaaaaaaaa"));
+		, ec, tracker_request::scrape_request, sha1_hash("aaa\0aaaaaaaaaaaaaaaa"));
 
 	TEST_EQUAL(ec, error_code());
 	TEST_EQUAL(resp.complete, 4);
@@ -248,7 +252,7 @@ TORRENT_TEST(parse_external_ip)
 	char const response[] = "d5:peers0:11:external ip4:\x01\x02\x03\x04" "e";
 	error_code ec;
 	tracker_response resp = parse_tracker_response(response
-		, ec, false, sha1_hash());
+		, ec, {}, sha1_hash());
 
 	TEST_EQUAL(ec, error_code());
 	TEST_EQUAL(resp.peers.size(), 0);
@@ -261,7 +265,7 @@ TORRENT_TEST(parse_external_ip6)
 		"16:\xf1\x02\x03\x04\0\0\0\0\0\0\0\0\0\0\xff\xff" "e";
 	error_code ec;
 	tracker_response resp = parse_tracker_response(response
-		, ec, false, sha1_hash());
+		, ec, {}, sha1_hash());
 
 	TEST_EQUAL(ec, error_code());
 	TEST_EQUAL(resp.peers.size(), 0);
@@ -347,12 +351,12 @@ void test_udp_tracker(std::string const& iface, address tracker, tcp::endpoint c
 	pack.set_bool(settings_pack::announce_to_all_trackers, true);
 	pack.set_bool(settings_pack::announce_to_all_tiers, true);
 
-	std::unique_ptr<lt::session> s(new lt::session(pack));
+	auto s = std::make_unique<lt::session>(pack);
 
 	error_code ec;
 	remove_all("tmp1_tracker", ec);
 	create_directory("tmp1_tracker", ec);
-	std::ofstream file(combine_path("tmp1_tracker", "temporary").c_str());
+	ofstream file(combine_path("tmp1_tracker", "temporary").c_str());
 	std::shared_ptr<torrent_info> t = ::create_torrent(&file, "temporary", 16 * 1024, 13, false);
 	file.close();
 
@@ -373,14 +377,14 @@ void test_udp_tracker(std::string const& iface, address tracker, tcp::endpoint c
 	{
 		print_alerts(*s, "s", false, false, std::bind(&connect_alert, _1, std::ref(peer_ep)));
 
-		if (num_udp_announces() == prev_udp_announces + 1)
+		if (num_udp_announces() == prev_udp_announces + 2)
 			break;
 
 		std::this_thread::sleep_for(lt::milliseconds(100));
 	}
 
-	// we should have announced to the tracker by now
-	TEST_EQUAL(num_udp_announces(), prev_udp_announces + 1);
+	// expect two announces, one each for v1 and v2
+	TEST_EQUAL(num_udp_announces(), prev_udp_announces + 2);
 
 	// if we remove the torrent before it has received the response from the
 	// tracker, it won't announce again to stop. So, wait a bit before removing.
@@ -391,7 +395,7 @@ void test_udp_tracker(std::string const& iface, address tracker, tcp::endpoint c
 	for (int i = 0; i < 50; ++i)
 	{
 		print_alerts(*s, "s", true, false, std::bind(&connect_alert, _1, std::ref(peer_ep)));
-		if (num_udp_announces() == prev_udp_announces + 2)
+		if (num_udp_announces() == prev_udp_announces + 4)
 			break;
 
 		std::this_thread::sleep_for(lt::milliseconds(100));
@@ -407,7 +411,7 @@ void test_udp_tracker(std::string const& iface, address tracker, tcp::endpoint c
 	std::printf("done\n");
 
 	// we should have announced the stopped event now
-	TEST_EQUAL(num_udp_announces(), prev_udp_announces + 2);
+	TEST_EQUAL(num_udp_announces(), prev_udp_announces + 4);
 
 	stop_udp_tracker();
 }
@@ -445,12 +449,12 @@ TORRENT_TEST(http_peers)
 	pack.set_int(settings_pack::tracker_receive_timeout, 1);
 	pack.set_str(settings_pack::listen_interfaces, "0.0.0.0:39775");
 
-	std::unique_ptr<lt::session> s(new lt::session(pack));
+	auto s = std::make_unique<lt::session>(pack);
 
 	error_code ec;
 	remove_all("tmp2_tracker", ec);
 	create_directory("tmp2_tracker", ec);
-	std::ofstream file(combine_path("tmp2_tracker", "temporary").c_str());
+	ofstream file(combine_path("tmp2_tracker", "temporary").c_str());
 	std::shared_ptr<torrent_info> t = ::create_torrent(&file, "temporary", 16 * 1024, 13, false);
 	file.close();
 
@@ -518,12 +522,12 @@ TORRENT_TEST(current_tracker)
 	pack.set_int(settings_pack::tracker_receive_timeout, 1);
 	pack.set_str(settings_pack::listen_interfaces, "0.0.0.0:39775");
 
-	std::unique_ptr<lt::session> s(new lt::session(pack));
+	auto s = std::make_unique<lt::session>(pack);
 
 	error_code ec;
 	remove_all("tmp3_tracker", ec);
 	create_directory("tmp3_tracker", ec);
-	std::ofstream file(combine_path("tmp3_tracker", "temporary").c_str());
+	ofstream file(combine_path("tmp3_tracker", "temporary").c_str());
 	std::shared_ptr<torrent_info> t = ::create_torrent(&file, "temporary", 16 * 1024, 13, false);
 	file.close();
 
@@ -578,12 +582,12 @@ void test_proxy(bool proxy_trackers)
 	pack.set_int(settings_pack::proxy_port, 4444);
 	pack.set_bool(settings_pack::proxy_tracker_connections, proxy_trackers);
 
-	std::unique_ptr<lt::session> s(new lt::session(pack));
+	auto s = std::make_unique<lt::session>(pack);
 
 	error_code ec;
 	remove_all("tmp2_tracker", ec);
 	create_directory("tmp2_tracker", ec);
-	std::ofstream file(combine_path("tmp2_tracker", "temporary").c_str());
+	ofstream file(combine_path("tmp2_tracker", "temporary").c_str());
 	std::shared_ptr<torrent_info> t = ::create_torrent(&file, "temporary", 16 * 1024, 13, false);
 	file.close();
 
@@ -634,6 +638,7 @@ TORRENT_TEST(tracker_proxy)
 }
 
 #ifndef TORRENT_DISABLE_LOGGING
+#ifndef TORRENT_DISABLE_ALERT_MSG
 namespace {
 void test_stop_tracker_timeout(int const timeout)
 {
@@ -683,7 +688,7 @@ void test_stop_tracker_timeout(int const timeout)
 	error_code ec;
 	remove_all("tmp4_tracker", ec);
 	create_directory("tmp4_tracker", ec);
-	std::ofstream file(combine_path("tmp4_tracker", "temporary").c_str());
+	ofstream file(combine_path("tmp4_tracker", "temporary").c_str());
 	std::shared_ptr<torrent_info> t = ::create_torrent(&file, "temporary", 16 * 1024, 13, false);
 	file.close();
 
@@ -701,12 +706,14 @@ void test_stop_tracker_timeout(int const timeout)
 	h.add_tracker(ae);
 
 	// make sure it announced a event=started properly
-	wait_for_alert(s, tracker_reply_alert::alert_type, "s");
+	// expect announces for v1 and v2 info hashes
+	for (int i = 0; i < 2; ++i)
+		wait_for_alert(s, tracker_reply_alert::alert_type, "s");
 
 	s.remove_torrent(h);
 
-	int const count = count_stopped_events(s, (timeout == 0) ? 0 : 1);
-	TEST_EQUAL(count, (timeout == 0) ? 0 : 1);
+	int const count = count_stopped_events(s, (timeout == 0) ? 0 : 2);
+	TEST_EQUAL(count, (timeout == 0) ? 0 : 2);
 }
 } // anonymous namespace
 
@@ -722,3 +729,4 @@ TORRENT_TEST(stop_tracker_timeout_zero_timeout)
 	test_stop_tracker_timeout(0);
 }
 #endif
+#endif
