@@ -1,6 +1,7 @@
 /*
 
-Copyright (c) 2008, Arvid Norberg
+Copyright (c) 2010, 2012-2019, Arvid Norberg
+Copyright (c) 2016-2018, Alden Torres
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -32,11 +33,12 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include "libtorrent/session.hpp"
 #include "libtorrent/session_settings.hpp"
+#include "libtorrent/session_params.hpp"
 #include "libtorrent/alert_types.hpp"
 #include "libtorrent/bencode.hpp"
 #include "libtorrent/time.hpp"
 #include "libtorrent/aux_/path.hpp"
-#include "libtorrent/utp_stream.hpp"
+#include "libtorrent/aux_/utp_stream.hpp"
 #include <tuple>
 #include <functional>
 
@@ -45,12 +47,20 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "settings.hpp"
 #include <fstream>
 
+#ifdef TORRENT_UTP_LOG_ENABLE
+#include "libtorrent/utp_stream.hpp"
+#endif
+
 using namespace lt;
 
 namespace {
 
 void test_transfer()
 {
+#ifdef TORRENT_UTP_LOG_ENABLE
+	lt::set_utp_stream_logging(true);
+#endif
+
 	// in case the previous run was terminated
 	error_code ec;
 	remove_all("tmp1_utp", ec);
@@ -147,6 +157,8 @@ TORRENT_TEST(utp)
 
 TORRENT_TEST(compare_less_wrap)
 {
+	using lt::aux::compare_less_wrap;
+
 	TEST_CHECK(compare_less_wrap(1, 2, 0xffff));
 	TEST_CHECK(!compare_less_wrap(2, 1, 0xffff));
 	TEST_CHECK(compare_less_wrap(100, 200, 0xffff));
