@@ -1,6 +1,9 @@
 /*
 
-Copyright (c) 2008-2013, Arvid Norberg
+Copyright (c) 2013-2019, Arvid Norberg
+Copyright (c) 2015, Jakob Petsovits
+Copyright (c) 2016, Andrei Kurushin
+Copyright (c) 2016, Alden Torres
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -31,9 +34,8 @@ POSSIBILITY OF SUCH DAMAGE.
 */
 
 #include "libtorrent/session.hpp"
-#include "libtorrent/file_pool.hpp"
+#include "libtorrent/session_params.hpp"
 #include "libtorrent/aux_/path.hpp"
-#include "libtorrent/storage.hpp"
 #include "libtorrent/bencode.hpp"
 #include "libtorrent/create_torrent.hpp"
 #include "libtorrent/alert_types.hpp"
@@ -42,6 +44,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/string_view.hpp"
 
 #include "test.hpp"
+#include "test_utils.hpp"
 #include "setup_transfer.hpp"
 #include "settings.hpp"
 #include "web_seed_suite.hpp"
@@ -215,24 +218,6 @@ void test_transfer(lt::session& ses, std::shared_ptr<torrent_info> torrent_file
 		// end up using all the cache anyway
 		torrent_status st = th.status();
 		TEST_EQUAL(st.is_seeding, true);
-
-		if (st.is_seeding)
-		{
-			std::int64_t const total_blocks = (torrent_file->total_size() + 0x3fff) / 0x4000;
-			// we need to sleep here a bit to let the session sync with the torrent stats
-			// commented out because it takes such a long time
-			for (int i = 0; i < 50; ++i)
-			{
-				cnt = get_counters(ses);
-				if (std::abs(int(cnt["disk.read_cache_blocks"] - total_blocks)) <= 2 &&
-					std::abs(int(cnt["disk.disk_blocks_in_use"] - total_blocks)) <= 2)
-					break;
-				std::printf("cache_size: %d/%d\n", int(cnt["disk.read_cache_blocks"])
-					, int(cnt["disk.disk_blocks_in_use"]));
-				std::this_thread::sleep_for(lt::milliseconds(100));
-			}
-			TEST_CHECK(std::abs(int(cnt["disk.disk_blocks_in_use"] - total_blocks)) <= 2);
-		}
 	}
 
 	std::cout << "total_size: " << total_size
@@ -399,14 +384,14 @@ int EXPORT run_http_suite(int proxy, char const* protocol, bool test_url_seed
 			pack.set_bool(settings_pack::enable_natpmp, false);
 			pack.set_bool(settings_pack::enable_upnp, false);
 			pack.set_bool(settings_pack::enable_dht, false);
-			lt::session ses(pack, {});
+			lt::session ses(session_params{pack, {}});
 
 			test_transfer(ses, torrent_file, proxy, protocol, test_url_seed
 				, chunked_encoding, test_ban, keepalive, proxy_peers);
 
 			if (test_url_seed && test_rename)
 			{
-				torrent_file->rename_file(file_index_t(0), combine_path(save_path, combine_path("torrent_dir", "renamed_test1")));
+				torrent_file->rename_file(0_file, combine_path(save_path, combine_path("torrent_dir", "renamed_test1")));
 				test_transfer(ses, torrent_file, 0, protocol, test_url_seed
 					, chunked_encoding, test_ban, keepalive, proxy_peers);
 			}
